/* gmcalread.c */

#include <string.h>
#include <gmcal/gmcalbasic.h>
#include <gmcal/gmcalread.h>

static gint gmcal_read_open		(FILE **fp, 
					char *path,
					char *open_type);
static gint gmcal_read_oma4_header	(FILE *fp, int *dimX, gushort *dataType, int *dimY);
static void gmcal_read_oma4_y_data	(FILE *fp, float *Y, int dimX, gushort dataType);

#define gmcal_blocksize 1024

static gint
gmcal_read_open(FILE **fp, char *path, char *open_type)
{
  if (!fp || fp[0]) {
    return GMCAL_READ_WRONG_IMPLEMENTATION;
  }

  if (!path || !strlen(path)) {
    return GMCAL_READ_NO_PATH;
  }

  if (!open_type || !strlen(open_type)) {
    return GMCAL_READ_WRONG_IMPLEMENTATION;
  }

  fp[0]=fopen(path, open_type);
  if (!fp[0]) {
    return GMCAL_READ_COULD_NOT_OPEN;
  }

  return GMCAL_SUCCESS;
}

gint
gmcal_read_line(gchar **line,
		gint *actual_len,
		FILE *fp)
{
  gchar *addon=NULL;
  gint addon_max=0;
  gint len;

  if (!actual_len[0]) {
    actual_len[0]=gmcal_blocksize;
    line[0]=(gchar *) g_malloc(gmcal_blocksize*sizeof(gchar));
  }

  if (!fgets(line[0],actual_len[0],fp)) return -1;
  if (!strlen(line[0])) return 0;

  if (line[0][strlen(line[0])-1]!='\n' && (strlen(line[0])==actual_len[0]-1)) {
    if ((len=gmcal_read_line(&addon, &addon_max, fp))==-1) return -1;
    actual_len[0]+=len;
    line[0]=(gchar *) g_realloc((void *) line[0], actual_len[0]*sizeof(gchar *));
    line[0]=strcat(line[0], addon);
    g_free((void *) addon);
  }

  return strlen(line[0]);
}

gint
gmcal_read_float_bin_vector_simple(gchar *path, 
				gfloat **X,
				gfloat **Y,
				gfloat ***Z,
				gint *dimX,
				gint *dimY,
				gchar **info)
{
  FILE *fp=NULL;
  gint i;
  gint error;
  gfloat *dummy=(gfloat *) g_malloc(gmcal_blocksize*sizeof(gfloat));
  gfloat *values=NULL;
  gint amount;
  gint counter;

  if ((error=gmcal_read_open(&fp, path, "rb"))) {
    return error;
  }

  counter=0;
  dimX[0]=0;
  while((amount=fread((void *)dummy,sizeof(gfloat),gmcal_blocksize,fp))) {
    values=(gfloat *) g_realloc((void *) values, (counter+1)*gmcal_blocksize*sizeof(gfloat));
    for (i=0; i<amount; i++)  values[i+counter*gmcal_blocksize]= dummy[i];
    counter++;
    dimX[0]+=amount;
  }
  values=(gfloat *) g_realloc((void *) values, dimX[0]*sizeof(gfloat));

  dimY[0]=1;

  if ((error=gmcal_basic_matrix_new(X, Y, Z, dimX[0], dimY[0]))) {
    g_free((void *) dummy);
    g_free((void *) values);
    return error;
  }

  gmcal_basic_vector_fill_with_index(X[0], dimX[0]);
  gmcal_basic_vector_fill_with_index(Y[0], dimY[0]);
  
  for (i=0; i<dimX[0]; i++) {
    Z[0][0][i]=values[i];
  }

  *info=g_strdup("");

  g_free((void *) dummy);
  g_free((void *) values);
  fclose(fp);
  
  return GMCAL_SUCCESS;
}

gint
gmcal_read_float_bin_matrix_gmcal(gchar *path, 
				gfloat **X,
				gfloat **Y,
				gfloat ***Z,
				gint *dimX,
				gint *dimY,
				gchar **info)
{
  FILE *fp=NULL;
  gchar *dummy=NULL;
  gint counter;
  gint error;
  gint i;
  gint len=0;

  if ((error=gmcal_read_open(&fp, path, "rb"))) {
    return error;
  }

  len=0;
  gmcal_read_line(&dummy, &len, fp);
  sscanf(dummy,"dimX: %d", dimX);
  gmcal_read_line(&dummy, &len, fp);
  sscanf(dummy,"dimY: %d", dimY);

  len=0;
  gmcal_read_line(info, &len, fp);
  if (info[0][strlen(info[0])-1]=='\n') {
    info[0][strlen(info[0])-1]='\0';
  }
  len=0;
  gmcal_read_line(&dummy, &len, fp);
  g_free((void *) dummy);
  dummy=NULL;
  
  if ((error=gmcal_basic_matrix_new(X, Y, Z, dimX[0], dimY[0]))) {
    g_free((void *) dummy);
    return error;
  }

  counter=fread((void *) X[0], sizeof(gfloat), dimX[0], fp);
  if (counter<dimX[0]) {
    fclose(fp);
    return GMCAL_READ_FILE_TOO_SMALL;
  }

  counter=fread((void *) Y[0], sizeof(gfloat), dimY[0], fp);
  if (counter<dimY[0]) {
    fclose(fp);
    return GMCAL_READ_FILE_TOO_SMALL;
  }

  for (i=0; i<dimY[0]; i++) {
    counter=fread((void *) Z[0][i], sizeof(float), dimX[0], fp);
    if (counter<dimX[0]) {
      fclose(fp);
      return GMCAL_READ_FILE_TOO_SMALL;
    }
  }

  g_free((void *) dummy);
  fclose(fp);
  
  return GMCAL_SUCCESS;
}

gint	
gmcal_read_float_bin_matrix_csma(gchar *path, 
				gfloat **X,
				gfloat **Y,
				gfloat ***Z,
				gint *dimX,
				gint *dimY,
				gchar **info)
{
  FILE *fp=NULL;
  gchar *dummy=(void *) g_malloc(4100*sizeof(char));
  gint counter;
  gint datatype;
  gint i, j;
  gint error;
  gint *id;
  glong *ld;
  guint *uid;


  if ((error=gmcal_read_open(&fp, path, "rb"))) {
    return error;
  }

  if ((counter=fread((void *) dummy,1,4100,fp))<4100) { 
    fclose(fp);
    return GMCAL_READ_FILE_TOO_SMALL;
  }

  dimX[0]=*(guint *) (dummy+42);
  dimY[0]=*(long *) (dummy+34);
  if (dimY[0]==-1 || dimY[0]>32767) {
    dimY[0]=*(long *)(dummy+664);
  }
  if (dimY[0]==-1 || dimY[0]>32767) {
    dimY[0]=*(int *)(dummy+656);
  }
  if (dimY[0]==-1 || dimY[0]>32767) {
    return GMCAL_READ_CORRUPT_HEADER;
  }

  datatype=*(int *) (dummy+108);

  if ((error=gmcal_basic_matrix_new(X, Y, Z, dimX[0], dimY[0]))) {
    g_free((void *) dummy);
    return error;
  }

  gmcal_basic_vector_fill_with_index(X[0], dimX[0]);
  gmcal_basic_vector_fill_with_index(Y[0], dimY[0]);

  switch(datatype) {
    case 0: 	/*  float */
    		for (i=0; i<dimY[0]; i++) {
    		  fread((void *) Z[0][i], dimX[0], sizeof(float), fp);
    		}
    		break;
    case 1: 	/* Long integer */
    		ld=(glong *) g_malloc(dimX[0]*sizeof(glong));
    		for (i=0; i<dimY[0]; i++) {
    		  fread((void *) ld, dimX[0], sizeof(glong), fp);
    		  for (j=0; j<dimX[0]; j++) {
    		    Z[0][i][j]=(gfloat) ld[j];
    		  }
    		}
    		g_free((void *) ld);
    		break;
    case 2: 	/* integer */
    		id=(gint *) g_malloc(dimX[0]*sizeof(gint));
    		for (i=0; i<dimY[0]; i++) {
    		  fread((void *) id, dimX[0], sizeof(gint), fp);
    		  for (j=0; j<dimX[0]; j++) {
    		    Z[0][i][j]=(float) id[j];
    		  }
    		}
    		g_free((void *) id);
    		break;
    case 3: 	/* unsigned integer */
    		printf("Format: unsigned integer\n");
    		uid=(guint *) g_malloc(dimX[0]*sizeof(guint));
    		for (i=0; i<dimY[0]; i++) {
    		  fread((void *) uid, dimX[0], sizeof(guint), fp);
    		  for (j=0; j<dimX[0]; j++) {
    		    Z[0][i][j]=(float) uid[j];
    		  }
    		}
    		g_free((void *) uid);
    		break;
  }
  
  *info=g_strdup("");

  g_free((void *) dummy);
  fclose(fp);
  
  return GMCAL_SUCCESS;
}

static int
gmcal_read_oma4_header(FILE *fp, gint *dimX, gushort *dataType, gint *dimY)
{
  gushort dataStart;
  gushort dim;

  fseek(fp,  41, SEEK_SET);
  fread(&dataStart, 1, sizeof(gushort), fp);
  fseek(fp, 125, SEEK_SET);
  fread(&dim, 1, sizeof(gushort), fp); *dimY=(gint) dim;
  fseek(fp, dataStart, SEEK_SET);
  fread(&dim, 1, sizeof(gushort), fp); *dimX=(gint) dim;
  fseek(fp, dataStart+8, SEEK_SET);
  fread(dataType, 1, sizeof(gushort), fp);
  fseek(fp, dataStart+48, SEEK_SET);
  
  switch(*dataType) {
    case 18: printf("OMA4 Format: 2 byte signed integer\n"); break;
    case 20: printf("OMA4 Format: 4 byte signed integer\n"); break;
    case 52: printf("OMA4 Format: 4 byte IEEE standard floating point\n"); break;
    case 56: printf("OMA4 Format: 8 byte IEEE standard floating point\n"); break;
    default: printf("Unknown or unimplemented format\n");
  }


  switch(*dataType) {
    case 1: /* 1 byte unsigned integer */
    case 2: /* 2 byte unsigned integer */
    case 4: /* 4 byte unsigned integer */
    	return GMCAL_READ_NOT_IMPLEMENTED;
    case 18: /* 2 byte signed integer */
    case 20: /* 4 byte signed integer */
    case 52: /* 4 byte IEEE standard floating point (SINGLE) */
    case 56: /* 8 byte IEEE standard floating point (DOUBLE) */
    	return GMCAL_SUCCESS;
    default:
    	return GMCAL_READ_CORRUPT_HEADER;
  } 
}

static void
gmcal_read_oma4_y_data(FILE *fp, gfloat *Y, gint dimX, gushort dataType)
{
  int i;
  
  switch(dataType) {
    case 18: /* 2 byte signed integer */
      { 
        gshort *Y2=(gshort *) g_malloc(dimX*sizeof(gshort));

        fread(Y2, dimX, sizeof(gshort), fp);
        for (i=0; i<dimX; i++) {
          Y[i]=(gfloat) Y2[i];
        }
      }
      return;
    case 20: /* 4 byte signed integer */
      { 
        gint *Y2=(gint *) g_malloc(dimX*sizeof(gint));

        fread(Y2, dimX, sizeof(gint), fp);
        for (i=0; i<dimX; i++) {
          Y[i]=(gfloat) Y2[i];
        }
      }
      return;
    case 52: /* 4 byte IEEE standard floating point (SINGLE) */
      fread(Y, dimX, sizeof(float), fp);
      return;
    case 56: /* 8 byte IEEE standard floating point (DOUBLE) */
      { 
        gdouble *Y2=(gdouble *) g_malloc(dimX*sizeof(gdouble));

        fread(Y2, dimX, sizeof(gdouble), fp);
        for (i=0; i<dimX; i++) {
          Y[i]=(gfloat) Y2[i];
        }
      }
      return;
  }
}

gint
gmcal_read_float_bin_matrix_oma4(gchar *path, 
				gfloat **X,
				gfloat **Y,
				gfloat ***Z,
				gint *dimX,
				gint *dimY,
				gchar **info)
{
  FILE *fp=NULL;
  gint error;
  gushort datatype;
  long seeker;
  gint i;

  if ((error=gmcal_read_open(&fp, path, "rb"))) {
    return error;
  }

  if ((error=gmcal_read_oma4_header(fp, &dimX[0], &datatype, &dimY[0]))
  	!=GMCAL_SUCCESS) {
    fclose(fp);
    return error;
  }

  if ((error=gmcal_basic_matrix_new(X, Y, Z, dimX[0], dimY[0]))) {
    return error;
  }

  gmcal_basic_vector_fill_with_index(Y[0], dimY[0]);

  
  if (dimY[0]) {
    gmcal_read_oma4_y_data(fp, Z[0][0], dimX[0], datatype);
    fread((void *) X[0], dimX[0], sizeof(gfloat), fp);
    fseek(fp, 48, SEEK_CUR);
  }
  
  seeker=48+dimX[0]*sizeof(float);
  
  for (i=1; i<dimY[0]; i++) {
    gmcal_read_oma4_y_data(fp, Z[0][i], dimX[0], datatype);
    fseek(fp, seeker, SEEK_CUR);
  }
     
  *info=g_strdup("");

  fclose(fp);
  
  return GMCAL_SUCCESS;
}

gint
gmcal_read_float_bmp_matrix_pco(gchar *path, 
				gfloat **X,
				gfloat **Y,
				gfloat ***Z,
				gint *dimX,
				gint *dimY,
				gchar **info)
{
  FILE *fp=NULL;
  guchar *value;
  gint error;
  gint i, j;
/* bmp variables... */
  char c;
  guint fileSize;
  gushort reserved;
  guint offset;
  guint headerSize;
  int width;
  int height;
  gushort planes;
  gushort bits;
  guint code;
  guint imageSize;
  gint xPixelPerMeter;
  gint yPixelPerMeter;
  guint colorsUsed;
  guint colorsImportant;

  if ((error=gmcal_read_open(&fp, path, "rb"))) {
    return error;
  }

  fread((void *) &c,sizeof(char),1,fp);
  fread((void *) &c,sizeof(char),1,fp);
  fread((void *) &fileSize,sizeof(guint),1,fp);
  fread((void *) &reserved,sizeof(gushort),1,fp);
  fread((void *) &reserved,sizeof(gushort),1,fp);
  fread((void *) &offset,sizeof(guint),1,fp);
  fread((void *) &headerSize,sizeof(guint),1,fp);
  fread((void *) &width,sizeof(gint),1,fp);
  fread((void *) &height,sizeof(gint),1,fp);
  fread((void *) &planes,sizeof(gushort),1,fp);
  fread((void *) &bits,sizeof(gushort),1,fp);
  fread((void *) &code,sizeof(guint),1,fp);
  fread((void *) &imageSize,sizeof(guint),1,fp);
  fread((void *) &xPixelPerMeter,sizeof(guint),1,fp);
  fread((void *) &yPixelPerMeter,sizeof(guint),1,fp);
  fread((void *) &colorsUsed,sizeof(guint),1,fp);
  fread((void *) &colorsImportant,sizeof(guint),1,fp);

  dimX[0]=width;
  dimY[0]=height;

  if ((error=gmcal_basic_matrix_new(X, Y, Z, dimX[0], dimY[0]))) {
    return error;
  }

  gmcal_basic_vector_fill_with_index(X[0], dimX[0]);
  gmcal_basic_vector_fill_with_index(Y[0], dimY[0]);

  value=(guchar *) g_malloc(3*dimX[0]*sizeof(guchar));

  for (i=0; i<dimY[0]; i++) {
    fread((void *) value, sizeof(guchar), 3*width, fp);
    for (j=0; j<width; j++) {
      Z[0][dimY[0]-i-1][j]=value[j*3];
    }
  }
  
  g_free((void *) value);

  *info=g_strdup("");

  fclose(fp);
  
  return GMCAL_SUCCESS;
}

gint
gmcal_read_float_ascii_matrix_gmcal(gchar *path, 
				gfloat **X,
				gfloat **Y,
				gfloat ***Z,
				gint *dimX,
				gint *dimY,
				gchar **info)
{
  FILE *fp=NULL;
  gchar *dummy=NULL;
  gint error;
  char cc[2];
  gint i, j;
  gint len=0;

  if ((error=gmcal_read_open(&fp, path, "rb"))) {
    return error;
  }

  len=0;
  gmcal_read_line(&dummy, &len, fp);
  sscanf(dummy,"dimX: %d", dimX);
  gmcal_read_line(&dummy, &len, fp);
  sscanf(dummy,"dimY: %d", dimY);

  len=0;
  gmcal_read_line(info, &len, fp);
  if (info[0][strlen(info[0])-1]=='\n') {
    info[0][strlen(info[0])-1]='\0';
  }
  len=strlen(dummy);
  gmcal_read_line(&dummy, &len, fp);
  g_free((void *) dummy);
  dummy=NULL;
  
  if ((error=gmcal_basic_matrix_new(X, Y, Z, dimX[0], dimY[0]))) {
    return error;
  }

  fgets(cc,2,fp);
  for (i=0; i<dimY[0]-1; i++) {
    fscanf(fp,"%g",&(Y[0][i]));
  }

  fscanf(fp,"%g\n",&(Y[0][dimY[0]-1]));
  for (i=0; i<dimX[0]; i++) {
    fscanf(fp,"%g",&(X[0][i]));
    for (j=0; j<dimY[0]-1; j++) {
      fscanf(fp,"%g",&(Z[0][j][i]));
    }
    if (fscanf(fp,"%g\n",&(Z[0][dimY[0]-1][i]))==EOF) {
      fclose(fp);
      return GMCAL_READ_FILE_TOO_SMALL;
    }
  }

  g_free((void *) dummy);
  fclose(fp);
  
  return GMCAL_SUCCESS;
}
