/* gmcalspline.c */

#include <gmcal/gmcalspline.h>

gint gmcal_spline_get_coeff		(gfloat *x,
					gfloat *y,
					gint n,
					gdouble **coeff)
{
  gint i;
  gdouble p,qn,sig,un;
  gdouble *u;
  
  if (n<3) {
    return GMCAL_SPLINE_NOT_ENOUGH_VALUES;
  }
  
  for (i=1; i<n; i++) {
    if (x[i]<x[i-1]) return GMCAL_SPLINE_X_VALUES_UNSORTED;
    if (x[i]==x[i-1]) return GMCAL_SPLINE_X_VALUES_NOT_DISTINCT;
  }
  
  if (!coeff) {
   return GMCAL_SPLINE_COEFF_IS_NULL;
  }

  if (coeff[0]) {
    return GMCAL_SPLINE_COEFF_0_NOT_NULL;
  }
  
  u=(double *) g_malloc0((n-1)*sizeof(double));
  
  coeff[0]=(double *) g_malloc(n*sizeof(gdouble));

  coeff[0][0]=u[0]=0.0;

  for (i=1;i<n-1;i++) {
    sig=(x[i]-x[i-1])/(x[i+1]-x[i-1]);
    p=sig*coeff[0][i-1]+2.0;
    coeff[0][i]=(sig-1.0)/p;
    u[i]=(y[i+1]-y[i])/(x[i+1]-x[i]) - (y[i]-y[i-1])/(x[i]-x[i-1]);
    u[i]=(6.0*u[i]/(x[i+1]-x[i-1])-sig*u[i-1])/p;
  }
  qn=un=0.0;

  coeff[0][n-1]=(un-qn*u[n-2])/(qn*coeff[0][n-2]+1.0);
  for (i=n-2; i>=0; i--)
    coeff[0][i]=coeff[0][i]*coeff[0][i+1]+u[i];
  g_free((void *) u);

  return GMCAL_SUCCESS;
}

gint gmcal_spline_get_poly_coeff	(gfloat *x, 
					gfloat *y,
					gint n,
					gdouble *coeff,
					gdouble ***poly)
{
  gint i;
  gint j;
  gdouble h;
  
  if (n<3) {
    return GMCAL_SPLINE_NOT_ENOUGH_VALUES;
  }
  
  for (i=1; i<n; i++) {
    if (x[i]<x[i-1]) return GMCAL_SPLINE_X_VALUES_UNSORTED;
    if (x[i]==x[i-1]) return GMCAL_SPLINE_X_VALUES_NOT_DISTINCT;
  }

  if (!coeff) {
   return GMCAL_SPLINE_COEFF_IS_NULL;
  }

  if (!poly) {
   return GMCAL_SPLINE_POLY_IS_NULL;
  }

  if (poly[0]) {
    return GMCAL_SPLINE_POLY_0_NOT_NULL;
  }

  poly[0]=(double **) g_malloc(n*sizeof(gdouble *));
  for (i=0; i<n; i++) {
    poly[0][i]=(double *) g_malloc(4*sizeof(double));
  }
  
  for (i=0; i<n-1; i++) {
    j=i+1;
    h=x[j]-x[i];
    poly[0][i][0]=(+coeff[i]/6.*x[j]*x[j]*x[j]-coeff[j]/6.*x[i]*x[i]*x[i])/h
        	-(y[j]/h-coeff[j]*h/6.)*x[i]+(y[i]/h-coeff[i]*h/6.)*x[j];
    poly[0][i][1]=(-coeff[i]/2.*x[j]*x[j]+coeff[j]/2.*x[i]*x[i])/h
        	+y[j]/h-coeff[j]*h/6.-y[i]/h+coeff[i]*h/6.;
    poly[0][i][2]=(+coeff[i]/2.*x[j]-coeff[j]/2.*x[i])/h;
    poly[0][i][3]=(-coeff[i]/6.+coeff[j]/6.)/h;
  }

  return GMCAL_SUCCESS;
}


gint gmcal_spline_get_interval		(gfloat *x, 
					gint n,
					gfloat x_value)
{
  gint kLow,kHigh,k;

  kLow=0;
  kHigh=n-1;
  while (kHigh-kLow > 1) {
    k=(kHigh+kLow) >> 1;
    if (x[k]>x_value) kHigh=k;
    else kLow=k;
  }

  return kLow;
}


gdouble	gmcal_spline_get_value		(gfloat *x,
					gfloat *y,
					gint n,
					gdouble *coeff,
					gfloat x_value,
					gint interval)
{
  gint kLow,kHigh;
  gdouble h,b,a;

  kLow=interval;
  kHigh=interval+1;


  h=x[kHigh]-x[kLow];

  a=(x[kHigh]-x_value)/h;
  b=(x_value-x[kLow])/h;

  a= (a*y[kLow]+b*y[kHigh]+((a*a*a-a)*coeff[kLow]+(b*b*b-b)*coeff[kHigh])*(h*h)/6.0);

  return a;
}

