/* gmcalwrite.c */

#include <string.h>
#include <math.h>
#include <gmcal/gmcalbasic.h>
#include <gmcal/gmcalwrite.h>

static gint gmcal_write_open		(FILE **fp, 
					char *path,
					char *open_type);
static gint
gmcal_write_open(FILE **fp, char *path, char *open_type)
{
  if (!fp || fp[0]) {
    return GMCAL_WRITE_WRONG_IMPLEMENTATION;
  }

  if (!path || !strlen(path)) {
    return GMCAL_WRITE_NO_PATH;
  }

  if (!open_type || !strlen(open_type)) {
    return GMCAL_WRITE_WRONG_IMPLEMENTATION;
  }

  fp[0]=fopen(path, open_type);
  if (!fp[0]) {
    return GMCAL_WRITE_COULD_NOT_OPEN;
  }

  return GMCAL_SUCCESS;
}

gint
gmcal_write_float_bin_vector_simple(gchar *path, 
				gfloat *Z,
				gint dimX)
{
  FILE *fp=NULL;
  gint error;

  if ((error=gmcal_basic_vector_check_integrity(Z, dimX))) {
    return error;
  }

  if ((error=gmcal_write_open(&fp, path, "wb"))) {
    return error;
  }

  fwrite((void *) Z, sizeof(gfloat), dimX, fp);

  fclose(fp);
  
  return GMCAL_SUCCESS;
}

gint
gmcal_write_float_bin_matrix_gmcal(gchar *path, 
				gfloat *X,
				gfloat *Y,
				gfloat **Z,
				gint dimX,
				gint dimY,
				gchar *info)
{
  FILE *fp=NULL;
  gint error;
  gint i;

  if ((error=gmcal_basic_matrix_check_integrity(X, Y, Z, dimX, dimY))) {
    return error;
  }

  if ((error=gmcal_write_open(&fp, path, "wb"))) {
    return error;
  }

  fprintf(fp, "dimX: %d\ndimY: %d\n%s\n%s\n", dimX, dimY, info,
     "PC Float Values, dimX values \"x dispersion\", dimY values \"y dispersion\" then dimY times dimX values");

  fwrite((void *) X, sizeof(gfloat), dimX, fp); 
  fwrite((void *) Y, sizeof(gfloat), dimY, fp); 

  for (i=0; i<dimY; i++) {
    fwrite((void *) Z[i], sizeof(gfloat), dimX, fp);
  }

  fclose(fp);
  
  return GMCAL_SUCCESS;
}

gint	
gmcal_write_float_ascii_matrix_gmcal(gchar *path, 
				gfloat *X,
				gfloat *Y,
				gfloat **Z,
				gint dimX,
				gint dimY,
				gchar *info)
{
  FILE *fp=NULL;
  gint error;
  gint i, j;

  if ((error=gmcal_basic_matrix_check_integrity(X, Y, Z, dimX, dimY))) {
    return error;
  }

  if ((error=gmcal_write_open(&fp, path, "w"))) {
    return error;
  }

  fprintf(fp, "dimX: %d\ndimY: %d\n%s\n%s\n", dimX, dimY, info,
     "First row: \"y dispersion\", first column: \"x dispersion\"");

  fprintf(fp, "%%\t");
  for (j=0; j<dimY-1; j++) {
    fprintf(fp, "%g\t", Y[j]);
  }
  fprintf(fp,"%g\n", Y[dimY-1]);

  for (i=0; i<dimX; i++) {
    fprintf(fp,  "%g\t", X[i]);
    for (j=0; j<dimY-1; j++) {
      fprintf(fp, "%.20g\t", Z[j][i]);
    }
    fprintf(fp, "%g\n", Z[dimY-1][i]);
  }

  fclose(fp);
  
  return GMCAL_SUCCESS;
}

#include <gd/gd.h>
#include <gd/gdfonts.h>

static void GifExportSetDataColors(gdImagePtr im, int maxColors, int *colors, int *singleColor, char *model);

static void
GifExportSetDataColors(gdImagePtr im, int maxColors, int *colors, int *singleColor, char *model)
{
    int red=0;
    int green=0;
    int blue=0;
    int r_col;
    int g_col;
    int b_col;
    int i=0;
    int rainbow=0;
    float col_factor=0;
    
    if (!strncmp(model,"rainbow",7)) {
      rainbow=1;
    } else if (!strncmp(model,"red",3)) {
      red=1; green=0; blue=0;
    } else if (!strncmp(model,"green",5)) {
      red=0; green=1; blue=0;
    } else if (!strncmp(model,"blue",4)) {
      red=0; green=0; blue=1;
    } else if (!strncmp(model,"cyan",4)) {
      red=0; green=1; blue=1;
    } else if (!strncmp(model,"yellow",6)) {
      red=1; green=1; blue=0;
    } else if (!strncmp(model,"magenta",7)) {
      red=1; green=0; blue=1;
    } else {
      red=1; green=1; blue=1;
    }
    *singleColor=gdImageColorAllocate(im,red/196*255,
                                	green/196*255,
                                        blue/196*255); 


    for (i=0; i<maxColors; i++) {
      int offset;
      offset=59;
      if (rainbow){
        r_col=offset+196*sin(PI/2.*i/(maxColors-1));
        g_col=offset+196*sin(PI/2.*i/((maxColors-1)/2+1));
        b_col=offset+196*cos(PI/2.*i/((maxColors-1)/2+1));
        if (b_col<offset) b_col=offset;
      } else {
	r_col=offset+196*sin(PI/2./(maxColors)*red*i);
	g_col=offset+green*col_factor*i;
	b_col=offset+blue*col_factor*i;
      }
      colors[i]=gdImageColorAllocate(im,r_col, g_col, b_col);
/*      printf("red=%d, green=%d, blue=%d, offset=%d\n", r_col, g_col, b_col, offset);*/
    }
}

gint	gmcal_write_float_gif_matrices_with_info(gchar *path, 
						gfloat **X,
						gfloat **Y,
						gfloat ***Z,
						gint *dimX,
						gint *dimY,
						gchar **info,
						gint num,
						gchar *model,
						gint maxColors,
						gint mode,
						gint imagesPerRow)
{
  gdImagePtr im;
  FILE *fp=NULL;
  gint black;
  gint white;
  gint singleColor;
  gint width;
  gint height;
  gfloat minZ[num];
  gfloat maxZ[num];
  gint sublen;
  gint legend_len;
  gint i, j, k;
  gint error;
  gfloat color_factor;
  gint color;
  gint *colors;
  gchar min_string[30];
  gchar mid_string[30];
  gchar max_string[30];
  gint row;
  gint col;

  imagesPerRow=MIN(MAX(1,imagesPerRow), num);

  maxColors=MIN(254,MAX(maxColors, 2));
  
  printf("maxColors=%d\n", maxColors);
  
  for (i=0; i<num; i++) {
    if ((error=gmcal_basic_matrix_check_integrity(X[i], Y[i], Z[i], dimX[i], dimY[i]))) {
      return error;
    }
  }

  if ((error=gmcal_write_open(&fp, path, "wb"))) {
    return error;
  }

  colors=(gint *) g_malloc(maxColors*sizeof(gint));
  
  width=0;
  height=0;
  sublen=0;
  legend_len=0;
  for (i=0; i<num; i++) {
    width=MAX(dimX[i], width);
    height=MAX(dimY[i], height);
    if (info[i]) {
      sublen=MAX(sublen, strlen(info[i]));
    } else {
      sublen=MAX(sublen, strlen("(No info string)"));
    }
    minZ[i]=Z[i][0][0];
    maxZ[i]=Z[i][0][0];
  
    for (j=0; j<dimY[i]; j++) 
      for (k=0; k<dimX[i]; k++) {
        minZ[i]=MIN(minZ[i], Z[i][j][k]);
        maxZ[i]=MAX(maxZ[i], Z[i][j][k]);
      }
 
    if (maxZ[i]==minZ[i]) maxZ[i]+=0.000001;
  
    sprintf(min_string, "%g", minZ[i]);
    sprintf(mid_string, "%g", (maxZ[i]+minZ[i])/2.);
    sprintf(max_string, "%g", maxZ[i]);
    legend_len=MAX(legend_len, MAX(strlen(mid_string), MAX(strlen(min_string), strlen(max_string)))*gdFontSmall->w);
  }
  legend_len+=35;
  
  width=MAX(width+legend_len, sublen)+4;
  height=height+2*(gdFontSmall->h+4)+4;

  im = gdImageCreate(width*imagesPerRow+1, height*((num+imagesPerRow-1)/imagesPerRow)+1);
  black = gdImageColorAllocate(im, 0, 0, 0);      
  white = gdImageColorAllocate(im, 255, 255, 255);        

  GifExportSetDataColors(im, maxColors, colors, &singleColor, model);

  gdImageFilledRectangle(im, 0, 0, width*imagesPerRow+1, height*(num+imagesPerRow-1)/imagesPerRow+1, white);
  
  for (i=0; i<num; i++) {
    row=i/imagesPerRow;
    col=i%imagesPerRow;
    gdImageRectangle(im, col*width, row*height, col*width+width, row*height+height, black);
    if (mode==1) {
      gdImageString(im, gdFontSmall, col*width+5, row*height+gdFontSmall->h, "Not implemented yet", black);
    } else {
      color_factor=(maxColors-1)/(maxZ[i]-minZ[i]);
      for (j=0; j<dimY[i]; j++) {
        for (k=0; k<dimX[i]; k++) {
          if (Z[i][j][k]<maxZ[i]) {
            color=(Z[i][j][k]-minZ[i])*color_factor;
            gdImageSetPixel(im, col*width+2+k, row*height+2+j, colors[color]);
          } else {
            gdImageSetPixel(im, col*width+2+k, row*height+2+j, white);
          }
        }
      }
      for (j=0; j<maxColors; j++) {
        gdImageFilledRectangle(im, col*width+2+dimX[i]+4, row*height+2+(gint)((gfloat)(dimY[i]-1)/maxColors*j), 
        			   col*width+2+dimX[i]+25, row*height+2+(gint)((gfloat)(dimY[i]-1)/maxColors*(j+1)), 
        			   colors[maxColors-j-1]);
      }

      sprintf(min_string, "%g", minZ[i]);
      sprintf(mid_string, "%g", (maxZ[i]+minZ[i])/2.);
      sprintf(max_string, "%g", maxZ[i]);
      gdImageString(im, gdFontSmall, col*width+2+dimX[i]+30, row*height+2+0, max_string, black);
      gdImageString(im, gdFontSmall, col*width+2+dimX[i]+30, row*height+2+dimY[i]-gdFontSmall->h, min_string, black);
      gdImageString(im, gdFontSmall, col*width+2+dimX[i]+30, row*height+2+dimY[i]/2-gdFontSmall->h+2, mid_string, black);
      sprintf(min_string, "%g", X[i][0]);
      sprintf(mid_string, "%g", (X[i][0]+X[i][dimX[i]-1])/2.);
      sprintf(max_string, "%g", X[i][dimX[i]-1]);
      gdImageString(im, gdFontSmall, col*width+2, row*height+height-2*gdFontSmall->h-9, min_string, black);
      gdImageString(im, gdFontSmall, col*width+2+(dimX[i]-strlen(mid_string)*gdFontSmall->w)/2, row*height+height-2*gdFontSmall->h-9, mid_string, black);
      gdImageString(im, gdFontSmall, col*width+2+dimX[i]-strlen(max_string)*gdFontSmall->w, row*height+height-2*gdFontSmall->h-9, max_string, black);
    }
    if (info[i] && strlen(info[i])) {
      gdImageString(im, gdFontSmall, col*width+2, row*height+height-gdFontSmall->h-3, info[i], black);
    } else {
      gdImageString(im, gdFontSmall, col*width+2, row*height+height-gdFontSmall->h-3, "No info ...", black);
    }
  }

  gdImageGif(im, fp);    
  fclose(fp);

  g_free((void *) colors);

  gdImageDestroy(im);

  return GMCAL_SUCCESS;
}
