/* gtkdatabox.c */

#include <string.h>
#include "gtk/gtksignal.h"
#include "gtk/gtktable.h"
#include "gtk/gtktogglebutton.h"
#include "gtk/gtkdrawingarea.h"
#include "gtk/gtkhscrollbar.h"
#include "gtk/gtkvscrollbar.h"
#include "gtk/gtkdatabox.h"

enum {
  GTK_DATABOX_ZOOM_SIGNAL,
  LAST_SIGNAL
};

static void gtk_databox_class_init          	(GtkDataboxClass *klass);
static void gtk_databox_init                	(GtkDatabox *box);
static gint gtk_databox_expose_callback		(GtkWidget *widget, GdkEventExpose *event, GtkDatabox *box);
static gint gtk_databox_configure_callback	(GtkWidget *widget, GdkEventConfigure *event, GtkDatabox *box);
static void gtk_databox_zoom_home		(GtkWidget *widget, GtkDatabox *box);
static void gtk_databox_zoomed			(GtkWidget *widget, GtkDatabox *box);
static void gtk_databox_x_adjustment_callback	(GtkWidget *widget, GtkDatabox *box);
static void gtk_databox_y_adjustment_callback	(GtkWidget *widget, GtkDatabox *box);
static gint gtk_databox_button_press_callback	(GtkWidget *widget, GdkEventButton *event, GtkDatabox *box);
static gint gtk_databox_motion_notify_callback	(GtkWidget *widget, GdkEventMotion *event, GtkDatabox *box);
static void gtk_databox_draw_request		(GtkWidget *widget, gboolean now, GdkRectangle redraw_rect, GtkDatabox *box);
static void gtk_databox_draw_request_full	(GtkWidget *widget, gboolean now, GtkDatabox *box);
static gint gtk_databox_draw_selection		(GtkWidget *widget, GtkDatabox *box, GdkEventExpose *event);
static void gtk_databox_draw            	(GtkWidget *widget, GtkDatabox *box, GdkEventExpose *event);
static void gtk_databox_new_data_gc		(GtkWidget *widget, GtkDatabox *box, gint index);
static void gtk_databox_draw_cross 		(GtkWidget *widget, GtkDatabox *box, gfloat xfactor, gfloat yfactor);

#define DATABOX_MAX_COLORS 7
static GdkColor InitColors[DATABOX_MAX_COLORS] = { 
     {0, 0, 0, 0},
     {0, 65535, 0, 0},
     {0, 0, 65535, 0},
     {0, 0, 0, 65535},
     {0, 65535, 0, 65535},
     {0, 0, 65535, 65535},
     {0, 65535, 65535, 0}
};


/* ------------------------------------------------------------
                   Gtk Databox Initialization
   ------------------------------------------------------------ */

static gint gtk_databox_signals[LAST_SIGNAL] = { 0 };

guint
gtk_databox_get_type ()
{
  static guint box_type = 0;

  if (!box_type)
    {
      GtkTypeInfo box_info =
      {
        "GtkDatabox",
        sizeof (GtkDatabox),
        sizeof (GtkDataboxClass),
        (GtkClassInitFunc) gtk_databox_class_init,
        (GtkObjectInitFunc) gtk_databox_init,
        (GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL
      };

      box_type = gtk_type_unique (gtk_vbox_get_type (), &box_info);
    }

  return box_type;
}

static void
gtk_databox_class_init (GtkDataboxClass *class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) class;

  gtk_databox_signals[GTK_DATABOX_ZOOM_SIGNAL] = gtk_signal_new ("gtk_databox_zoom",
                                         GTK_RUN_FIRST,
                                         object_class->type,
                                         GTK_SIGNAL_OFFSET (GtkDataboxClass, gtk_databox),
                                         gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);


  gtk_object_class_add_signals (object_class, gtk_databox_signals, LAST_SIGNAL);

  class->gtk_databox = NULL;
}

static void
gtk_databox_init (GtkDatabox *box)
{
  GtkWidget *table;
  GtkWidget *widget;
  GtkWidget *hscroll;
  GtkWidget *vscroll;
  gint i;

  table = gtk_table_new (2, 2, FALSE);
  gtk_container_add (GTK_CONTAINER(box), table);
  gtk_widget_show (table);

  widget=box->draw=gtk_drawing_area_new();
  gtk_widget_set_events(widget, GDK_BUTTON_PRESS_MASK
                                 | GDK_POINTER_MOTION_MASK);
  gtk_signal_connect(GTK_OBJECT (widget), "configure_event",
                      GTK_SIGNAL_FUNC (gtk_databox_configure_callback),
                      box);
  gtk_signal_connect(GTK_OBJECT (widget), "expose_event",
                      GTK_SIGNAL_FUNC (gtk_databox_expose_callback),
                      box);
  gtk_signal_connect(GTK_OBJECT (widget), "button_press_event",
                      GTK_SIGNAL_FUNC (gtk_databox_button_press_callback),
                      box);
  gtk_signal_connect(GTK_OBJECT (widget), "motion_notify_event",
                      GTK_SIGNAL_FUNC (gtk_databox_motion_notify_callback),
                      box);
  gtk_widget_set_usize(widget, 20,30);
  gtk_table_attach(GTK_TABLE(table), widget,
                        1,2,
                        1,2,
                        GTK_FILL|GTK_EXPAND|GTK_SHRINK,
                        GTK_FILL|GTK_EXPAND|GTK_SHRINK,
                        0,0);
  gtk_widget_show(widget);
  box->adjX = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 1.0, 0.1, 0.9, 1.0));
  box->adjY = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 1.0, 0.1, 0.9, 1.0));
  gtk_signal_connect(GTK_OBJECT(box->adjY), "value_changed",
                          GTK_SIGNAL_FUNC (gtk_databox_y_adjustment_callback),
                          box);
  gtk_signal_connect(GTK_OBJECT(box->adjX), "value_changed",
                          GTK_SIGNAL_FUNC (gtk_databox_x_adjustment_callback),
                          box);
  hscroll = gtk_hscrollbar_new(box->adjX);
  vscroll = gtk_vscrollbar_new(box->adjY);
  gtk_table_attach(GTK_TABLE(table), hscroll,
                        1,2,
                        2,3,
                        GTK_FILL|GTK_EXPAND|GTK_SHRINK,
                        0,
                        0,0);
  gtk_table_attach(GTK_TABLE(table), vscroll,
                        2,3,
                        1,2,
                        0,
                        GTK_FILL|GTK_EXPAND|GTK_SHRINK,
                        0,0);

  gtk_widget_show(hscroll);
  gtk_widget_show(vscroll);

  box->pixmap=NULL;
  box->select_pixmap=NULL;
  box->data.dimX=0;
  box->data.dimY=0;
  box->data.X=NULL;
  box->data.Y=NULL;
  box->data.Z=NULL;
  box->data.XY_Z=NULL;
  box->n_color=DATABOX_MAX_COLORS;
  box->data_color=(GdkColor *) g_malloc(DATABOX_MAX_COLORS*sizeof(GdkColor));
  for (i=0; i<box->n_color; i++) {
    box->data_color[i]=InitColors[i];
  }
  box->n_gc=0;
  box->data_gc=NULL;
  box->select_gc=NULL;
  box->cross_border=10;
  box->flags=0;
  gtk_databox_new_data(box, GTK_DATABOX_NO_DATA);
}

GtkWidget*
gtk_databox_new ()
{
  return GTK_WIDGET ( gtk_type_new (gtk_databox_get_type ()));
}

/* ------------------------------------------------------------
                   Internal Callback Handling
   ------------------------------------------------------------ */

static gint
gtk_databox_configure_callback(GtkWidget *widget, GdkEventConfigure *event,
                    GtkDatabox *box)
{
  gfloat xoff, yoff;
  gfloat xpart, ypart;

  xoff=box->adjX->value/box->adjX->upper;
  yoff=box->adjY->value/box->adjY->upper;
  xpart=box->adjX->page_size/box->adjX->upper;
  ypart=box->adjY->page_size/box->adjY->upper;

  gdk_window_get_size (widget->window, &(box->size.x), &(box->size.y));

  box->adjX->upper=(gfloat)((gint) (box->size.x/xpart));
  box->adjY->upper=(gfloat)((gint) (box->size.y/xpart));
  box->adjX->value=(gfloat)((gint) (xoff*box->adjX->upper));
  box->adjY->value=(gfloat)((gint) (yoff*box->adjY->upper));
  box->adjX->page_size=box->size.x;
  box->adjY->page_size=box->size.y;

  gtk_adjustment_changed(box->adjX);
  gtk_adjustment_changed(box->adjY);

  if (box->pixmap)
    gdk_pixmap_unref(box->pixmap);

  box->pixmap = gdk_pixmap_new(widget->window,
                          box->size.x,
                          box->size.y,
                          -1);

  gdk_draw_rectangle (box->pixmap,
                      widget->style->bg_gc[0],
                      TRUE,
                      0, 0,
                      box->size.x,
                      box->size.y);

  box->select_pixmap = gdk_pixmap_new(widget->window,
                          box->size.x,
                          box->size.y,
                          -1);

  gdk_draw_rectangle (box->select_pixmap,
                      widget->style->black_gc,
                      TRUE,
                      0, 0,
                      box->size.x,
                      box->size.y);

  return TRUE;
}

static gint
gtk_databox_expose_callback(GtkWidget *widget, GdkEventExpose *event,
                    GtkDatabox *box)
{
  gtk_databox_draw(box->draw, box, event);

  gdk_draw_pixmap(widget->window,
      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
      box->pixmap,
      event->area.x, event->area.y,
      event->area.x, event->area.y,
      event->area.width, event->area.height);

  if (box->selection_flag) {
    gtk_databox_draw_selection(box->draw, box, event);
  }
  
  return TRUE;
}

static void
gtk_databox_x_adjustment_callback(GtkWidget *widget, GtkDatabox *box)
{
   gtk_databox_draw_request_full(box->draw, TRUE, box);
}

static void
gtk_databox_y_adjustment_callback(GtkWidget *widget, GtkDatabox *box)
{
   gtk_databox_draw_request_full(box->draw, TRUE, box);

}

static gint
gtk_databox_button_press_callback(GtkWidget *widget, GdkEventButton *event, GtkDatabox *box)
{
  gint x;
  gint y;
  guint button;

  if (event->type!=GDK_BUTTON_PRESS)
    return FALSE;

  button=event->button;
  x=event->x;
  y=event->y;

  if (button==1) {
    if (box->selection_flag) {
      box->selection_flag=0;
      gtk_databox_draw_selection(widget, box, NULL);
      if (MIN(box->marked.x, box->select.x)<x && x<MAX(box->marked.x, box->select.x)
       && MIN(box->marked.y, box->select.y)<y && y<MAX(box->marked.y, box->select.y))
      {
        box->adjX->lower=0;
        box->adjY->lower=0;
        box->adjX->page_size=box->size.x;
        box->adjY->page_size=box->size.y;
        box->adjX->value=(gfloat)((glong)((box->adjX->value+MIN(box->marked.x, box->select.x))
        		*box->size.x/ABS(box->marked.x-box->select.x)));
        box->adjY->value=(gfloat)((gint)((box->adjY->value+MIN(box->marked.y, box->select.y))
        		*box->size.y/ABS(box->marked.y-box->select.y)));
        box->adjX->upper=(gfloat)((glong)((box->adjX->upper*box->size.x/ABS(box->marked.x-box->select.x))));
        box->adjY->upper=(gfloat)((glong)((box->adjY->upper*box->size.y/ABS(box->marked.y-box->select.y))));

        gtk_adjustment_changed(box->adjX);
        gtk_adjustment_changed(box->adjY);

        x=box->size.x/2;
        y=box->size.y/2;
        gtk_databox_zoomed(widget, box);
      }
    }
  } else if (button==3) {
    if (box->selection_flag) {
      box->selection_flag=0;
      gtk_databox_draw_selection(widget, box, NULL);
    } else if (event->state&GDK_SHIFT_MASK){
      gtk_databox_zoom_home(widget, box);
    } else {
      box->adjX->lower=0;
      box->adjY->lower=0;
      box->adjX->page_size=box->size.x;
      box->adjY->page_size=box->size.y;
      box->adjX->value=(gfloat)((gint)(MAX(0, (box->adjX->value-box->adjX->page_size/2)/2)));
      box->adjY->value=(gfloat)((gint)(MAX(0, (box->adjY->value-box->adjY->page_size/2)/2)));
      box->adjX->upper=(gfloat)((gint)(MAX(box->size.x, box->adjX->upper/2)));
      box->adjY->upper=(gfloat)((gint)(MAX(box->size.y, box->adjY->upper/2)));

      gtk_adjustment_changed(box->adjX);
      gtk_adjustment_changed(box->adjY);

      x=box->size.x/2;
      y=box->size.y/2;
      gtk_databox_zoomed(widget, box);
    }
  }
  box->marked.x=x;
  box->marked.y=y;

  return TRUE;
}

static gint
gtk_databox_motion_notify_callback(GtkWidget *widget, GdkEventMotion *event,GtkDatabox *box)
{
  gint x, y;
  GdkModifierType state;

  x = event->x;
  y = event->y;
  state = event->state;

  if (state & GDK_BUTTON1_MASK) {
    gint width;
    gint height;

    gdk_window_get_size (widget->window, &width, &height);
    x=MAX(0,MIN(width-1,x));
    y=MAX(0,MIN(height-1,y));

    if (box->selection_flag) {
      gtk_databox_draw_selection(box->draw, box, NULL);
    } else {
      box->selection_flag=1;
    }

    box->select.x=x;
    box->select.y=y;
    gtk_databox_draw_selection(box->draw, box, NULL);
  }

  return TRUE;
}

/* ------------------------------------------------------------
                   External Functions
   ------------------------------------------------------------ */

void
gtk_databox_show_cross(GtkDatabox *box, gboolean flag)
{
  g_return_if_fail(box);
  g_return_if_fail(GTK_IS_DATABOX(box));
   
  if (flag) box->flags|=1<<GTK_DATABOX_SHOW_CROSS;
  else box->flags&=!(1<<GTK_DATABOX_SHOW_CROSS);
}

void
gtk_databox_get_coordinates(GtkDatabox *box, gint x, gint y, gfloat *cx, gfloat *cy)
{
  *cx=box->min.x
      +(box->adjX->value+x)
      *(box->max.x-box->min.x)/(box->adjX->upper-1);
  *cy=box->min.z
      +(box->adjY->upper-box->adjY->value-y-1)
      *(box->max.z-box->min.z)/(box->adjY->upper-1);
}

void
gtk_databox_get_marked_coordinates(GtkDatabox *box, gfloat *cx, gfloat *cy)
{
  gtk_databox_get_coordinates(box, box->marked.x, box->marked.y, cx, cy);
}

void
gtk_databox_get_delta_coordinates(GtkDatabox *box, gfloat *cx, gfloat *cy)
{
  gfloat mx, my;

  gtk_databox_get_coordinates(box, box->marked.x, box->marked.y, &mx, &my);
  gtk_databox_get_coordinates(box, box->select.x, box->select.y, cx, cy);
  *cx-=mx;
  *cy-=my;
}

void
gtk_databox_new_data(GtkDatabox *box, gint type)
{
  gint i, j;
  gfloat width;
  gfloat height;

  g_return_if_fail(GTK_IS_DATABOX(box));

  switch(type) {
    case GTK_DATABOX_NO_DATA:
      break;
    case GTK_DATABOX_LINES_2D:
    case GTK_DATABOX_POINTS_2D:
      box->min.x=box->data.X[0];
      box->min.z=box->data.Z[0][0];
      box->max.x=box->data.X[0];
      box->max.z=box->data.Z[0][0];
      for (i=0; i<box->data.dimX; i++) {
        box->min.x=MIN(box->data.X[i], box->min.x);
        box->max.x=MAX(box->data.X[i], box->max.x);
        for (j=0; j<box->data.dimY; j++) {
          box->min.z=MIN(box->data.Z[j][i], box->min.z);
          box->max.z=MAX(box->data.Z[j][i], box->max.z);
        }
      }
      width=MAX(1e-10, box->max.x-box->min.x)/10.;
      height=MAX(1e-10, box->max.z-box->min.z)/10.;
      box->min.x-=width;
      box->max.x+=width;
      box->min.z-=height;
      box->max.z+=height;
      break;
  }
  box->data.type=type;
  gtk_databox_zoom_home(box->draw, box);
}

void
gtk_databox_redraw(GtkDatabox *box)
{
  gtk_databox_draw_request_full(box->draw, TRUE, box);
}


/* ------------------------------------------------------------
                   Drawing Functions
   ------------------------------------------------------------ */

static void
gtk_databox_zoom_home(GtkWidget *widget, GtkDatabox *box)
{
  box->selection_flag=0;

  if (box->data.type==GTK_DATABOX_NO_DATA) {
    box->min.x=0;
    box->min.y=0;
    box->min.z=0;
    box->max.x=1;
    box->max.y=1;
    box->max.z=1;
  }

  box->adjX->value=box->adjX->lower=0;
  box->adjX->page_size=box->adjX->upper=MAX(box->size.x-1, 2);
  box->adjX->step_increment=box->adjX->page_size/20;
  box->adjX->page_increment=box->adjX->page_size*0.9;

  box->adjY->value=box->adjY->lower=0;
  box->adjY->page_size=box->adjY->upper=MAX(box->size.y-1, 2);
  box->adjY->step_increment=box->adjY->page_size/20;
  box->adjY->page_increment=box->adjY->page_size*0.9;

  gtk_adjustment_changed(box->adjX);
  gtk_adjustment_changed(box->adjY);
  gtk_databox_zoomed(widget, box);
}

static void
gtk_databox_zoomed(GtkWidget *widget, GtkDatabox *box)
{
  GdkRectangle redraw_rect;

  gtk_signal_emit (GTK_OBJECT (box), gtk_databox_signals[GTK_DATABOX_ZOOM_SIGNAL]);

  redraw_rect.x=0;
  redraw_rect.y=0;
  redraw_rect.width=box->size.x;
  redraw_rect.height=box->size.y;

  gtk_databox_draw_request(widget, TRUE, redraw_rect, box);
}

static void
gtk_databox_draw_request(GtkWidget *widget, gboolean now, GdkRectangle redraw_rect, GtkDatabox *box)
{
  box->redraw_rect.width=MIN(box->size.x-1, MAX(box->redraw_rect.x+box->redraw_rect.width,
  				redraw_rect.x+redraw_rect.width)
  			-MIN(box->redraw_rect.x, redraw_rect.x));
  box->redraw_rect.height=MIN(box->size.y-1, MAX(box->redraw_rect.y+box->redraw_rect.height,
  				redraw_rect.y+redraw_rect.height)
  			-MIN(box->redraw_rect.y, redraw_rect.y));
  box->redraw_rect.x=MAX(0, MIN(box->redraw_rect.x, redraw_rect.x));
  box->redraw_rect.y=MAX(0, MIN(box->redraw_rect.y, redraw_rect.y));

  if (now) gtk_widget_draw(widget, &redraw_rect);
}

static void
gtk_databox_draw_request_full(GtkWidget *widget, gboolean now, GtkDatabox *box)
{
  GdkRectangle redraw_rect;

  redraw_rect.x=0;
  redraw_rect.y=0;
  redraw_rect.width=box->size.x;
  redraw_rect.height=box->size.y;

  gtk_databox_draw_request(widget, now, redraw_rect, box);
}

static gint
gtk_databox_draw_selection(GtkWidget *widget, GtkDatabox *box, GdkEventExpose *event)
{
  if (!box->select_gc) {
    GdkGCValues values;
    GdkColormap* colormap;
    GdkColor color;

    color.red=65535;
    color.green=65535;
    color.blue=65535;
    colormap=gtk_widget_get_colormap(widget);
    gdk_color_alloc(colormap, &color);
    values.foreground=color;
    values.function=GDK_XOR;
    box->select_gc=gdk_gc_new_with_values(widget->window,
			&values,
			GDK_GC_FUNCTION | GDK_GC_FOREGROUND);
  }


  if (event) {
    gdk_draw_rectangle(box->select_pixmap,
                    widget->style->black_gc,
                    TRUE,
                    0, 0,
                    box->size.x, box->size.y);
    gdk_draw_rectangle(box->select_pixmap,
                    widget->style->white_gc,
                    FALSE,
                    MIN(box->marked.x , box->select.x),
                    MIN(box->marked.y , box->select.y),
                    ABS(box->marked.x - box->select.x),
                    ABS(box->marked.y - box->select.y));
    gdk_draw_pixmap(widget->window,
      box->select_gc,
      box->select_pixmap,
      event->area.x, event->area.y,
      event->area.x, event->area.y,
      event->area.width, event->area.height);
  } else {
    gdk_draw_rectangle(widget->window,
                    box->select_gc,
                    FALSE,
                    MIN(box->marked.x , box->select.x),
                    MIN(box->marked.y , box->select.y),
                    ABS(box->marked.x - box->select.x),
                    ABS(box->marked.y - box->select.y));

  }
    
  return TRUE;
}

void
gtk_databox_set_color(GtkDatabox *box, gint index, gushort red, gushort green, gushort blue)
{
  GdkColormap *colormap;
  GtkWidget *widget;
  gboolean color_allocate_success;
  
  g_return_if_fail(GTK_IS_DATABOX(box));
  g_return_if_fail(index<box->n_color);
  widget=box->draw;
  g_return_if_fail(GTK_IS_WIDGET(widget));
  
  box->data_color[index].red=red;
  box->data_color[index].green=green;
  box->data_color[index].blue=blue;
  
  if (box->n_color!=box->n_gc) return;
  
  colormap=gtk_widget_get_colormap(widget);
  if (box->data_color[index].pixel) {
    gdk_colormap_free_colors(colormap, &box->data_color[index], 1);
  }
  gdk_colormap_alloc_colors(colormap, &box->data_color[index], 1, FALSE, TRUE, &color_allocate_success);
  g_return_if_fail(color_allocate_success);
  gdk_gc_set_foreground(box->data_gc[index], &box->data_color[index]);
}

void
gtk_databox_set_colors(GtkDatabox *box, gint n_color, GdkColor *colors)
{
  gint i;
  GdkColormap *colormap;
  GtkWidget *widget;
  
  g_return_if_fail(GTK_IS_DATABOX(box));
  g_return_if_fail(n_color>0);
  widget=box->draw;
  g_return_if_fail(GTK_IS_WIDGET(widget));
  
  if (!box->n_gc) {
    box->data_color=(GdkColor *) g_realloc((void *) box->data_color, n_color*sizeof(GdkColor));
    box->n_color=n_color;
    for (i=0; i<n_color; i++) {
      box->data_color[i]=colors[i];
    }
  }

  if (box->n_gc>n_color) {
    for (i=n_color; i<box->n_gc; i++) {
      gdk_gc_destroy(box->data_gc[i]);
    }
    box->n_gc=n_color;
  }
  box->data_gc=(GdkGC **) g_realloc((void *) box->data_gc, n_color*sizeof(GdkGC));
  
  colormap=gtk_widget_get_colormap(widget);
  if (box->n_color>n_color) {
    for (i=n_color; i<box->n_color; i++) {
      if (box->data_color[i].pixel) {
        gdk_colormap_free_colors(colormap, &box->data_color[i], 1);
      }
    }
    box->n_color=n_color;
  }

  box->data_color=(GdkColor *) g_realloc((void *) box->data_color, n_color*sizeof(GdkColor));
  
  for (i=box->n_color; i<n_color; i++) {
    box->data_color[i].pixel=0;
  }
  
  for (i=0; i<n_color; i++) {
    box->data_color[i].red=colors[i].red;
    box->data_color[i].green=colors[i].green;
    box->data_color[i].blue=colors[i].blue;
  }

  for (i=0; i<box->n_color; i++) {
    gtk_databox_set_color(box, i, colors[i].red, colors[i].green, colors[i].blue);
  }
  for (i=box->n_color; i<n_color; i++) {
    gtk_databox_new_data_gc(widget, box, i);
  }
}

static void
gtk_databox_new_data_gc(GtkWidget *widget, GtkDatabox *box, gint index)
{
  GdkGCValues values;
  GdkColormap* colormap;
  gboolean color_allocate_success;
  
  colormap=gtk_widget_get_colormap(widget);
  gdk_colormap_alloc_colors(colormap, &box->data_color[index], 1, FALSE, TRUE, &color_allocate_success);
  g_return_if_fail(color_allocate_success);
  values.foreground=box->data_color[index];
  values.function=GDK_COPY;
  box->data_gc[index]=gdk_gc_new_with_values(widget->window,
  				&values,
  				GDK_GC_FUNCTION | GDK_GC_FOREGROUND);
}

static void
gtk_databox_draw_cross (GtkWidget *widget, GtkDatabox *box, gfloat xfactor, gfloat yfactor)
{
  gint x, y;
  char *str;
   
  xfactor=(box->adjX->upper-1)/(box->max.x-box->min.x);
  yfactor=(box->adjY->upper-1)/(box->max.z-box->min.z);

  x=(int)((0-box->min.x)*xfactor-box->adjX->value);
  y=(int)(box->adjY->upper-1-(0-box->min.z)*yfactor-box->adjY->value);
  gdk_draw_line(box->pixmap, widget->style->fg_gc[0], 
                         x, box->cross_border, x, box->size.y - box->cross_border);
  gdk_draw_line(box->pixmap, widget->style->fg_gc[0], 
                         box->cross_border, y, box->size.x - box->cross_border, y);
  str = "0";
  x-=gdk_text_width(widget->style->font, str, strlen(str));
  y+=gdk_text_height(widget->style->font, str, strlen(str)) + 2;
  gdk_draw_string(box->pixmap, widget->style->font, widget->style->fg_gc[0], x, y, str);
}

static void
gtk_databox_draw (GtkWidget *widget, GtkDatabox *box, GdkEventExpose *event)
{
  gint i,j;
  gint x, y;
  gint x1, y1;
  gint count;
  GdkPoint *points;
  GdkSegment *segments;
  gfloat xfactor;
  gfloat yfactor;

  g_return_if_fail(GTK_IS_DATABOX(box));
  if (!GTK_WIDGET_VISIBLE(widget)) return;

  if (box->n_gc!=box->n_color) {
    for (i=0; i<box->n_gc; i++) {
      gdk_gc_destroy(box->data_gc[i]);
    }
    box->n_gc=box->n_color;
    box->data_gc=(GdkGC **) g_realloc((void *) box->data_gc, box->n_color*sizeof(GdkGC *));

    for (i=0; i<box->n_gc; i++) {
      gtk_databox_new_data_gc(widget, box, i);
    }
  }

  gdk_draw_rectangle (box->pixmap,
                      widget->style->bg_gc[0],
                      TRUE,
                      0, 0,
                      box->size.x,
                      box->size.y);

  switch(box->data.type) {
    case GTK_DATABOX_POINTS_2D:
      points=(GdkPoint *) g_malloc(box->data.dimX*sizeof(GdkPoint));
      xfactor=(box->adjX->upper-1)/(box->max.x-box->min.x);
      yfactor=(box->adjY->upper-1)/(box->max.z-box->min.z);
      if (box->flags&(1<<GTK_DATABOX_SHOW_CROSS)) {
        gtk_databox_draw_cross(widget, box, xfactor, yfactor);
      }
      for (i=0; i<box->data.dimY; i++) {
        count=0;
        for (j=0; j<box->data.dimX; j++) {
          x=(box->data.X[j]-box->min.x)*xfactor-box->adjX->value+0.5;
          if (x>=0 && x<box->size.x) {
            y=box->adjY->upper-1-(box->data.Z[i][j]-box->min.z)*yfactor-box->adjY->value+0.5;
            if (y>=0 && y<box->size.y) {
              points[count].x=x;
              points[count].y=y;
              count++;
            }
          }
        }
        if (count) {
           gdk_draw_points(box->pixmap,
   		box->data_gc[MIN(i,box->n_gc-1)],
      		points, count);
        }
      }
      g_free((void *) points);
      break;
    case GTK_DATABOX_LINES_2D:
      segments=(GdkSegment *) g_malloc(box->data.dimX*sizeof(GdkSegment));
      xfactor=(box->adjX->upper-1)/(box->max.x-box->min.x);
      yfactor=(box->adjY->upper-1)/(box->max.z-box->min.z);
      if (box->flags&(1<<GTK_DATABOX_SHOW_CROSS)) {
        gtk_databox_draw_cross(widget, box, xfactor, yfactor);
      }
      for (i=0; i<box->data.dimY; i++) {
        count=0;
        x1=(box->data.X[0]-box->min.x)*xfactor-box->adjX->value+0.5;
        y1=box->adjY->upper-1-(box->data.Z[i][0]-box->min.z)*yfactor-box->adjY->value+0.5;
        for (j=1; j<box->data.dimX; j++) {
          x=(box->data.X[j]-box->min.x)*xfactor-box->adjX->value+0.5;
          y=box->adjY->upper-1-(box->data.Z[i][j]-box->min.z)*yfactor-box->adjY->value+0.5;
          if ((x1>=0 && x1<box->size.x && y1>=0 && y1<box->size.y) ||
              (x>=0 && x<box->size.x && y>=0 && y<box->size.y)) {
            segments[count].x1=x1;
            segments[count].y1=y1;
            segments[count].x2=x;
            segments[count].y2=y;
            count++;
          }
          x1=x;
          y1=y;
        }
        if (count) {
          gdk_draw_segments(box->pixmap,
	       		box->data_gc[MIN(i,box->n_gc-1)],
          		segments, count);
        }
      }
      g_free((void *) segments);
      break;
    case GTK_DATABOX_NO_DATA:
      gdk_draw_text (box->pixmap,
		     widget->style->font,
                     widget->style->fg_gc[0],
                     10,10+gdk_text_height(widget->style->font,
                                     "NO DATA",
                                      strlen("NO DATA")),
                     "NO DATA",
         	     strlen("NO_DATA"));
      break;
  }
}

