/* gtkdatabox.h */

#ifndef __GTK_DATABOX_H__
#define __GTK_DATABOX_H__


#include <gdk/gdk.h>
#include <gtk/gtkvbox.h>


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GTK_TYPE_DATABOX            (gtk_databox_get_type ())
#define GTK_DATABOX(obj)            GTK_CHECK_CAST (obj, gtk_databox_get_type (), GtkDatabox)
#define GTK_DATABOX_CLASS(klass)    GTK_CHECK_CLASS_CAST (klass, gtk_databox_get_type (), GtkDataboxClass)
#define GTK_IS_DATABOX(obj)         GTK_CHECK_TYPE (obj, gtk_databox_get_type ())
#define GTK_IS_DATABOX_CLASS(klass) GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DATABOX))


typedef struct _GtkDataboxCoord  GtkDataboxCoord;
typedef struct _GtkDataboxData   GtkDataboxData;
typedef struct _GtkDatabox       GtkDatabox;
typedef struct _GtkDataboxClass  GtkDataboxClass;
typedef struct _GtkDataboxValue  GtkDataboxValue;

enum {
   GTK_DATABOX_NO_DATA=0,
   GTK_DATABOX_POINTS_2D,
   GTK_DATABOX_LINES_2D,
};

enum {
  GTK_DATABOX_SHOW_CROSS=0,
};

struct _GtkDataboxCoord
{
  gint x;
  gint y;
};

struct _GtkDataboxValue
{
  gfloat x;
  gfloat y;
  gfloat z;
};

struct _GtkDataboxData
{
  guint type;
  gint dimX;
  gint dimY;
  gfloat *X;
  gfloat *Y;
  gfloat **Z;
  gfloat *XY_Z;
};

struct _GtkDatabox
{
  GtkVBox vbox;

  GtkDataboxData data;
  GtkWidget *draw;
  GtkAdjustment *adjX;
  GtkAdjustment *adjY;
  GdkPixmap *pixmap;
  GdkPixmap *select_pixmap;
  gint n_color;
  GdkColor *data_color;
  glong flags;
  gint n_gc;
  GdkGC **data_gc;
  GdkGC *select_gc;
  gint cross_border;
  GtkDataboxCoord size;
  GtkDataboxCoord marked;
  GtkDataboxCoord select;
  GdkRectangle redraw_rect;
  GtkDataboxValue min;
  GtkDataboxValue max;
  gboolean selection_flag;
};

struct _GtkDataboxClass
{
  GtkVBoxClass parent_class;

  void (* gtk_databox) (GtkDatabox *box);
};

guint		gtk_databox_get_type			(void);
GtkWidget*	gtk_databox_new				(void);
void 		gtk_databox_get_coordinates		(GtkDatabox *box, gint x, gint y, gfloat *cx, gfloat *cy);
void 		gtk_databox_get_marked_coordinates	(GtkDatabox *box, gfloat *cx, gfloat *cy);
void 		gtk_databox_get_delta_coordinates	(GtkDatabox *box, gfloat *cx, gfloat *cy);
void 		gtk_databox_new_data			(GtkDatabox *box, gint type);
void		gtk_databox_redraw			(GtkDatabox *box);
void		gtk_databox_set_color			(GtkDatabox *box, gint index, gushort red, gushort green, gushort blue);
void		gtk_databox_set_colors			(GtkDatabox *box, gint n_colors, GdkColor *colors);
void		gtk_databox_show_cross			(GtkDatabox *box, gboolean flag);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __GTK_DATABOX_H__ */

