/* gtkengine.c */

#include <gtk/gtkengine.h>
#include <stdio.h>
#include <string.h>

static void gtk_engine_class_init              (GtkEngineClass *klass);
static void gtk_engine_init                    (GtkEngine *engine);

/* ------------------------------------------------------------
                   Gtk Engine Initialization
   ------------------------------------------------------------ */

guint
gtk_engine_get_type ()
{
  static guint engine_type = 0;

  if (!engine_type)
    {
      GtkTypeInfo engine_info =
      {
        "GtkEngine",
        sizeof (GtkEngine),
        sizeof (GtkEngineClass),
        (GtkClassInitFunc) gtk_engine_class_init,
        (GtkObjectInitFunc) gtk_engine_init,
        (GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL
      };

      engine_type = gtk_type_unique (gtk_object_get_type (), &engine_info);
    }

  return engine_type;
}

static void
gtk_engine_class_init (GtkEngineClass *class)
{
}

static void
gtk_engine_init (GtkEngine *engine)
{
  engine->name=NULL;
  engine->num_vars=0;
  engine->vars=NULL;
  engine->num_functions=0;
  engine->functions=NULL;
}

GtkObject *
gtk_engine_new ()
{
  return GTK_OBJECT ( gtk_type_new (gtk_engine_get_type ()));
}

/* ------------------------------------------------------------
 *
 *                  External Functions
 *
 * ------------------------------------------------------------ */

gint 
gtk_engine_run(GtkEngine *engine)
{
  gint i;
  
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  for (i=0; i<engine->num_functions; i++) {
    engine->functions[i].fnct(engine->functions[i].target, 
    			      engine->functions[i].num,
    			      engine->functions[i].value);
  }
  
  return 1;
}

gint 
gtk_engine_set_name(GtkEngine *engine, gchar *name)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  g_free(engine->name);
  engine->name=g_strdup(name);
  
  return 1;
}

gchar *
gtk_engine_get_name(GtkEngine *engine)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), NULL);
  
  return engine->name;
}

gint
gtk_engine_set_variables(GtkEngine *engine, guint num_vars, GtkEngineVar *vars)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  engine->num_vars=num_vars;
  engine->vars=vars;
  
  return 1;
}

gint
gtk_engine_get_variables(GtkEngine *engine, GtkEngineVar **vars)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), -1);
  
  *vars=engine->vars;
  
  return engine->num_vars;
}

gint
gtk_engine_set_functions(GtkEngine *engine, guint num_functions, GtkEngineFunction *functions)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  engine->num_functions=num_functions;
  engine->functions=functions;
  
  return 1;
}

gint
gtk_engine_set_user_data(GtkEngine *engine, gint index, gint user_data)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  engine->vars[index].user_data=user_data;
  
  return 1;
}

gint
gtk_engine_set_value(GtkEngine *engine, gint index, gdouble value)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  engine->vars[index].value=value;
  
  return 1;
}

gdouble
gtk_engine_get_value(GtkEngine *engine, gint index)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), 0);
  
  return engine->vars[index].value;
}

gint
gtk_engine_get_value_index_by_name(GtkEngine *engine, gchar *name)
{
  gint i;
  g_return_val_if_fail(GTK_IS_ENGINE(engine), -1);
  
  for (i=0; i<engine->num_vars; i++) {
    if (strlen(name)==strlen(engine->vars[i].name) && !strcmp(name, engine->vars[i].name))
      return i;
  }
  
  return -1;
}

gdouble *
gtk_engine_get_value_pointer(GtkEngine *engine, gint index)
{
  g_return_val_if_fail(GTK_IS_ENGINE(engine), NULL);
  
  if (-1<index && index<engine->num_vars) {
    return &engine->vars[index].value;
  }
  
  return NULL;
}

/* ------------------------------------------------------------
 *
 *                  Internal Functions
 *
 * ------------------------------------------------------------ */

