/* gtkengine.h */

#ifndef __GTK_ENGINE_H__
#define __GTK_ENGINE_H__

#include <gtk/gtkobject.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define GTK_TYPE_ENGINE			(gtk_engine_get_type ())
#define GTK_ENGINE(obj)			GTK_CHECK_CAST (obj, gtk_engine_get_type (), GtkEngine)
#define GTK_ENGINE_CLASS(klass)		GTK_CHECK_CLASS_CAST (klass, gtk_engine_get_type (), GtkEngineClass)
#define GTK_IS_ENGINE(obj)		GTK_CHECK_TYPE (obj, gtk_engine_get_type ())
#define GTK_IS_ENGINE_CLASS(klass)	GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_ENGINE))

typedef struct _GtkEngineVar		GtkEngineVar;
typedef struct _GtkEngineFunction	GtkEngineFunction;
typedef struct _GtkEngine		GtkEngine;
typedef struct _GtkEngineClass		GtkEngineClass;

struct _GtkEngineVar
{
  gchar *name;
  gchar *desc;
  gdouble value;
  gint user_data;
  gpointer user_pointer;
};

struct _GtkEngineFunction
{
  double *target;
  int num;
  double **value;
  int (*fnct)(double *target, int num, double **value);
  char *name;
  char *desc;
};

struct _GtkEngine
{
  GtkObject *object;

  char *name;
  guint num_vars;
  GtkEngineVar *vars;
  guint num_functions;
  GtkEngineFunction *functions;
};

struct _GtkEngineClass
{
  GtkObjectClass parent_class;
};

guint           gtk_engine_get_type			(void);
GtkObject *	gtk_engine_new				(void);
gint		gtk_engine_run				(GtkEngine *engine);
gint		gtk_engine_set_name			(GtkEngine *engine, 
							 gchar *name);
gchar *		gtk_engine_get_name			(GtkEngine *engine);
gint		gtk_engine_set_variables		(GtkEngine *engine,
							 guint num_vars, 
							 GtkEngineVar *vars);
gint		gtk_engine_get_variables		(GtkEngine *engine,
							 GtkEngineVar **vars);
gint		gtk_engine_set_functions		(GtkEngine *engine, 
							 guint num_functions,
							 GtkEngineFunction *functions);
gint		gtk_engine_set_user_data		(GtkEngine *engine, 
							 gint index, 
							 gint user_data);
gint		gtk_engine_set_value			(GtkEngine *engine, 
							 gint index, 
							 gdouble value);
gdouble		gtk_engine_get_value			(GtkEngine *engine, 
							 gint index);
gint		gtk_engine_get_value_index_by_name	(GtkEngine *engine, 
							 gchar *name);
gdouble *	gtk_engine_get_value_pointer		(GtkEngine *engine, 
							 gint index);



#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __GTK_ENGINE_H__ */
