/* gtkextfilesel.c */

#include <stdio.h>
#include <string.h>
#include "gtk/gtkcombo.h"
#include "gtk/gtksignal.h"
#include "gtk/gtkmain.h"
#include "gtk/gtklist.h"
#include "gtk/gtkclist.h"
#include "gtk/gtkentry.h"
#include "gtk/gtkextfilesel.h"

static void gtk_extended_file_selection_class_init	(GtkExtendedFileSelectionClass *klass);
static void gtk_extended_file_selection_init		(GtkExtendedFileSelection *box);
static void gtk_extended_file_selection_file_type_changed(GtkWidget *combo, GtkExtendedFileSelection *efs);
static void gtk_extended_file_selection_update_file_list(GtkWidget *combo, GtkExtendedFileSelection *efs);
static void gtk_extended_file_selection_dir_button (GtkWidget *widget, gpointer user_data);
static void gtk_extended_file_selection_dir_changed (GtkWidget *widget, gpointer user_data);

/* ------------------------------------------------------------
 *        Gtk Extended File Selection  Initialization
 * ------------------------------------------------------------ */

enum {
  GTK_EFS_CHANGED_SIGNAL,
  LAST_SIGNAL
};

static gint gtk_extended_file_selection_signals[LAST_SIGNAL] = { 0 };

guint
gtk_extended_file_selection_get_type ()
{
  static guint efs_type = 0;

  if (!efs_type)
    {
      GtkTypeInfo efs_info =
      {
        "GtkExtendedFileSelection",
        sizeof (GtkExtendedFileSelection),
        sizeof (GtkExtendedFileSelectionClass),
        (GtkClassInitFunc) gtk_extended_file_selection_class_init,
        (GtkObjectInitFunc) gtk_extended_file_selection_init,
        (GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL
      };

      efs_type = gtk_type_unique (gtk_file_selection_get_type (), &efs_info);
    }

  return efs_type;
}

static void
gtk_extended_file_selection_class_init (GtkExtendedFileSelectionClass *class)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass*) class;

  gtk_extended_file_selection_signals[GTK_EFS_CHANGED_SIGNAL] = gtk_signal_new ("extended_file_list_changed",
                                         GTK_RUN_FIRST,
                                         object_class->type,
                                         GTK_SIGNAL_OFFSET (GtkExtendedFileSelectionClass, extended_file_list_changed),
                                         gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);


  gtk_object_class_add_signals (object_class, gtk_extended_file_selection_signals, LAST_SIGNAL);

  class->extended_file_list_changed = NULL;
}

static void
gtk_extended_file_selection_init (GtkExtendedFileSelection *efs)
{
  efs->file_type=0;
}


GtkWidget*
gtk_extended_file_selection_new (gchar *title)
{
  GtkExtendedFileSelection *efs;

  efs = gtk_type_new (gtk_extended_file_selection_get_type ());
  gtk_window_set_title (GTK_WINDOW (efs), title);
  
  return GTK_WIDGET (efs);
}

/* Adapted from gtk-app-devel-list, Andreas Tille: tille@physik.uni-halle.de */
void
gtk_extended_file_selection_goto(GtkExtendedFileSelection *efs)
{
  GtkFileSelection *fs=GTK_FILE_SELECTION(efs);
  char *pre_file;
  char *list_name;
  gint i;
  
  g_return_if_fail(efs);
  g_return_if_fail(GTK_IS_EXTENDED_FILE_SELECTION(efs));
  
  pre_file=gtk_entry_get_text(GTK_ENTRY(fs->selection_entry));
  if (pre_file) {
    i=-1;
     while(gtk_clist_get_text(GTK_CLIST(fs->file_list), ++i, 0, &list_name)) {
      if (list_name && !strcmp(list_name, pre_file) && strlen(list_name)==strlen(pre_file)) {
        gtk_clist_select_row(GTK_CLIST(fs->file_list), i, 0);
        gtk_clist_moveto(GTK_CLIST(fs->file_list), i, 0, 0.5, 0.0);
        break;
      }
    }
  }
}

GtkWidget *
gtk_extended_file_selection_type_popup(GtkExtendedFileSelection *efs,
					GList *types,
					guint index)
{
  GtkWidget *combo;
  GtkWidget *hbox;
  GtkWidget *label;
  
  g_return_val_if_fail(efs, NULL);
  g_return_val_if_fail(GTK_IS_EXTENDED_FILE_SELECTION(efs), NULL);

  label=gtk_label_new("Type of file:");
  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(efs)->main_vbox), label, FALSE, FALSE, 0);
  gtk_widget_show(label);

  hbox=gtk_hbox_new(TRUE, 5);
  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(efs)->main_vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);

  combo=efs->combo=gtk_combo_new ();
  gtk_combo_set_popdown_strings (GTK_COMBO (combo), types);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry), FALSE);
  
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(efs)->selection_entry), "changed",
                      (GtkSignalFunc) gtk_extended_file_selection_dir_button, 
                      (gpointer) efs);
  gtk_signal_connect(GTK_OBJECT (efs), "extended_file_list_changed",
                      (GtkSignalFunc) gtk_extended_file_selection_dir_changed, 
                      (gpointer) efs);
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO (combo)->list), "selection_changed",
  		GTK_SIGNAL_FUNC(gtk_extended_file_selection_file_type_changed),
  		(gpointer) efs);
            
  gtk_list_select_item(GTK_LIST(GTK_COMBO (combo)->list), index);

  gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
  gtk_widget_show(combo);
  
  return combo;
}

/* ------------------------------------------------------------
 *        Gtk Extended File Selection Internal Calls
 * ------------------------------------------------------------ */

static void
gtk_extended_file_selection_file_type_changed(GtkWidget *widget, GtkExtendedFileSelection *efs)
{
  GList *list_children;
  GList *list_selection;
  
  list_children=GTK_LIST(GTK_COMBO(efs->combo)->list)->children;
  list_selection=GTK_LIST(GTK_COMBO(efs->combo)->list)->selection;
  if (list_children && list_selection) {
    efs->file_type=g_list_index(list_children, list_selection->data);
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(efs), "./");
  }
}

static void
gtk_extended_file_selection_update_file_list(GtkWidget *entry, GtkExtendedFileSelection *efs)
{
  GtkFileSelection *fs=GTK_FILE_SELECTION(efs);
  gchar *extension;
  gchar *list_extension;
  gchar *list_name;
  guint i, j;
  
  extension=gtk_entry_get_text(GTK_ENTRY(entry));
  extension=strrchr(extension, '.');
  
  if (extension) {
    extension=&extension[1];
    i=0;
    gtk_clist_freeze(GTK_CLIST(fs->file_list));
    while(gtk_clist_get_text(GTK_CLIST(fs->file_list), i, 0, &list_name)) {
      if ((list_extension=strrchr(list_name, '.'))==NULL) {
        gtk_clist_remove(GTK_CLIST(fs->file_list), i);
        continue;
      }
      list_extension=&list_extension[1];
      if (strlen(list_extension)!=strlen(extension)) {
        gtk_clist_remove(GTK_CLIST(fs->file_list), i);
        continue;
      }
      for (j=0; j<strlen(extension); j++) {
        switch(extension[j]) {
          case '?': break;
          case '#': if (list_extension[j]<'0' || list_extension[j]>'9') {
                      gtk_clist_remove(GTK_CLIST(fs->file_list), i);
                      j=strlen(extension);
                      i--;
          	    }
	            break;
	  default: if (list_extension[j]!=extension[j]) {
                      gtk_clist_remove(GTK_CLIST(fs->file_list), i);
                      j=strlen(extension);
                      i--;
          	    }
	            break;
        }
      }
      
      i++;
    }
    gtk_clist_thaw(GTK_CLIST(fs->file_list));
    
  }
}

static void
gtk_extended_file_selection_dir_button (GtkWidget *widget,
                        	       gpointer user_data)
{
  GtkExtendedFileSelection *efs = NULL;
  GtkFileSelection *fs = NULL;

  g_return_if_fail (widget);
  g_return_if_fail (GTK_IS_ENTRY (widget));

  g_return_if_fail(user_data);
  efs = GTK_EXTENDED_FILE_SELECTION (user_data);
  g_return_if_fail (efs != NULL);
  g_return_if_fail (GTK_IS_FILE_SELECTION (efs));
  fs = GTK_FILE_SELECTION (user_data);

  gtk_signal_emit(GTK_OBJECT(efs), gtk_extended_file_selection_signals[GTK_EFS_CHANGED_SIGNAL]);
}

static void 
gtk_extended_file_selection_dir_changed (GtkWidget *widget, 
						gpointer user_data)
{
  GtkExtendedFileSelection *efs = NULL;

  g_return_if_fail(user_data);
  efs = GTK_EXTENDED_FILE_SELECTION (user_data);
  g_return_if_fail (efs != NULL);
  g_return_if_fail (GTK_IS_EXTENDED_FILE_SELECTION (efs));

  gtk_extended_file_selection_update_file_list(GTK_COMBO(efs->combo)->entry, efs);
}
