/* Thanks to Andreas Tille <tille@physik.uni-halle.de> for the first version 
of this program */

/* Test for GtkDatabox widget */
#define POINTS  2000

#include <stdio.h>

#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#include <gtk/gtk.h>
#include <gtk/gtkdatabox.h>

#define MY_MAX_COLORS 6
static GdkColor colors[MY_MAX_COLORS]={
  {0, 0xFFFF, 0, 0},
  {0, 0, 0xFFFF, 0},
  {0, 0, 0, 0xFFFF},
  {0, 0xFFFF, 0, 0xFFFF},
  {0, 0, 0xFFFF, 0xFFFF},
  {0, 0xFFFF, 0xFFFF, 0},
  };
  
GtkWidget *
show_create(GtkWidget *container)
{
  GtkWidget *box;       /* GtkDatabox */
  int        i;

  box = gtk_databox_new ();
  gtk_container_add(GTK_CONTAINER(container), box);

   GTK_DATABOX(box)->data.X=(gfloat *) g_malloc(POINTS*sizeof(gfloat));
   GTK_DATABOX(box)->data.Z=(gfloat **) g_malloc(4*sizeof(gfloat *));
   GTK_DATABOX(box)->data.Z[0]=(gfloat *) g_malloc(POINTS*sizeof(gfloat));
   GTK_DATABOX(box)->data.Z[1]=(gfloat *) g_malloc(POINTS*sizeof(gfloat));
   GTK_DATABOX(box)->data.Z[2]=(gfloat *) g_malloc(POINTS*sizeof(gfloat));
   GTK_DATABOX(box)->data.Z[3]=(gfloat *) g_malloc(POINTS*sizeof(gfloat));
   for (i = 0; i < POINTS; i++ ) {
      GTK_DATABOX(box)->data.X[i]=i;
      GTK_DATABOX(box)->data.Z[0][i]=100.*sin(i*2*PI/200+1.);
      GTK_DATABOX(box)->data.Z[1][i]=100.*cos(i*2*PI/200+1.);
      GTK_DATABOX(box)->data.Z[2][i]=100.*(cos(i*2*PI/200+1.))*(cos(i*2*PI/200+1.));
      GTK_DATABOX(box)->data.Z[3][i]=-100.*(sin(i*2*PI/200+1.))*(sin(i*2*PI/200+1.));
   }
   GTK_DATABOX(box)->data.dimX=POINTS;
   GTK_DATABOX(box)->data.dimY=4;
   gtk_databox_set_colors(GTK_DATABOX(box), MY_MAX_COLORS, colors);
   gtk_databox_set_color(GTK_DATABOX(box), 0, 0xFFFF, 0, 0);
   gtk_databox_set_color(GTK_DATABOX(box), 1, 0, 0xFFFF, 0);
   gtk_databox_new_data(GTK_DATABOX(box), GTK_DATABOX_LINES_2D);
   gtk_databox_show_cross(GTK_DATABOX(box), TRUE);
   return box;
}


void
do_exit (GtkWidget *widget, GtkWidget *window)
{
  gtk_widget_destroy (window);
  gtk_main_quit ();
}

static void
create_main_window (void)
{
   GtkWidget *window;
   GtkWidget *box;
   GtkWidget *container;
   GtkWidget *button;

   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &window);
   gtk_signal_connect(GTK_OBJECT(window), "delete-event", GTK_SIGNAL_FUNC(gtk_true), NULL);
   gtk_widget_set_usize(window, 500, 500);
      
   container = gtk_vbox_new (FALSE, 0);
   gtk_container_add(GTK_CONTAINER(window), container);

   box=show_create(container);
   
   button = gtk_button_new_with_label ("close");
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(do_exit), GTK_OBJECT(window));
   gtk_box_pack_start (GTK_BOX (container), button, FALSE, FALSE, 0);

   gtk_widget_show_all(window);
   
   while(gtk_events_pending()) {
     gtk_main_iteration();
   }
   
   gtk_databox_set_colors(GTK_DATABOX(box), 7, colors);
   gtk_databox_set_color(GTK_DATABOX(box), 2, 0, 0, 0xFFFF);
   gtk_databox_set_color(GTK_DATABOX(box), 3, 0x7FFF, 0x7FFF, 0x7FFF);
   gtk_databox_redraw(GTK_DATABOX(box));
}

int
main (int argc, char *argv[])
{
  gtk_init (&argc, &argv);

  create_main_window ();

  gtk_main ();

  return 0;
}
