%{
  #include "script_language.h"  /* Contains definition of Symrec        */
  static int yylex();
  static void yyerror(char *s);
%}
%union {
  double     	d_val;  	/* For returning double numbers.                   */
  int     	l_val;  	/* For returning int numbers.                   */
  char     	*s_val;  	/* For returning strings.                   */
  void     	*v_val;  	/* For returning void.                   */
  symrec	*ptr;   	/* For returning symbol-table pointers */
}
%{
static int check_arg(symrec *fnct, symrec *args);
%}

%token <d_val>  D_VAL        /* Simple double precision number   */
%token <l_val>  L_VAL        /* Simple double precision number   */
%token <s_val>  S_VAL        /* Simple double precision number   */
%token <v_val>  V_VAL        /* Simple double precision number   */
%token <ptr>    D_VAR D_FNC /* Variable and Function            */
%token <ptr>    L_VAR L_FNC /* Variable and Function            */
%token <ptr>    S_VAR S_FNC /* Variable and Function            */
%token <ptr>    V_FNC EXIT_FNC /* Function and EXIT           */
%type  <ptr>	func_pars
%type  <d_val>  d_exp
%type  <d_val>  d_func
%type  <l_val>  l_func
%type  <s_val>  s_func
%type  <l_val>	l_exp
%type  <s_val>  s_exp

%right '='
%left '-' '+'
%left '*' '/'
%left NEG     /* Negation--unary minus */
%right '^'    /* Exponentiation        */

/* Grammar follows */

%%
input:   /* empty */
        | input line
;

line:
          '\n'
        |  exp '\n'
        | error '\n' { yyerrok;                  }
;

exp:      d_exp  { printf ("double: \t%.10g\n", $1); }
        | l_exp  { printf ("int: \t%d\n", (int) $1); }
        | s_exp  { printf ("string: \t%s\n", $1); }
        | v_exp  { printf ("void...\n"); }


d_exp:    D_VAL                { $$ = $1;                         }
        | D_VAR                { $$ = $1->value.d_var;              }
        | L_VAR '=' d_exp        { $$ = $3; $1->value.d_var = $3;  $1->type=D_VAR; }
        | D_VAR '=' d_exp        { $$ = $3; $1->value.d_var = $3;     }
        | S_VAR '=' d_exp        { $$ = $3; $1->value.d_var = $3;   $1->type=D_VAR; }
	| d_func		{ $$ = $1;			}
        | d_exp '+' d_exp        { $$ = $1 + $3;                    }
        | d_exp '+' l_exp        { $$ = $1 + $3;                    }
        | l_exp '+' d_exp        { $$ = $1 + $3;                    }
        | d_exp '-' d_exp        { $$ = $1 - $3;                    }
        | d_exp '-' l_exp        { $$ = $1 - $3;                    }
        | l_exp '-' d_exp        { $$ = $1 - $3;                    }
        | d_exp '*' d_exp        { $$ = $1 * $3;                    }
        | d_exp '*' l_exp        { $$ = $1 * $3;                    }
        | l_exp '*' d_exp        { $$ = $1 * $3;                    }
        | d_exp '/' d_exp        { $$ = $1 / $3;                    }
        | d_exp '/' l_exp        { $$ = $1 / $3;                    }
        | l_exp '/' d_exp        { $$ = $1 / $3;                    }
        | '-' d_exp  %prec NEG { $$ = -$2;                        }
        | d_exp '^' d_exp        { $$ = pow ($1, $3);                    }
        | d_exp '^' l_exp        { $$ = pow ($1, $3);                    }
        | l_exp '^' d_exp        { $$ = pow ($1, $3);                    }
        | l_exp '^' l_exp        { $$ = pow ($1, $3);                    }
        | '(' d_exp ')'        { $$ = $2;                         }
/**/
;

l_exp:    L_VAL                { $$ = $1;                         }
        | L_VAR                { $$ = $1->value.l_var;              }
        | L_VAR '=' l_exp        { $$ = $3; $1->value.l_var = $3;     }
        | D_VAR '=' l_exp        { $$ = $3; $1->value.l_var = $3;   $1->type=L_VAR; }
        | S_VAR '=' l_exp        { $$ = $3; $1->value.l_var = $3;   $1->type=L_VAR; }
	| l_func		{ $$ = $1;			}
        | l_exp '+' l_exp        { $$ = $1 + $3;                    }
        | l_exp '-' l_exp        { $$ = $1 - $3;                    }
        | l_exp '*' l_exp        { $$ = $1 * $3;                    }
        | l_exp '/' l_exp        { $$ = $1 / $3;                    }
        | '-' l_exp  %prec NEG { $$ = -$2;                        }
        | '(' l_exp ')'        { $$ = $2;                         }
;


s_exp:    S_VAL                { $$ = $1;                    }
        | S_VAR                { $$ = $1->value.s_var;              }
        | L_VAR '=' s_exp        { $$ = $3; $1->value.s_var = $3;  $1->type=S_VAR; }
        | D_VAR '=' s_exp        { $$ = $3; $1->value.s_var = $3;  $1->type=S_VAR; }
        | S_VAR '=' s_exp        { $$ = $3; $1->value.s_var = $3;  }
	| s_func		{ $$ = $1;			}
;

v_exp:    V_VAL                {                         }
        | EXIT_FNC                { exit(1);                  }
;

d_func:  D_FNC '(' func_pars ')'  { if (check_arg($1, $3)) { $$ = (*($1->value.d_fnc))($3); } else { $$=0; } }
;

l_func:  L_FNC '(' func_pars ')'  { if (check_arg($1, $3)) { $$ = (*($1->value.l_fnc))($3); } else { $$=0; } }
;

s_func:  S_FNC '(' func_pars ')'  { if (check_arg($1, $3)) { $$ = ($1->value.s_fnc)($3); } else { $$=(char *) 0; } }
;

func_pars: 			{ $$=(symrec *) 0; }
	| d_exp			{ $$=newarg(&$1, D_VAL); }
	| l_exp			{ $$=newarg(&$1, L_VAL); }
	| s_exp			{ $$=newarg(&$1, S_VAL); }
	| func_pars ',' d_exp	{ $$=addarg($1, &$3, D_VAL); }
	| func_pars ',' l_exp	{ $$=addarg($1, &$3, L_VAL); }
	| func_pars ',' s_exp	{ $$=addarg($1, &$3, S_VAL); }
;

/* End of grammar */
%%

void gmcal_init_script (gmcal_func_init *gmcal_funcs)
{
  int i,j;
  symrec *ptr;
  char *dummy;
  char *par;
  int type=0;
  
  sym_table = (symrec *) 0;
  
  for (i = 0; gmcal_funcs[i].fname != 0; i++)
    {
      j=0;
      switch (gmcal_funcs[i].type) {
        case GMCAL_SCRIPT_FLOAT: type=D_FNC;
				 break;
	case GMCAL_SCRIPT_INT:   type=L_FNC;
				 break;
	case GMCAL_SCRIPT_EXIT:  type=EXIT_FNC;
				 break;
      }
      ptr = putsym (gmcal_funcs[i].fname, type);
      switch (type) {
        case D_FNC:	(void *) (ptr->value.d_fnc) =  gmcal_funcs[i].fnct;
			break;
        case L_FNC:	(void *) (ptr->value.l_fnc) =  gmcal_funcs[i].fnct;
			break;
        case EXIT_FNC:	(void *) (ptr->value.v_fnc) =  gmcal_funcs[i].fnct;
			break;
      }
      ptr->no_args=gmcal_funcs[i].no_args;
      ptr->par_list=gmcal_funcs[i].par_list;
      ptr->parameters=(int *) calloc(43,sizeof(int));
      if (gmcal_funcs[i].no_args) {
        dummy=(char *) calloc(strlen(gmcal_funcs[i].parameters)+1,sizeof(char));
        strcpy(dummy, gmcal_funcs[i].parameters);
        par=strtok(dummy,",");
        while(par){
          if (par[0]=='i' && !strncmp(par,"int",3))
            ptr->parameters[j]=L_VAL;
          else if (par[0]=='f' && !strncmp(par,"float",5))
            ptr->parameters[j]=D_VAL;
          else if (par[0]=='s' && !strncmp(par,"string",6))
            ptr->parameters[j]=S_VAL;
          else 
            ptr->parameters[j]=-1;
          par=strtok(NULL,",");
          j++;
        }
        free((void *) dummy);
      }
    }
}

int check_arg(symrec *fnct, symrec *args) 
{
  int count;
  symrec *arg;
  
  if (!args) {
    if (!fnct->no_args) return 1;
    error("No arguments given for %s(%d args), result undefined\n",fnct->name, fnct->no_args);
    return 0;
  }
  
  if (fnct->no_args!=args->no_args) {
    error("Wrong number of arguments (%d instead of %d) given for %s\n, result undefined", args->no_args, fnct->no_args, fnct->name);
    return 0;
  }
  
  count=args->no_args;
  arg=args;
  
  
  while(count) {
    count--;
    switch(fnct->parameters[count]) {
      case L_VAL: 	
      			switch(arg->type) {
      			  case L_VAL: 	break;
      			  case D_VAL: 	arg->value.l_var=(int) arg->value.d_var;
      			  		break;
      			  default:	arg->value.l_var=(int) 0;
      			  		warning("Argument %d unsuitable for %s: should be \"int\", replaced by 0\n", count+1, fnct->name);
      			}
      			arg->type=L_VAL;
      			break;
      case D_VAL: 	
      			switch(arg->type) {
      			  case D_VAL: 	break;
      			  case L_VAL: 	arg->value.d_var=(double) arg->value.l_var;
      			  		break;
      			  default:	arg->value.d_var=(double) 0;
      			  		warning("Argument %d unsuitable for %s: should be \"double\", replaced by 0.0\n", count+1, fnct->name);
      			}
      			arg->type=D_VAL;
      			break;
      case S_VAL: 	switch(arg->type) {
      			  case S_VAL: 	break;
      			  default:	arg->value.s_var=(char *) NULL;
      			  		warning("Argument %d unsuitable for %s should be \"string\", replaced by \"undefined\"\n", count+1, fnct->name);
      			}
      			arg->type=S_VAL;
      			break;
      default:		error("Function has not been initialized correctly. Result is undefined.\n");
      			return 0;
    }
    arg=arg->next;
    
  }

  return 1;
}

symrec *addarg(symrec *table, void *value, int type)
{
  symrec *ptr;
  ptr=(symrec *) malloc(sizeof(symrec));
  ptr->name=NULL;
  ptr->type=type;
  ptr->next=table;
  switch(type) {
    case D_VAL: ptr->value.d_var=*((double *) value);
    		break;
    case L_VAL: ptr->value.l_var=*((int *) value);
    		break;
    case S_VAL: ptr->value.s_var=(char *) malloc((strlen(*((char **) value))+1)*sizeof(char));
    		strcpy(ptr->value.s_var,*((char **) value));
    		printf("adding string %s\n",ptr->value.s_var);
    		break;
    default:	warning("Unknown type as function argument");
  }
  ptr->no_args=table->no_args+1;
  return ptr;
}


symrec *newarg(void *value, int type)
{
  symrec *ptr;
  ptr=(symrec *) malloc(sizeof(symrec));
  ptr->name=NULL;
  ptr->type=type;
  ptr->next=(symrec *) 0;
  switch(type) {
    case D_VAL: ptr->value.d_var=*((double *) value);
    		break;
    case L_VAL: ptr->value.l_var=*((int *) value);
    		break;
    case S_VAL: ptr->value.s_var=(char *) malloc((strlen(*((char **) value))+1)*sizeof(char));
    		strcpy(ptr->value.s_var,*((char **) value));
    		printf("adding string %s\n",ptr->value.s_var);
    		break;
    default:	warning("Unknown type as function argument");
  }
  ptr->no_args=1;
  return ptr;
}

#include <ctype.h>
static void yyerror (char *s)  /* Called by yyparse on error */
{
  error("%s\n", s);
}

static int yylex ()
{
  int c;
  int buffer;

  /* Ignore whitespace, get first nonwhite character.  */
  while ((c = getchar ()) == ' ' || c == '\t');

  if (c == EOF)
    return 0;
  /* Char starts a number => parse the number.         */
  if (isdigit(c)) {
    ungetc(c, stdin);
    scanf ("%d", &yylval.l_val);
    c=getchar();
    if (c=='.') {
      ungetc('.', stdin);
      ungetc('0', stdin);
      buffer=yylval.l_val;
      scanf ("%lf", &yylval.d_val);
      yylval.d_val+=buffer;
      return D_VAL;
    } else {
      ungetc(c, stdin);
    }
    return L_VAL;
  }
  if (c == '.' && isdigit(c=getchar())) {
    ungetc (c, stdin);
    ungetc ('.', stdin);
    scanf ("%lf", &yylval.d_val);
    return D_VAL;
  }

  if (c=='\"') {
    char *symbuf = 0;
    int length = 0;
    int i;

    c=getchar();
    /* Initially make the buffer int enough
         for a 40-character symbol name.  */
    if (length == 0)
      length = 40, symbuf = (char *)malloc (length + 1);
    i = 0;
    do {
      /* If buffer is full, make it bigger.        */
      if (i == length)
      {
        length *= 2;
        symbuf = (char *)realloc (symbuf, length + 1);
      }
      /* Add this character to the buffer.         */
      symbuf[i++] = c;
      /* Get another character.                    */
      if (c=='\\') {
        c=getchar();
        switch (c) {
          case  't': symbuf[i-1]='\t'; break;
          case  'n': symbuf[i-1]=("\n")[0]; break;
          default  : symbuf[i-1]=c;
        }
      }
      c = getchar ();
    } while (c != EOF && c!='\n' && c!='\"');
    symbuf[i]='\0';
    yylval.s_val=symbuf;
    return S_VAL;
  }

  /* Char starts an identifier => read the name.       */
  if (isalpha (c))
    {
      symrec *s;
      char *symbuf = 0;
      int length = 0;
      int i;
      /* Initially make the buffer int enough
         for a 40-character symbol name.  */
      if (length == 0)
        length = 40, symbuf = (char *)malloc (length + 1);

      i = 0;
      do
        {
          /* If buffer is full, make it bigger.        */
          if (i == length)
            {
              length *= 2;
              symbuf = (char *)realloc (symbuf, length + 1);
            }
          /* Add this character to the buffer.         */
          symbuf[i++] = c;
          /* Get another character.                    */
          c = getchar ();
        }
      while (c != EOF && isalnum (c));

      ungetc (c, stdin);
      symbuf[i] = '\0';
      s = getsym (symbuf);
      if (s == 0)
      {
        s = putsym (symbuf, L_VAR);
        yylval.ptr = s;
        return s->type;
      } else {
        yylval.ptr = s;
        return s->type;
      }
    }

  /* Any other character is a token by itself.        */
  return c;
}

int gmcal_commandList(symrec *params) 
{
  symrec *command=sym_table;
  
  printf("-------------------- CommandList ---------------------------\n");
  
  while(command) {
    switch (command->type) {
      case D_FNC: 	printf("float %s(%s)\n",command->name, command->par_list);
      			break;
      case L_FNC: 	printf("int   %s(%s)\n",command->name, command->par_list);
      			break;
      case S_FNC: 	printf("string %s(%s)\n",command->name, command->par_list);
      			break;
    }
    command=command->next;
  }
  
  return 0;
}

