#ifndef __GMCAL_SCRIPT_MATRIX_H__
#define __GMCAL_SCRIPT_MATRIX_H__

#include <script.h>

gmcal_data *first_data;
gmcal_data *last_data;

enum {
  GMCAL_SCRIPT_ASCII=1,
  GMCAL_SCRIPT_BIN,
  GMCAL_SCRIPT_BIN_VECTOR,
  GMCAL_SCRIPT_BMP,
  GMCAL_SCRIPT_SPEC,
  GMCAL_SCRIPT_OMA4,
  GMCAL_SCRIPT_ABEL_SYMMETRICAL,
  GMCAL_SCRIPT_ABEL_TRANSFORM,
  GMCAL_SCRIPT_ABEL_MATRIX_INVERSE,
  GMCAL_SCRIPT_ABEL_SPLINE_REAL_INVERSE,
  GMCAL_SCRIPT_ABEL_FOURIER_REAL_INVERSE,
  GMCAL_SCRIPT_ABEL_BACK_DISTRIBUTION_INVERSE,
  GMCAL_SCRIPT_ABEL_AVERAGE_BACK_DISTRIBUTION_INVERSE,
  GMCAL_SCRIPT_ABEL_PRE_CALCULATED_AVERAGE_BACK_DISTRIBUTION_INVERSE
};

int gmcal_script_new_matrix(int dimX, int dimY, float minX, float maxX, float minY, float maxY, float value);
int gmcal_script_get_matrix_attributes(int index, int *dimX, int *dimY, float *minX, float *maxX, float *minY, float *maxY);
int gmcal_script_destroy_matrix(int index);
int gmcal_script_destroy_all_matrices();
int gmcal_script_set_dispersion_x(int index, float min, float max);
int gmcal_script_set_matrix_entry(int index, int row, int col, float value);
int gmcal_script_set_matrix_info(int index, char *info);
int gmcal_script_copy_matrix(int index);
int gmcal_script_write_matrix(int index, char *path, int type);
int gmcal_script_write_row_as_star(int index, int row, char *path);
int gmcal_script_write_gif(int index, char *path, char *model, int maxColors, int mode);
int gmcal_script_write_all_gif(char *path, char *model, int maxColors, int mode, int imagesPerRow);
int gmcal_script_load_matrix(char *path, int type);
int gmcal_script_create_abel_copy(int index, int type, int iterations);
int gmcal_script_create_transposed_matrix(int index);
int gmcal_script_add_matrix_to_matrix(int index, int index2, float value);
int gmcal_script_add_number_to_matrix(int index, float value);
int gmcal_script_multiply_matrix_by_matrix(int index, int index2, float value);
int gmcal_script_multiply_matrix_by_number(int index, float value);
int gmcal_script_divide_matrix_by_matrix(int index, int index2, float value);
int gmcal_script_divide_matrix_by_number(int index, float value);
int gmcal_script_create_copy_with_normed_rows(int index);
int gmcal_script_create_copy_with_normed_cols(int index);
int gmcal_script_create_copy_with_normed_matrix(int index);
int gmcal_script_row_polynom(int index, int row, double a0, double a1, double a2, double a3, double a4, double a5);
int gmcal_script_col_polynom(int index, int col, double a0, double a1, double a2, double a3, double a4, double a5);
int gmcal_script_flip_matrix_rows(int index);
int gmcal_script_flip_matrix_cols(int index);
int gmcal_script_create_resized_matrix(int index, int newX, int newY);
int gmcal_script_create_planck_from_x(int index, double temp);
int gmcal_script_create_spline_from_x_and_row(int index, int row, float minX, float maxX, int steps);
int gmcal_script_create_matrix_from_row(int index, int row, int count);
int gmcal_script_set_dispersion_x_from_row(int target, int index, int row);
int gmcal_script_create_matrix_cut(int index, int left, int top, int width, int height);

#endif /* __GMCAL_SCRIPT_MATRIX_H__ */
