#include <stdio.h>
#include <gfit.h>
#include <gfit_math.h>
#include <gfit_chi.h>
#include <gtk/gtkdatabox.h>
#include <gtk/gtkengine.h>

static gint gfit_chi_function(GFitChiSquare *trans, gfloat *p);
static gint gfit_chi_sub(gint *itest, gfloat *x, gfloat *f, gint limit, gfloat *absacc, gfloat relacc, gfloat *xstep);
static gint gfit_chiquad(GFitChiSquare *trans);

gint gfit_chi(gfit_data *gfit)
{
  GtkEngineVar *vars;
  gint max_vars;
  GFitChiSquare trans;
  gint i;
  
  if (!gfit) return FALSE;
  if (!gfit->engine) return FALSE;
  if (!GTK_IS_ENGINE(gfit->engine)) return FALSE;
  if (!gfit->box) return FALSE;
  if (!GTK_IS_DATABOX(gfit->box)) return FALSE;
  if (!(GTK_DATABOX(gfit->box)->data.dimX>0)) return FALSE;
  if (!(GTK_DATABOX(gfit->box)->data.dimY==2)) return FALSE;
  if (!((max_vars=gtk_engine_get_variables(GTK_ENGINE(gfit->engine), &vars))!=-1)) {
    gfit_warning("No fit engine loaded...\n");
    return FALSE;
  }

  trans.user_num_pars=max_vars;
  trans.user_pars=(gfloat *) g_malloc(trans.user_num_pars*sizeof(gfloat));
  trans.user_varianz=(gfloat *) g_malloc(trans.user_num_pars*sizeof(gfloat));

  for (i=0; i<max_vars; i++) {
    trans.user_pars[i]=vars[i].value;
    trans.user_varianz[i]=(vars[i].user_data==-10)? 0 : vars[i].user_data;
  }

  trans.user_acc=1.;
  trans.user_step=1000.;
  trans.user_limit=2000;
  trans.user_num_points=GTK_DATABOX(gfit->box)->data.dimX;

  trans.user_x=GTK_DATABOX(gfit->box)->data.X;
  trans.user_y=GTK_DATABOX(gfit->box)->data.Z[0];
  trans.user_z=(gfloat *) g_malloc(trans.user_num_points*sizeof(gfloat));
  trans.user_data=(gpointer) gfit;
  trans.res_pars=NULL;
  trans.res_stat_errs=NULL;
  trans.res_chi_square=0;
  trans.res_steps=0;
  
  for (i=0; i<trans.user_num_points; i++) {
    trans.user_z[i]=0;
  }
  
  printf("gfit_chi...\n");
  
  gfit_chiquad(&trans);
  
  printf("Chi-Fehler-Code: %d, %d, %d\n", trans.res_merr[0], trans.res_merr[1], trans.res_merr[2]);
  printf("To get an idea of what that message could mean: gfit_chi.h\n");

  for (i=0; i<trans.user_num_pars; i++) {
   vars[i].value=trans.res_pars[i];
   printf("%s: %g, %g\n", vars[i].desc, trans.res_pars[i], trans.user_varianz[i]);
  }

  return TRUE;
  
}

static gint
gfit_chi_function(GFitChiSquare *trans, gfloat *p)
{
  GtkEngineVar *vars;
  gfit_data *gfit=(gfit_data *) trans->user_data;
  gint max_vars;
  gdouble *x;
  gdouble *y;
  gint i, j;

  if (!((max_vars=gtk_engine_get_variables(GTK_ENGINE(gfit->engine), &vars))!=-1)) {
    gfit_warning("No fit engine loaded...\n");
    return FALSE;
  }

  x=gtk_engine_get_value_pointer(GTK_ENGINE(gfit->engine), 
  			gtk_engine_get_value_index_by_name(GTK_ENGINE(gfit->engine), "X"));
  y=gtk_engine_get_value_pointer(GTK_ENGINE(gfit->engine), 
  			gtk_engine_get_value_index_by_name(GTK_ENGINE(gfit->engine), "Y"));

  if (!x || !y) {
    gfit_warning("Fit Function MUST contain X and Y (variable names are case sensitiv!)");
    return FALSE;
  }

  j=0;
  for (i=0; i<trans->user_num_pars; i++) {
    float factor;
    if (trans->user_varianz[i]<0) vars[i].value=p[j++];
    if (trans->user_varianz[i]>0) {
      factor=(trans->user_pars[i]>0)? trans->user_varianz[i]*trans->user_pars[i]/100.0 : -trans->user_varianz[i]*trans->user_pars[i]/100.0;
      vars[i].value=trans->user_pars[i]+factor*sin(p[j++]);
    }
  }

  for (i=0; i<trans->user_num_points; i++) {
    *x=trans->user_x[i];
    gtk_engine_run(GTK_ENGINE(gfit->engine));
    trans->user_z[i]=*y-trans->user_y[i];
  }
  
  return TRUE;
}

static gint
gfit_chiquad(GFitChiSquare *trans)
{
    /* workspace minimum: (N+1)*M + 3*N + 1 (float values) */
    gfloat *work=(gfloat *) g_malloc0(((trans->user_num_pars+1)*trans->user_num_points+3*trans->user_num_pars+1)*sizeof(gfloat));
    gfloat *a=(gfloat *) g_malloc(trans->user_num_pars*sizeof(gfloat));
    gfloat *p=(gfloat *) g_malloc(trans->user_num_pars*sizeof(gfloat));

    gint i__1=0, i__2=0, i__3=0;
    gfloat r__1=0, r__2=0;

    /* Local variables */
    gint n=0;
    gfloat fsec=0;
    gfloat chiq=0, fmin=0;
    gint iljl=0, kini=0;
    gint iinv=0, kinv=0;
    gfloat b=0;
    gint i=0, j=0, k=0, l=0;
    gfloat sigma=0;
    gint icont=0;
    gfloat astep=0;
    gint iless=0;
    gfloat dstep=0;
    gint i1=0, i2=0, k1=0, nplus=0;
    gfloat xstep=0, bb=0, fa=0, fc=0;
    gint i12=0, ie=0, ig=0, mc=0, ii=0;
    gfloat dm=0;
    gint jj=0, il=0, kk=0, jl=0;
    gfloat av=0;
    gint kl=0, jn=0;
    gfloat xc=0;
    gint nn=0, is=0, it=0, in=0;
    gfloat aw=0;
    gint kn=0, nk=0, mv=0;
    gfloat xl=0;
    gint igamax=0, mdsich=0;
    gint incinp=0, incinv=0, i222=0, i123=0, i2j=0, ki1=0, kstore=0, mplusn=0;
    gfloat xi1=0, acc=0;
    gint itc=0, lil=0, lim=0, iip=0, jjp=0, ljl=0, iss=0, kst=0;
    gfloat yyy=0;
    gint i2il=0, i2jl=0, kni1=0;
    gfloat dec10=0;

/* 
 * MERR : Drei-Elemente-Array. Enthaelt -nach- RETURN Fehlermeldung:   *
 * > MERR(1)=0 > Kein Fehler ermittelt.                                *
 * > MERR(1)=1 > Genauigkeit A(I) unerreichbar oder Chiquadrat ist     *
 *               keine Funktion von P(I), mit I=MERR(2).               *
 * > MERR(1)=2 > Zwei Parameter sind -nicht- unabhaengig voneinander:  *
 *               P(I) = f( P(K) ), mit I=MERR(2) und K=MERR(3).        *
 * > MERR(1)=3 > Abbruch der Anpassung, das Chiquadrat-Minimum wurde   *
 *               nach MERR(2)-Iterationen nicht erreicht.(---> LIMIT)  *
 * > MERR(1)=4 > Waehrend der Rechnung sind Rundungsfehler aufgetreten *
 * > MERR(1)=5 > Waehrend der Rechnung kam es zu Fliesskomma-Ueberlauf *
 * > MERR(1)=6 > Kontrollierter Abbruch (negatives M aus "CHIFUN") .   *
 * > MERR(1)=7 > Fehler im CHIQUAD-Aufruf oder in der Modellfunktion.  *
 */


  trans->res_pars=(gfloat *) g_malloc(trans->user_num_pars*sizeof(float));
  trans->res_stat_errs=(gfloat *) g_malloc(trans->user_num_pars*sizeof(float));
  n=0;
  for (i=0; i<trans->user_num_pars; i++) {
    if (trans->user_varianz[i]==0) continue;
    if (trans->user_varianz[i]<0 || ((trans->user_varianz[i]>0) && (trans->user_pars[i]==0))) p[n]=trans->user_pars[i];
    else p[n]=0.0;
    if (trans->user_varianz[i]<0) a[n]=(trans->user_pars[i]!=0)? trans->user_pars[i]*trans->user_acc/100.0 : trans->user_acc;
    else a[n]=PI*trans->user_acc/100.0;

    printf("trans->user_varianz[i]=%g, p[n]=%g, trans->user_acc=%g, a[i]=%g\n", trans->user_varianz[i], p[n], trans->user_acc, a[n]);

    n++;
  }


    /* Function Body */
    mdsich = trans->user_num_points;
    trans->res_merr[0] = GFIT_CHI_OK;
    trans->res_merr[1] = 0;
    trans->res_merr[2] = 0;
    lim = ABS(trans->user_limit);
    trans->user_limit = -lim;
    ig = -1;
L1:
    itc = 0;
    mc = 0;
    if ((i__1 = n - 1) < 0) {
	goto L152;
    } else if (i__1 == 0) {
	goto L2;
    } else {
	goto L5;
    }
L2:
    xstep = trans->user_step * a[0];
    acc = a[0];
    b = (float)0.;
    work[0] = (float)0.;
    ie = 4;
    ii = lim - 1;
    iss = 2;
L4:
    xc = p[0];
/* ---------------------------------------------------------- CHIQUAD --- 
*/
    gfit_chi_sub(&iss, &xc, &chiq, ii, &acc, b, &xstep);
/* ---------------------------------------------------------- CHIQUAD --- 
*/
    p[0] = xc;
    goto L50;
L5:
    iss = 1;
    mplusn = trans->user_num_points+ n;
    kst = n + mplusn;
    nplus = n + 1;
    kinv = nplus * (mplusn + 1);
    kstore = kinv - mplusn - 1;
    ie = 1;
    goto L50;
L6:
    nn = n + n;
    k = nn;
    i__1 = trans->user_num_points;
    for (i = 1; i <= i__1; i++) {
	k++;
	work[k-1] = trans->user_z[i-1];
    }
    switch ((int)iss) {
	case 1:  goto L8;
	case 2:  goto L30;
    }
L8:
    iinv = 2;
    k = kst;
    i = 1;
L9:
    if (ig == 0) {
	goto L11;
    }
    xc = p[i-1];
    p[i-1] += a[i-1];
    ie = 2;
    goto L50;
L10:
    p[i-1] = xc;
    switch ((int)iss) {
	case 1:  goto L11;
	case 2:  goto L30;
    }
L11:
    i__1 = n;
    for (j = 1; j <= i__1; j++) {
	k++;
	work[k-1] = (float)0.;
	work[j-1] = (float)0.;
    }
    sigma = (float)0.;
    kk = nn;
    il = i * (i - 1) / 2;
    i__1 = trans->user_num_points;
    for (j = 1; j <= i__1; j++) {
	kk++;
	trans->user_z[j-1] -= work[kk-1];
	dm = trans->user_z[j-1] * trans->user_z[j-1];
	jl = il + j;
	if (j > i) {
	    jl = i + j * (j - 1) / 2;
	}
	i2jl = i2 + jl;
	i222 = i2 + j * (j + 1) / 2;
	if (ig == 0) {
/* Computing 2nd power */
	    r__1 = work[i2jl-1];
	    dm = r__1 * r__1 * av / work[i222-1];
	}
	sigma += dm;
    }
    if (sigma <= (float)0.) {
	goto L85;
    } else {
	goto L14;
    }
L14:
    sigma = (float)1. / sqrt(sigma);
    j = k - n + i;
    work[j-1] = sigma * a[i-1];
    if (ig == 0) {
	work[j-1] = sigma;
    }
    i__1 = trans->user_num_points;
    for (j = 1; j <= i__1; j++) {
	k++;
	dm = trans->user_z[j-1];
	jl = il + j;
	if (j > i) {
	    jl = i + j * (j - 1) / 2;
	}
	i2jl = i2 + jl;
	i222 = i2 + j * (j + 1) / 2;
	if (ig == 0) {
	    dm = work[i2jl-1] * sqrt(av / work[i222-1]);
	}
	work[k-1] = sigma * dm;
	kk = nn + j;
	i__2 = i;
	for (ii = 1; ii <= i__2; ii++) {
	    kk += mplusn;
	    work[ii-1] += work[kk-1] * work[k-1];
	    if (j - trans->user_num_points>= 0) {
		goto L15;
	    } else {
		goto L17;
	    }
L15:
	    if (i - ii <= 0) {
		goto L17;
	    } else {
		goto L16;
	    }
L16:
	    if (work[ii-1] - (float)1. >= (float)0.) {
		goto L87;
	    } else {
		goto L17;
	    }
L17:
	    ;
	}
    }
    iless = i - 1;
    igamax = n + i - 1;
    incinv = n - iless;
    incinp = incinv + 1;
    if (iless <= 0) {
	goto L18;
    } else {
	goto L19;
    }
L18:
    work[kinv-1] = (float)1.;
    goto L27;
L19:
    b = (float)1.;
    i__2 = igamax;
    for (j = nplus; j <= i__2; j++) {
	work[j-1] = (float)0.;
    }
    kk = kinv;
    i__2 = iless;
    for (ii = 1; ii <= i__2; ii++) {
	iip = ii + n;
	work[iip-1] += work[kk-1] * work[ii-1];
	jl = ii + 1;
	if (jl - iless <= 0) {
	    goto L21;
	} else {
	    goto L23;
	}
L21:
	i__1 = iless;
	for (jj = jl; jj <= i__1; jj++) {
	    kk++;
	    jjp = jj + n;
	    work[iip-1] += work[kk-1] * work[jj-1];
	    work[jjp-1] += work[kk-1] * work[ii-1];
	}
L23:
	b -= work[ii-1] * work[iip-1];
	kk += incinp;
    }
    if (b != (float)0.) {
	b = (float)1. / b;
    }
    if (b == (float)0.) {
	trans->res_merr[0] = GFIT_CHI_FLOAT_EXCEPTION;
    }
    if (b == (float)0.) {
	goto L1000;
    }
    kk = kinv;
    i__2 = igamax;
    for (ii = nplus; ii <= i__2; ii++) {
	bb = -(double)b * work[ii-1];
	i__1 = igamax;
	for (jj = ii; jj <= i__1; jj++) {
	    work[kk-1] -= bb * work[jj-1];
	    kk++;
	}
	work[kk-1] = bb;
	kk += incinv;
    }
    work[kk-1] = b;
L27:
    switch ((int)iinv) {
	case 1:  goto L102;
	case 2:  goto L28;
    }
L28:
    i++;
    if (i - n <= 0) {
	goto L9;
    } else {
	goto L29;
    }
L29:
    iinv = 1;
L30:
    chiq = (float)0.;
    kl = nn;
    i__2 = trans->user_num_points;
    for (i = 1; i <= i__2; i++) {
	kl++;
	trans->user_z[i-1] = work[kl-1];
	chiq += work[kl-1] * work[kl-1];
    }
    switch ((int)iss) {
	case 1:  goto L95;
	case 2:  goto L109;
    }
L32:
    itc++;
    k = n;
    kk = kst;
    i__2 = n;
    for (i = 1; i <= i__2; i++) {
	k++;
	work[k-1] = (float)0.;
	kk += n;
	work[i-1] = (float)0.;
	i__1 = trans->user_num_points;
	for (j = 1; j <= i__1; j++) {
	    kk++;
	    work[i-1] += work[kk-1] * trans->user_z[j-1];
	}
    }
    dm = (float)0.;
    k = kinv;
    i__1 = n;
    for (ii = 1; ii <= i__1; ii++) {
	iip = ii + n;
	work[iip-1] += work[k-1] * work[ii-1];
	jl = ii + 1;
	if (jl - n <= 0) {
	    goto L34;
	} else {
	    goto L36;
	}
L34:
	i__2 = n;
	for (jj = jl; jj <= i__2; jj++) {
	    jjp = jj + n;
	    k++;
	    work[iip-1] += work[k-1] * work[jj-1];
	    work[jjp-1] += work[k-1] * work[ii-1];
	}
	k++;
L36:
	if (dm - (r__1 = work[ii-1] * work[iip-1], ABS(r__1)) <= (float)0.) {
	    goto L37;
	} else {
	    goto L38;
	}
L37:
	dm = (r__1 = work[ii-1] * work[iip-1], ABS(r__1));
	kl = ii;
L38:
	;
    }
    ii = n + mplusn * kl;
    dstep = (float).1;
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	jl = n + i;
	work[i-1] = (float)0.;
	i__2 = nn;
	for (j = nplus; j <= i__2; j++) {
	    jl += mplusn;
	    work[i-1] += work[j-1] * work[jl-1];
	}
	ii++;
	work[ii-1] = work[jl-1];
	work[jl-1] = p[i-1];
	if ((r__1 = a[i-1] * dstep, ABS(r__1)) - (r__2 = work[i-1], ABS(r__2)) 
		<= (float)0.) {
	    goto L40;
	} else {
	    goto L41;
	}
L40:
	dstep = (r__1 = work[i-1] / a[i-1], ABS(r__1));
L41:
	;
    }
    if ((float)1. - dstep <= (float)0.) {
	goto L43;
    } else {
	goto L42;
    }
L42:
    icont = 2;
L43:
    i__1 = trans->user_num_points;
    for (i = 1; i <= i__1; i++) {
	ii++;
	jl++;
	work[ii-1] = work[jl-1];
	work[jl-1] = trans->user_z[i-1];
    }
    fc = chiq;
    acc = (float).1 / dstep;
    ie = 3;
    is = 3;
    it = 3;
    xc = (float)0.;
    xl = (float)0.;
    xstep = -(double)(r__1 = trans->user_step / dstep, ABS(r__1));
    if (xstep + (float).5 >= (float)0.) {
	goto L46;
    } else {
	goto L45;
    }
L45:
    xstep = (float)-.5;
/* ---------------------------------------------------------- CHIQUAD --- 
*/
L46:
    gfit_chi_sub(&it, &xc, &fc, 20, &acc, .1 , &xstep);
/* ---------------------------------------------------------- CHIQUAD --- 
*/
    switch ((int)it) {
	case 1:  goto L47;
	case 2:  goto L67;
	case 3:  goto L93;
	case 4:  goto L93;
    }
L47:
    switch ((int)iss) {
	case 1:  goto L48;
	case 2:  goto L67;
    }
L48:
    xl = xc - xl;
    i__1 = n;
    for (j = 1; j <= i__1; j++) {
	p[j-1] += xl * work[j-1];
    }
    xl = xc;
L50:
    mc++;
    if (ig == 0) {
	goto L132;
    }
/* ---------------------------------------------------------- CHIQUAD --- 
*/
    gfit_chi_function(trans, p);
/* ---------------------------------------------------------- CHIQUAD --- 
*/
    if (trans->user_num_points< 0) {
	goto L90;
    } else if (trans->user_num_points== 0) {
        trans->res_merr[0]=GFIT_CHI_STOPPED_BY_USER;
	goto L152;
    } else {
	goto L51;
    }
L51:
    if (mc - lim >= 0) {
	goto L91;
    } else {
	goto L52;
    }
L52:
    switch ((int)ie) {
	case 1:  goto L6;
	case 2:  goto L10;
	case 3:  goto L53;
	case 4:  goto L104;
	case 5:  goto L122;
    }
L53:
    fc = (float)0.;
    i__1 = trans->user_num_points;
    for (j = 1; j <= i__1; j++) {
	b = trans->user_z[j-1];
	fc += b * b;
    }
    if (fc == (float)0.) {
	goto L103;
    }
    switch ((int)is) {
	case 1:  goto L58;
	case 2:  goto L58;
	case 3:  goto L55;
    }
L55:
    k = n;
    if ((r__1 = fc - chiq) < (float)0.) {
	goto L56;
    } else if (r__1 == 0) {
	goto L46;
    } else {
	goto L57;
    }
L56:
    is = 2;
    fmin = fc;
    fsec = chiq;
    goto L64;
L57:
    is = 1;
    fmin = chiq;
    fsec = fc;
    goto L64;
L58:
    if (fc - fsec >= (float)0.) {
	goto L46;
    } else {
	goto L59;
    }
L59:
    k = kstore;
    switch ((int)is) {
	case 1:  goto L60;
	case 2:  goto L61;
    }
L60:
    k = n;
L61:
    if ((r__1 = fc - fmin) < (float)0.) {
	goto L63;
    } else if (r__1 == 0) {
	goto L46;
    } else {
	goto L62;
    }
L62:
    fsec = fc;
    goto L64;
L63:
    is = 3 - is;
    fsec = fmin;
    fmin = fc;
L64:
    i__1 = n;
    for (j = 1; j <= i__1; j++) {
	k++;
	work[k-1] = p[j-1];
    }
    i__1 = trans->user_num_points;
    for (j = 1; j <= i__1; j++) {
	k++;
	work[k-1] = trans->user_z[j-1];
    }
    goto L46;
L67:
    k = kstore;
    kk = n;
    switch ((int)is) {
	case 1:  goto L69;
	case 2:  goto L68;
	case 3:  goto L69;
    }
L68:
    k = n;
    kk = kstore;
L69:
    sigma = (float)0.;
    dm = (float)0.;
    jj = kstore;
    i__1 = n;
    for (j = 1; j <= i__1; j++) {
	k++;
	kk++;
	jj++;
	p[j-1] = work[k-1];
	work[jj-1] = work[k-1] - work[kk-1];
    }
    i__1 = trans->user_num_points;
    for (j = 1; j <= i__1; j++) {
	k++;
	kk++;
	jj++;
	trans->user_z[j-1] = work[k-1];
	work[jj-1] = work[k-1] - work[kk-1];
	sigma += work[jj-1] * work[jj-1];
	dm += work[jj-1] * trans->user_z[j-1];
    }
    switch ((int)iss) {
	case 1:  goto L72;
	case 2:  goto L104;
    }
L72:
    j = kinv;
    kk = nplus - kl;
    i__1 = kl;
    for (i = 1; i <= i__1; i++) {
	k = j + kl - i;
	j = k + kk;
	work[i-1] = work[k-1];
	work[k-1] = work[j - 1-1];
    }
    if (kl - n >= 0) {
	goto L76;
    } else {
	goto L74;
    }
L74:
    kl++;
    jj = k;
    i__1 = n;
    for (i = kl; i <= i__1; i++) {
	k++;
	j = j + nplus - i;
	work[i-1] = work[k-1];
	work[k-1] = work[j - 1-1];
    }
    work[jj-1] = work[k-1];
    b = (float)1. / work[kl - 1-1];
    work[kl - 1-1] = work[n-1];
    goto L77;
L76:
    b = (float)1. / work[n-1];
L77:
    k = kinv;
    i__1 = iless;
    for (i = 1; i <= i__1; i++) {
	bb = b * work[i-1];
	i__2 = iless;
	for (j = i; j <= i__2; j++) {
	    work[k-1] -= bb * work[j-1];
	    k++;
	}
	k++;
    }
    if (chiq - fmin <= (float)0.) {
	goto L82;
    } else {
	goto L80;
    }
L80:
    chiq = fmin;
    dstep = ABS(xc) * dstep;
    if (dstep - (float)1. <= (float)0.) {
	goto L82;
    } else {
	goto L81;
    }
L81:
    icont = 1;
L82:
    xl = -(double)dm / fmin;
    dec10 = (r__1 = sigma + dm * xl, ABS(r__1));
    if (dec10 < (float)5e-18) {
	dec10 = (float)5e-18;
    }
    sigma = (float)1. / sqrt(dec10);
    k = kstore;
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	k++;
	work[k-1] = sigma * work[k-1];
	work[i-1] = (float)0.;
    }
    i__1 = trans->user_num_points;
    for (i = 1; i <= i__1; i++) {
	k++;
	work[k-1] = sigma * (work[k-1] + xl * trans->user_z[i-1]);
	kk = nn + i;
	i__2 = n;
	for (j = 1; j <= i__2; j++) {
	    kk += mplusn;
	    work[j-1] += work[kk-1] * work[k-1];
	}
    }
    goto L19;
L85:
    trans->res_merr[0] = GFIT_CHI_PRECISION_UNREACHABLE;
    trans->res_merr[1] = i;
    work[0] = (float)1.;
    goto L89;
L87:
    trans->res_merr[0] = GFIT_CHI_PARAMETERS_NOT_INDEPENDEND;
    trans->res_merr[1] = ii;
    trans->res_merr[2] = i;
    work[0] = (float)2.;
L89:
    ig = 1;
    iss = 2;
    goto L30;
L90:
    trans->user_num_points= -(trans->user_num_points);
    mc = -mc;
    ig = 1;
L91:
    trans->res_merr[0] = GFIT_CHI_MORE_ITERATIONS_NEEDED;
    trans->res_merr[1] = mc;
    work[0] = (float)3.;
    iss = 2;
    goto L52;
L93:
    trans->res_merr[0] = GFIT_CHI_ROUND_OFF_ERRORS;
    work[0] = (float)4.;
    iss = 2;
    goto L67;
L95:
    icont = 1;
L102:
    switch ((int)icont) {
	case 1:  goto L32;
	case 2:  goto L103;
    }
L103:
    work[0] = (float)0.;
    iss = 2;
L104:
    chiq = (float)0.;
    i__2 = trans->user_num_points;
    for (j = 1; j <= i__2; j++) {
	chiq += trans->user_z[j-1] * trans->user_z[j-1];
    }
    if (iss == 1) {
	goto L4;
    }
L109:
    if (trans->user_limit > 0) {
	ig = 0;
    }
    if (ig != 0) {
	goto L110;
    } else {
	goto L135;
    }
L110:
    i__2 = trans->user_num_points;
    for (i = 1; i <= i__2; i++) {
	k = n + i;
	work[k-1] = trans->user_z[i-1];
    }
    i__2 = n;
    for (i = 1; i <= i__2; i++) {
	xc = p[i-1];
	p[i-1] += a[i-1];
	mc++;
/* ---------------------------------------------------------- CHIQUAD 
--- */
	gfit_chi_function(trans, p);
/* ---------------------------------------------------------- CHIQUAD 
--- */
	p[i-1] = xc;
	if (trans->user_num_points< 0) {
	    goto L112;
	} else if (trans->user_num_points== 0) {
            trans->res_merr[0]=GFIT_CHI_STOPPED_BY_USER;
	    goto L152;
	} else {
	    goto L113;
	}
L112:
	ie = 5;
	goto L90;
L113:
	i__1 = trans->user_num_points;
	for (j = 1; j <= i__1; j++) {
	    k++;
	    jn = j + n;
	    work[k-1] = (trans->user_z[j-1] - work[jn-1]) / a[i-1];
	}
    }
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	ii = k + 1;
	il = n + i * trans->user_num_points;
	i__2 = i;
	for (j = 1; j <= i__2; j++) {
	    jl = n + j * trans->user_num_points;
	    k++;
	    work[k-1] = (float)0.;
	    i__3 = trans->user_num_points;
	    for (l = 1; l <= i__3; l++) {
		lil = l + il;
		ljl = l + jl;
		work[k-1] += work[lil-1] * work[ljl-1];
	    }
	}
    }
L122:
    i__1 = trans->user_num_points;
    for (i = 1; i <= i__1; i++) {
	in = i + n;
	trans->user_z[i-1] = work[in-1];
    }
    if (ie == 5) {
	goto L152;
    }
    ig = 0;
    i = trans->user_num_points+ n + trans->user_num_points* n;
    il = n * (n * 5 + 11) / 2 + trans->user_num_points+ 2;
    i2 = il;
    jl = i - il;
    k = n * (n + 1) / 2;
    if (jl < 0) {
	goto L124;
    } else if (jl == 0) {
	goto L127;
    } else {
	goto L125;
    }
L124:
    il = il + k + 1;
L125:
    ii = 1;
    if (jl < 0) {
	ii = -1;
    }
    i__1 = k;
    for (i = 1; i <= i__1; i++) {
	il += ii;
	iljl = il + jl;
	work[il-1] = work[iljl-1];
    }
L127:
    k = i2 - trans->user_num_points- n - n;
    k1 = k - 1;
    work[k1-1] = work[0];
    work[k-1] = chiq;
    aw = (float)0.;
    i__1 = trans->user_num_points;
    for (i = 1; i <= i__1; i++) {
	k++;
	aw += trans->user_z[i-1];
	work[k-1] = trans->user_z[i-1];
    }
/* Computing MAX */
    i__1 = trans->user_num_points- n;
    aw = (chiq - aw * aw / (float) (trans->user_num_points)) / (float) MAX(i__1,1);
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	k++;
	work[k-1] = p[i-1];
	kn = k + n;
	work[kn-1] = a[i-1];
    }
    astep = trans->user_step;
    trans->user_step = (float)1e6;
    trans->user_limit = n * 10 + 100;
    mv = trans->user_num_points;
    trans->user_num_points= n;
    i1 = 0;
L130:
    i1++;
    i12 = i1 * (i1 + 1) / 2 + i2;
    av = work[i12-1];
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	i123 = i2 + i * (i + 1) / 2;
	yyy = av * work[i123-1];
	if (yyy == (float)0.) {
	    trans->res_merr[0] = GFIT_CHI_FLOAT_EXCEPTION;
	}
	if (yyy == (float)0.) {
	    goto L1000;
	}
	p[i-1] = (float)0.;
	a[i-1] = (float).5 / sqrt(av * work[i123-1]);
    }
    goto L1;
L132:
    i__1 = n;
    for (j = 1; j <= i__1; j++) {
	sigma = (float)0.;
	jl = j * (j - 1) / 2;
	i__3 = n;
	for (i = 1; i <= i__3; i++) {
	    il = i + jl;
	    if (i > j) {
		il = j + i * (i - 1) / 2;
	    }
	    i2il = i2 + il;
	    sigma += p[i-1] * work[i2il-1];
	}
	if (j == i1) {
	    sigma += (float)-1.;
	}
	i2j = i2 + j * (j + 1) / 2;
	if (work[i2j-1] == (float)0.) {
	    trans->res_merr[0] = GFIT_CHI_FLOAT_EXCEPTION;
	}
	if (work[i2j-1] == (float)0.) {
	    goto L1000;
	}
	trans->user_z[j-1] = sigma * sqrt(av / work[i2j-1]);
    }
    goto L51;
L135:
    k = n * (n * 3 + 6) + mv + 2;
    if (p[i1-1] >= (float)0.) {
	goto L138;
    } else {
	goto L136;
    }
L136:
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	p[i-1] = -(double)p[i-1];
    }
L138:
    ki1 = k + i1;
    work[ki1-1] = sqrt(p[i1-1] * av);
    xi1 = p[i1-1];
    if (xi1 == (float)0.) {
	goto L140;
    }
    fa = sqrt(aw / p[i1-1]);
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	kni1 = k + n * i1 + i;
	work[kni1-1] = fa * p[i-1];
    }
L140:
    if (i1 - n >= 0) {
	goto L141;
    } else {
	goto L130;
    }
L141:
    trans->user_num_points= mv;
    k = n * (n * 3 + 6) + trans->user_num_points+ 2;
L1000:
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	kini = k + i * n + i;
	trans->res_stat_errs[i-1] = work[kini-1];
    }
    work[2] = (float) i2;
    work[3] = (float) k;
    k = i2 - trans->user_num_points- n - n;
    k1 = k - 1;
    work[0] = work[k1-1];
    chiq = work[k-1];
    i__1 = trans->user_num_points;
    for (i = 1; i <= i__1; i++) {
	k++;
	trans->user_z[i-1] = work[k-1];
    }
    i__1 = n;
    for (i = 1; i <= i__1; i++) {
	k++;
	p[i-1] = work[k-1];
	nk = n + k;
	a[i-1] = work[nk-1];
    }
    trans->user_step = astep;
    trans->user_limit = lim;
L152:
    work[1] = chiq;
    trans->res_chi_square = chiq / (float) MAX(trans->user_num_points,1);
/* wegen trans->user_num_points=n bei Fehlermeldung 5 setze zurck: trans->user_num_points=mdsich  22.7.91 F.G. */
    trans->user_num_points= mdsich;

    j=0;
    for (i=0; i<trans->user_num_pars; i++) {
      float factor;
      if (trans->user_varianz[i]==0) trans->res_pars[i]=trans->user_pars[i];
      if (trans->user_varianz[i]<0) trans->res_pars[i]=p[j++];
      if (trans->user_varianz[i]>0) {
        factor=(trans->user_pars[i]>0)? trans->user_varianz[i]*trans->user_pars[i]/100.0 : -trans->user_varianz[i]*trans->user_pars[i]/100.0;
        trans->res_pars[i]=trans->user_pars[i]+factor*sin(p[j++]);
      }
    }
    trans->res_steps=trans->user_limit;
printf("Chi ends here...\n");

    return 0;
}

static gint
gfit_chi_sub(gint *itest, 
       gfloat *x, 
       gfloat *f, 
       gint limit, 
       gfloat *absacc, 
       gfloat relacc, 
       gfloat *xstep)
{
    /* System generated locals */
    float r__1, r__2;

    /* Local variables */
    static int iinc;
    static float xinc, d, da, db, dc, fb, fc, fa;
    static int mc, is;

/* *--------------------------------------------------------- CHIQUAD ---*
 */
/* * Bestimmt das Minimum der Funktion F(X) durch Iteration.             *
 */
/* * ABSACC       : Absolute Genauigkeit                                 *
 */
/* * RELACC       : Relative Genauigkeit                                 *
 */
/* * XSTEP        : Iterationsschrittweite                               *
 */
/* *--------------------------------------------------------- CHIQUAD ---*
 */
    switch ((int)*itest) {
	case 1:  goto L7;
	case 2:  goto L1;
	case 3:  goto L1;
    }
L1:
    is = 6 - *itest;
    *itest = 1;
    iinc = 1;
    xinc = *xstep + *xstep;
    mc = is - 3;
    if (mc <= 0) {
	goto L9;
    } else {
	goto L6;
    }
L2:
    mc++;
    if (limit - mc >= 0) {
	goto L6;
    } else {
	goto L3;
    }
L3:
    *itest = 4;
L4:
    *x = db;
    *f = fb;
    if (fb - fc <= (float)0.) {
	goto L6;
    } else {
	goto L5;
    }
L5:
    *x = dc;
    *f = fc;
L6:
    return 0;
/* ---------------------------------------------------------- CHIQUAD --- 
*/
L7:
    switch ((int)is) {
	case 1:  goto L17;
	case 2:  goto L15;
	case 3:  goto L10;
	case 4:  goto L8;
    }
L8:
    is = 3;
L9:
    dc = *x;
    fc = *f;
    *x += *xstep;
    goto L2;
L10:
    if ((r__1 = fc - *f) < (float)0.) {
	goto L12;
    } else if (r__1 == 0) {
	goto L11;
    } else {
	goto L13;
    }
L11:
    *x += xinc;
    xinc += xinc;
    goto L2;
L12:
    db = *x;
    fb = *f;
    xinc = -(double)xinc;
    goto L14;
L13:
    db = dc;
    fb = fc;
    dc = *x;
    fc = *f;
L14:
    *x = dc + dc - db;
    is = 2;
    goto L2;
L15:
    da = db;
    db = dc;
    fa = fb;
    fb = fc;
L16:
    dc = *x;
    fc = *f;
    goto L27;
L17:
    if (fb - fc >= (float)0.) {
	goto L18;
    } else {
	goto L21;
    }
L18:
    if (*f - fb >= (float)0.) {
	goto L16;
    } else {
	goto L19;
    }
L19:
    fa = fb;
    da = db;
L20:
    fb = *f;
    db = *x;
    goto L27;
L21:
    if (fa - fc <= (float)0.) {
	goto L23;
    } else {
	goto L22;
    }
L22:
    xinc = fa;
    fa = fc;
    fc = xinc;
    xinc = da;
    da = dc;
    dc = xinc;
L23:
    xinc = dc;
    if ((d - db) * (d - dc) >= (float)0.) {
	goto L24;
    } else {
	goto L16;
    }
L24:
    if (*f - fa >= (float)0.) {
	goto L26;
    } else {
	goto L25;
    }
L25:
    fc = fb;
    dc = db;
    goto L20;
L26:
    fa = *f;
    da = *x;
L27:
    if (fb - fc <= (float)0.) {
	goto L28;
    } else {
	goto L29;
    }
L28:
    iinc = 2;
    xinc = dc;
    if (fb - fc != (float)0.) {
	goto L29;
    } else {
	goto L45;
    }
L29:
    if ((da - db) * (da - dc) != (float)0.) {
	goto L30;
    } else {
	goto L31;
    }
L30:
    d = (fa - fb) / (da - db) - (fa - fc) / (da - dc);
    if (d != (float)0.) {
	goto L34;
    } else {
	goto L31;
    }
L31:
    *x = (float)4.;
    if (fb - fc >= (float)0.) {
	goto L32;
    } else {
	goto L33;
    }
L32:
    *x = (float)-4.;
L33:
    *x = db - *x * (dc - db);
    is = 2;
    *itest = 1;
    goto L2;
L34:
    if (d * (db - dc) >= (float)0.) {
	goto L35;
    } else {
	goto L41;
    }
L35:
    d = (db + dc - (fb - fc) / d) * (float).5;
    if ((r__1 = d - *x, ABS(r__1)) - ABS(*absacc) <= (float)0.) {
	goto L37;
    } else {
	goto L36;
    }
L36:
    if ((r__1 = d - *x, ABS(r__1)) - (r__2 = d * relacc, ABS(r__2)) <= (
	    float)0.) {
	goto L37;
    } else {
	goto L38;
    }
L37:
    *itest = 2;
    goto L4;
L38:
    is = 1;
    *x = d;
    if ((r__1 = (da - dc) * (dc - d)) < (float)0.) {
	goto L2;
    } else if (r__1 == 0) {
	goto L46;
    } else {
	goto L39;
    }
L39:
    is = 2;
    switch ((int)iinc) {
	case 1:  goto L40;
	case 2:  goto L43;
    }
L40:
    if (ABS(xinc) - (r__1 = dc - d, ABS(r__1)) >= (float)0.) {
	goto L2;
    } else {
	goto L42;
    }
L41:
    is = 2;
    switch ((int)iinc) {
	case 1:  goto L42;
	case 2:  goto L44;
    }
L42:
    *x = dc;
    goto L11;
L43:
    if ((r__1 = xinc - *x, ABS(r__1)) - (r__2 = *x - dc, ABS(r__2)) <= (
	    float)0.) {
	goto L44;
    } else {
	goto L2;
    }
L44:
    *x = (xinc + dc) * (float).5;
    if ((xinc - *x) * (*x - dc) <= (float)0.) {
	goto L46;
    } else {
	goto L2;
    }
L45:
    *x = (db + dc) * (float).5;
    if ((db - *x) * (*x - dc) <= (float)0.) {
	goto L46;
    } else {
	goto L2;
    }
L46:
    *itest = 3;
    goto L4;
} 

