/* gfit_chi.h */

#ifndef __GFIT_CHI_H_
#define __GFIT_CHI_H_

#include <glib.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GFitChiSquare GFitChiSquare;
struct _GFitChiSquare
{
  gint user_num_pars; 	/* Total number of parameters */
  gfloat *user_pars;	/* User defined parameters (starting values) */
  gfloat *user_varianz;	/* User defined varianz of parameters: 
  			 * 0: constant parameter (not fitted), 
  			 * <0 unlimited variation of the parameter allowed,
  			 * >0 allowed percentage of variation of starting value */
  gfloat user_acc;	/* Desired interval [...p...] of convergence 
  			 * (percentage of starting value (unlimited fit) 
  			 * or value range (limited fit)  
  			 * 1. is a good value to start with ...*/
  gfloat user_step;	/* Range of first step in variation is calculated like this:
  			 * Width of interval times user_step */
  gint user_limit;	/* Maximum number of gfit_chi_function calls*/
  gint user_num_points;	/* Number of data points */
  gfloat *user_x;	/* X values */
  gfloat *user_y;	/* Y values */
  gfloat *user_z;	/* Difference between fit and values 
  			 * Calculated in gfit_chi_function */
  gpointer user_data;	/* Data needed by gfit_chi_function for calculating */
  gint res_steps;	/* Number of iterations */
  gfloat *res_pars;	/* Parameters determined by the fitting algorithm */
  gfloat *res_stat_errs;/* Statistical errors of determined parameters */
  gfloat res_chi_square;/* Mean chi square value */
  gint res_merr[3];	/* Error codes by the algorithm, if any */
};

enum {
  GFIT_CHI_OK=0,
  GFIT_CHI_PRECISION_UNREACHABLE,	/* chi square may be no function of 
  					parameter i, res_merr[1] holds i */
  GFIT_CHI_PARAMETERS_NOT_INDEPENDEND,  /* res_merr[1] and res_merr[2] hold their indices */
  GFIT_CHI_MORE_ITERATIONS_NEEDED,
  GFIT_CHI_ROUND_OFF_ERRORS,		/* Not that critical ... */
  GFIT_CHI_FLOAT_EXCEPTION,		/* Might be worse ... */
  GFIT_CHI_STOPPED_BY_USER		/* trans->user_data_points<0 */
};

gint gfit_chi(gfit_data *gfit);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __GTK_DATABOX_H__ */

