#include <glib.h>
#include <gfit_math.h>
#include <gfit_engine_functions.h>

gint
gfit_assign(gdouble *target, gint num, gdouble **value)
{
  if (num!=1) {
    *target=0;
    return -1;
  }

  *target=*value[0];

  return 1;
}

gint
gfit_add(gdouble *target, gint num, gdouble **value)
{
  gint i;

  if (num<2) {
    *target=0;
    return 0;
  }

  *target=0;
  for (i=0; i<num; i++) {
    *target+=*value[i];
  }

  return 1;
}

gint
gfit_sub(gdouble *target, gint num, gdouble **value)
{
  gint i;

  if (num<2) {
    *target=0;
    return 0;
  }

  *target=*value[0];
  for (i=1; i<num; i++) {
    *target-=*value[i];
  }

  return 1;
}

gint
gfit_mul(gdouble *target, gint num, gdouble **value)
{
  gint i;

  if (num<2) {
    *target=0;
    return 0;
  }

  *target=*value[0];
  for (i=1; i<num; i++) {
    *target*=*value[i];
  }

  return 1;
}

gint
gfit_div(gdouble *target, gint num, gdouble **value)
{
  gint i;

  if (num<2) {
    *target=0;
    return 0;
  }

  *target=*value[0];
  for (i=1; i<num; i++) {
    if (*value[i]) {
      *target/=*value[i];
    } else {
      return 0;
    }
  }

  return 1;
}

gint
gfit_sin(gdouble *target, gint num, gdouble **value)
/*
 * value[0]: X
 * value[1]: A		Amplitude
 * value[2]: W		Wavelength
 * value[3]: P		Phase
 *
 * Calculates A*sin(X*2*PI/W+P)
 *
 */
{
  if (num!=4) {
    *target=0;
    return 0;
  }

  if (*value[2]) {
    *target=*value[1]*sin(*value[0]*2.*PI/(*value[2])+(*value[3]));
  } else {
    *target=*value[1];
  }

  return 1;
}

gint
gfit_exp(gdouble *target, gint num, gdouble **value)
/*
 * value[0]: X
 * value[1]: A		Amplitude
 * value[2]: T		Tau
 *
 * Calculates A*exp(X/T)
 *
 */
{

  if (num!=3) {
    *target=0;
    return 0;
  }

  if (*value[2]) {
    *target=*value[1]*exp(*value[0]/(*value[2]));
  } else {
    *target=0;
  }

  return 1;
}

gint
gfit_cos(gdouble *target, gint num, gdouble **value)
/*
 * value[0]: X
 * value[1]: A		Amplitude
 * value[2]: W		Wavelength
 * value[3]: P		Phase
 *
 * Calculates A*cos(X*2*PI/W+P)
 *
 */
{
  if (num!=4) {
    *target=0;
    return 0;
  }

  if (*value[2]) {
    *target=*value[1]*cos(*value[0]*2.*PI/(*value[2])+(*value[3]));
  } else {
    *target=*value[1];
  }

  return 1;
}

gint
gfit_tan(gdouble *target, gint num, gdouble **value)
/*
 * value[0]: X
 * value[1]: A		Amplitude
 * value[2]: W		Wavelength
 * value[3]: P		Phase
 *
 * Calculates A*tan(X*2*PI/W+P)
 *
 */
{
  if (num!=4) {
    *target=0;
    return 0;
  }

  if (*value[2]) {
    *target=*value[1]*tan(*value[0]*2.*PI/(*value[2])+(*value[3]));
  } else {
    *target=*value[1];
  }

  return 1;
}

gint
gfit_cot(gdouble *target, gint num, gdouble **value)
/*
 * value[0]: X
 * value[1]: A		Amplitude
 * value[2]: W		Wavelength
 * value[3]: P		Phase
 *
 * Calculates A*cot(X*2*PI/W+P)
 *
 */
{
  if (num!=4) {
    *target=0;
    return 0;
  }

  if (*value[2]) {
    *target=*value[1]/tan(*value[0]*2.*PI/(*value[2])+(*value[3]));
  } else {
    *target=*value[1];
  }

  return 1;
}

gint
gfit_polynom(gdouble *target, gint num, gdouble **value)
/*
 * value[0]: X
 * value[1]: X0
 * value[2]: A0
 * value[3]: A1
 * ...
 *
 * Calculates A0+A1*(X-X0)+A2*(X-X0)^2+A3*(X-X0)^3...
 *
 */
{
  gint i;
  
  if (num<3) {
    *target=0;
    return 0;
  }

  *target=*value[num-1];
  for (i=num-2; i>1; i--) {
    *target=*target*(*value[0]-*value[1])+*value[i];
  }

  return 1;
}

