#include <gfit_math.h>
#include <gfit.h>
#include <gfit_file.h>
#include <gfit_load.h>
#include <gfit_fit.h>
#include <gtk/gtkdatabox.h>
#include <gtk/gtkextfilesel.h>
#include <string.h>

static void		gfit_file_open_define_popups();
static void 		gfit_dialog_cancel(GtkWidget *widget, gpointer *callback_data);
static void 		gfit_file_open_ok(GtkWidget *widget, gpointer callback_data);

static gchar *gfit_file_open_popup_entries[GFIT_FILE_OPEN_MAX_NUM];


/*--------------------------------------------------------------
 *
 *               EXTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

void
gfit_file_open(gfit_data *gfit)
{
  static guint init_flag=0;
  GList *cbitems=NULL;
  gint i;

  if (gfit->dialog) return;

  if (!init_flag) {
    init_flag=1;
    gfit_file_open_define_popups();
  }

  gfit->dialog=gtk_extended_file_selection_new("Open data file...");
  gtk_window_set_modal (GTK_WINDOW(gfit->dialog),TRUE);
  gtk_signal_connect(GTK_OBJECT(gfit->dialog), "destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                     &(gfit->dialog));
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(gfit->dialog)->cancel_button), "pressed",
                     GTK_SIGNAL_FUNC(gfit_dialog_cancel),
                     gfit);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(gfit->dialog)->ok_button), "pressed",
                     GTK_SIGNAL_FUNC(gfit_file_open_ok),
                     gfit);

  for (i=0; i<GFIT_FILE_OPEN_MAX_NUM; i++) {
    cbitems = g_list_append(cbitems, gfit_file_open_popup_entries[i]);
  }

  gtk_extended_file_selection_type_popup(GTK_EXTENDED_FILE_SELECTION(gfit->dialog), cbitems, GFIT_FILE_OPEN_STAR_BINARY);
  gtk_widget_show(gfit->dialog);
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(gfit->dialog), gfit->path);
  gtk_entry_select_region(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry),0,strlen(gfit->path));
  gtk_extended_file_selection_goto(GTK_EXTENDED_FILE_SELECTION(gfit->dialog));

  return;
}

void
gfit_sample_create(gfit_data *gfit, gint points)
{
  gint i;

  gfit_destroy_engine(gfit);
  GTK_DATABOX(gfit->box)->data.X=(gfloat *) g_malloc(points*sizeof(gfloat));
  GTK_DATABOX(gfit->box)->data.Z=(gfloat **) g_malloc(1*sizeof(gfloat *));
  GTK_DATABOX(gfit->box)->data.Z[0]=(gfloat *) g_malloc(points*sizeof(gfloat));
  for (i=0; i<points; i++) {
    GTK_DATABOX(gfit->box)->data.X[i]=i;
    GTK_DATABOX(gfit->box)->data.Z[0][i]=100.*sin(i*2*PI/200+1.);
  }
  GTK_DATABOX(gfit->box)->data.dimX=points;
  GTK_DATABOX(gfit->box)->data.dimY=1;
  gtk_databox_new_data(GTK_DATABOX(gfit->box), GTK_DATABOX_POINTS_2D);
}

/*--------------------------------------------------------------
 *
 *               INTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

static void
gfit_file_open_define_popups()
{
  gint i;
  
  for (i=0; i<GFIT_FILE_OPEN_MAX_NUM; i++)
  {
    switch(i) {
      case GFIT_FILE_OPEN_STAR_BINARY: 
        gfit_file_open_popup_entries[i]="Binary Float Vector *.?##";
        break;
      case GFIT_FILE_OPEN_GMCAL_ASCII: 
        gfit_file_open_popup_entries[i]="ASCII Matrix *.dat";
        break;
      case GFIT_FILE_OPEN_CSMA_BINARY: 
        gfit_file_open_popup_entries[i]="CSMA Matrix *.spe";
        break;
      case GFIT_FILE_OPEN_ALL: 
        gfit_file_open_popup_entries[i]="All Files";
        break;
      default:
      	gfit_file_open_popup_entries[i]="";
    }
  }
}

static void
gfit_dialog_cancel(GtkWidget *widget, gpointer *callback_data)
{
  gfit_data *gfit=(gfit_data *) callback_data;

  gtk_widget_destroy(gfit->dialog);
}

static void
gfit_file_open_ok(GtkWidget *widget, gpointer callback_data)
{
  gfit_data *gfit=(gfit_data *) callback_data;

  gfit_destroy_engine(gfit);
  gfit_load_data(gfit, GTK_EXTENDED_FILE_SELECTION(gfit->dialog)->file_type);
}

