#include <sys/stat.h>
#include <stdlib.h>
#include <string.h>
#include <gfit.h>
#include <gfit_fit.h>
#include <gtk/gtkengine.h>
#include <gtk/gtkextfilesel.h>
#include <gtk/gtkdatabox.h>
#include <gmcal/gmcalread.h>
#include <gdk/gdkkeysyms.h>

#define __GFIT_FIT_C__
#include <gfit_engine_functions.h>
#undef __GFIT_FIT_C__

#define GFIT_FIT_TABLE_WIDTH 3

static gchar *gfit_fit_open_popup_entries[GFIT_FIT_OPEN_MAX_NUM];
static gchar *gfit_fit_save_popup_entries[GFIT_FIT_SAVE_MAX_NUM];

typedef struct {
  gint index;
  GtkWidget *entry;
  gfit_data *gfit;
  gint type;
  gdouble def;
} parameter;

enum {
  PAR_PLUS,
  PAR_MINUS,
  PAR_RESET,
  PAR_FIT
};

enum {
  GFIT_STEP_TYPE_ABSOLUT,
  GFIT_STEP_TYPE_PERCENT
};

static void gfit_fit_open_define_popups();
static void gfit_fit_save_define_popups();
static void gfit_dialog_cancel(GtkWidget *widget, gpointer *callback_data);
static void gfit_engine_open_ok(GtkWidget *widget, gpointer *callback_data);
static void gfit_engine_save_ok(GtkWidget *widget, gpointer *callback_data);
static void gfit_show_engine(gfit_data *gfit);
static gint gfit_par_changed(GtkWidget *widget, parameter *par);
static void gfit_connect_button(gfit_data *gfit, gint index, GtkWidget *entry, gint type, GtkWidget *button, gdouble def);
static gint gfit_step_width_changed(GtkWidget *widget, GdkEventFocus *event, gfit_data *gfit);
static gint gfit_step_type_changed(GtkWidget *widget, gfit_data *gfit);
static gint gfit_parameter_changed(GtkWidget *widget, GdkEventKey *event, parameter *par);

static gint gfit_fit_load(gfit_data *gfit, char *path);
static gint gfit_fit_write(gfit_data *gfit, char *path);

static gint gfit_find_func_ptr(gchar *s);
static void gfit_fit_recalc(gfit_data *gfit);



/*--------------------------------------------------------------
 *
 *               EXTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

void
gfit_fit_external(gfit_data *gfit)
{
  gint i;
  gchar dummy[1000];
  
  gfit_fit_recalc(gfit);
  
  for (i=0; i<GTK_ENGINE(gfit->engine)->num_vars; i++) { 
    if (GTK_ENGINE(gfit->engine)->vars[i].user_pointer) {
      sprintf(dummy, "%f", GTK_ENGINE(gfit->engine)->vars[i].value);
      gtk_entry_set_text(GTK_ENTRY(GTK_ENGINE(gfit->engine)->vars[i].user_pointer), dummy);
    }
  }

  return;
}

void
gfit_fit_engine_save(gfit_data *gfit)
{
  static guint init_flag=0;
  GList *cbitems=NULL;
  gint i;

  printf("Saving engine...?\n");

  if (gfit->dialog) return;
  if (!gfit->engine) return;

  printf("Yes\n");

  if (!init_flag) {
    init_flag=1;
    gfit_fit_save_define_popups();
  }

  gfit->dialog=gtk_extended_file_selection_new("Save fit function...");
  gtk_window_set_modal (GTK_WINDOW(gfit->dialog),TRUE);
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(gfit->dialog), gfit->path);
  gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry), "*.gff");
  gtk_entry_select_region(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry),0,strlen("*.gff"));
  gtk_signal_connect(GTK_OBJECT(gfit->dialog), "destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                     &(gfit->dialog));
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(gfit->dialog)->cancel_button), "pressed",
                     GTK_SIGNAL_FUNC(gfit_dialog_cancel),
                     gfit);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(gfit->dialog)->ok_button), "pressed",
                     GTK_SIGNAL_FUNC(gfit_engine_save_ok),
                     gfit);

  for (i=0; i<GFIT_FIT_SAVE_MAX_NUM; i++) {
    cbitems = g_list_append(cbitems, gfit_fit_save_popup_entries[i]);
  }

  gtk_extended_file_selection_type_popup(GTK_EXTENDED_FILE_SELECTION(gfit->dialog), cbitems, GFIT_FIT_SAVE_SIMPLE_GFIT_FILE);
  gtk_widget_show(gfit->dialog);
  gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry), "*.gff");
  gtk_entry_select_region(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry),0,strlen("*.gff"));
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(gfit->dialog), gfit->path);
  gtk_extended_file_selection_goto(GTK_EXTENDED_FILE_SELECTION(gfit->dialog));

  return;
}

void
gfit_fit_engine_open(gfit_data *gfit)
{
  static guint init_flag=0;
  GList *cbitems=NULL;
  gint i;

  if (gfit->dialog) return;
  if (GTK_DATABOX(gfit->box)->data.type==GTK_DATABOX_NO_DATA) {
    gfit_warning("No data is present.\n");
    return;
  }

  if (!init_flag) {
    init_flag=1;
    gfit_fit_open_define_popups();
  }

  gfit->dialog=gtk_extended_file_selection_new("Select fit function...");
  gtk_window_set_modal (GTK_WINDOW(gfit->dialog),TRUE);
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(gfit->dialog), gfit->path);
  gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry), "*.gff");
  gtk_entry_select_region(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry),0,strlen("*.gff"));
  gtk_signal_connect(GTK_OBJECT(gfit->dialog), "destroy",
                     GTK_SIGNAL_FUNC(gtk_widget_destroyed),
                     &(gfit->dialog));
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(gfit->dialog)->cancel_button), "pressed",
                     GTK_SIGNAL_FUNC(gfit_dialog_cancel),
                     gfit);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(gfit->dialog)->ok_button), "pressed",
                     GTK_SIGNAL_FUNC(gfit_engine_open_ok),
                     gfit);

  for (i=0; i<GFIT_FIT_OPEN_MAX_NUM; i++) {
    cbitems = g_list_append(cbitems, gfit_fit_open_popup_entries[i]);
  }

  gtk_extended_file_selection_type_popup(GTK_EXTENDED_FILE_SELECTION(gfit->dialog), cbitems, GFIT_FIT_OPEN_SIMPLE_GFIT_FILE);
  gtk_widget_show(gfit->dialog);
  gtk_entry_set_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry), "*.gff");
  gtk_entry_select_region(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry),0,strlen("*.gff"));
  gtk_file_selection_set_filename(GTK_FILE_SELECTION(gfit->dialog), gfit->path);
  gtk_extended_file_selection_goto(GTK_EXTENDED_FILE_SELECTION(gfit->dialog));

  return;
}

static GtkEngineVar sample_vars[]={
  { "X", "X Value", 0, -10},
  { "Y", "Y Value", 0, -10},
  { "period", "Wavelength of sinus", 10.5, -1},
  { "amp", "Amplitude of sinus", 20, -1},
  { "phase", "Phase offset", 0, -1},
  { "offset", "Amplitude offset", 0, 0},
  { "wave", "Buffer variable", 0, -10},
};
static gint sample_num_vars=sizeof(sample_vars)/sizeof(sample_vars[0]);

static gdouble *sin_args[]={&sample_vars[0].value, &sample_vars[3].value, &sample_vars[2].value, &sample_vars[4].value};
static gdouble *add_args[]={&sample_vars[6].value, &sample_vars[5].value};

static GtkEngineFunction sample_functions[]={
  { &sample_vars[6].value, 4, sin_args, gfit_sin, "sinus", "sinus with amplitude, wavelength and phase"},
  { &sample_vars[1].value, 2, add_args, gfit_add, "add", "add a variable number of values"},
};
static gint sample_num_functions=sizeof(sample_functions)/sizeof(sample_functions[0]);

void
gfit_fit_sample_create(gfit_data *gfit)
{
  if (GTK_DATABOX(gfit->box)->data.type==GTK_DATABOX_NO_DATA) {
    gfit_warning("No data is present.\n");
    return;
  }

  gfit_destroy_engine(gfit);

  printf("Creating sample fit %d variables...\n", sample_num_vars);
  
  gfit->engine=gtk_engine_new();
  gtk_engine_set_name(GTK_ENGINE(gfit->engine), "Sinus Sample");
  gtk_engine_set_variables(GTK_ENGINE(gfit->engine), sample_num_vars, sample_vars);
  gtk_engine_set_functions(GTK_ENGINE(gfit->engine), sample_num_functions, sample_functions);
  
  gfit_show_engine(gfit);
}

void
gfit_destroy_engine(gfit_data *gfit)
{
  printf("Destroying....\n");

  if (gfit->par_table && GTK_IS_TABLE(gfit->par_table)) {
    gtk_widget_destroy(gfit->par_table);
  }
  gfit->par_table=NULL;
  
  if (gfit->fit_flag && GTK_DATABOX(gfit->box)->data.dimY) {
    GTK_DATABOX(gfit->box)->data.dimY--;
    g_free(GTK_DATABOX(gfit->box)->data.Z[GTK_DATABOX(gfit->box)->data.dimY]);
    GTK_DATABOX(gfit->box)->data.Z[GTK_DATABOX(gfit->box)->data.dimY]=NULL;
    GTK_DATABOX(gfit->box)->data.Y=(float *) g_realloc((void *) GTK_DATABOX(gfit->box)->data.Y, GTK_DATABOX(gfit->box)->data.dimY*sizeof(gfloat) );
  }
  gfit->fit_flag=0;
}

/*--------------------------------------------------------------
 *
 *               INTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

static void
gfit_fit_open_define_popups()
{
  gint i;
  
  for (i=0; i<GFIT_FIT_OPEN_MAX_NUM; i++)
  {
    switch(i) {
      case GFIT_FIT_OPEN_SIMPLE_GFIT_FILE: 
        gfit_fit_open_popup_entries[i]="GFit Fit File *.gff";
        break;
      case GFIT_FIT_OPEN_ALL: 
        gfit_fit_open_popup_entries[i]="All Files";
        break;
      default:
      	gfit_fit_open_popup_entries[i]="";
    }
  }
}

static void
gfit_fit_save_define_popups()
{
  gint i;
  
  for (i=0; i<GFIT_FIT_SAVE_MAX_NUM; i++)
  {
    switch(i) {
      case GFIT_FIT_SAVE_SIMPLE_GFIT_FILE: 
        gfit_fit_save_popup_entries[i]="GFit Fit File *.gff";
        break;
      case GFIT_FIT_SAVE_ALL: 
        gfit_fit_save_popup_entries[i]="All Files";
        break;
      default:
      	gfit_fit_save_popup_entries[i]="";
    }
  }
}

static void
gfit_dialog_cancel(GtkWidget *widget, gpointer *callback_data)
{
  gfit_data *gfit=(gfit_data *) callback_data;

  gtk_widget_destroy(gfit->dialog);
}

static void
gfit_engine_open_ok(GtkWidget *widget, gpointer *callback_data)
{
  gfit_data *gfit=(gfit_data *) callback_data;
  gchar *filename;
  gchar *path;
  struct stat buf;

  if (GTK_EXTENDED_FILE_SELECTION(gfit->dialog)->file_type!=GFIT_FIT_OPEN_SIMPLE_GFIT_FILE) {
    gfit_warning("File type (or automatic type detection) not implemented yet...\n");
    return;
  }
  
  filename=gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry));

  if (!filename || !strlen(filename)) {
    gfit_warning("No filename specified...");
    return;
  }

  path=gtk_file_selection_get_filename(GTK_FILE_SELECTION(gfit->dialog));
  if (stat(path, &buf)) {
    gfit_warning("Could not retrieve stats for file\nIt does not seem to exist...");
    return;
  }
#ifndef _WIN32
  if (!S_ISREG(buf.st_mode)) {
    gfit_warning("No regular file\nPlease choose another one...");
    return;
  }
#endif
  gfit_destroy_engine(gfit);

  if (gfit_fit_load(gfit, path)) {
    gfit_warning("An error occurred...");
  }
  gfit_show_engine(gfit);

  gtk_widget_destroy(gfit->dialog);
}

static void
gfit_engine_save_ok(GtkWidget *widget, gpointer *callback_data)
{
  gfit_data *gfit=(gfit_data *) callback_data;
  gchar *filename;
  gchar *path;
  struct stat buf;

  if (GTK_EXTENDED_FILE_SELECTION(gfit->dialog)->file_type!=GFIT_FIT_OPEN_SIMPLE_GFIT_FILE) {
    gfit_warning("File type (or automatic type detection) not implemented yet...\n");
    return;
  }
  
  filename=gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry));

  if (!filename || !strlen(filename)) {
    gfit_warning("No filename specified...");
    return;
  }

  path=gtk_file_selection_get_filename(GTK_FILE_SELECTION(gfit->dialog));
  if (!stat(path, &buf)) {
#ifndef _WIN32
    if (!S_ISREG(buf.st_mode)) {
      gfit_warning("No regular file\nPlease choose another one...");
      return;
    }
#endif
  }

  if (gfit_fit_write(gfit, path)) {
    gfit_warning("An error occurred...");
    return;
  }
  
  gtk_widget_destroy(gfit->dialog);
}

static gint
gfit_par_changed(GtkWidget *widget, parameter *par)
{
  gdouble value;
  gchar dummy[30];

  value=gtk_engine_get_value(GTK_ENGINE(par->gfit->engine), par->index);
  switch(par->type) {
    case PAR_PLUS:
        if (par->gfit->step_type==GFIT_STEP_TYPE_ABSOLUT) {
      	  value+=par->gfit->step_width;
      	} else {
      	  value*=(1+par->gfit->step_width/100);
      	}
        sprintf(dummy,"%f",value);
        gtk_entry_set_text(GTK_ENTRY(par->entry), dummy);
        gtk_engine_set_value(GTK_ENGINE(par->gfit->engine), par->index, value);
        gfit_fit_recalc(par->gfit);
    	break;
    case PAR_MINUS:
        if (par->gfit->step_type==GFIT_STEP_TYPE_ABSOLUT) {
      	  value-=par->gfit->step_width;
      	} else {
      	  value/=(1+par->gfit->step_width/100);
      	}
        sprintf(dummy,"%f",value);
        gtk_entry_set_text(GTK_ENTRY(par->entry), dummy);
        gtk_engine_set_value(GTK_ENGINE(par->gfit->engine), par->index, value);
        gfit_fit_recalc(par->gfit);
    	break;
    case PAR_RESET:
    	value=par->def;
        sprintf(dummy,"%f",value);
        gtk_entry_set_text(GTK_ENTRY(par->entry), dummy);
        gtk_engine_set_value(GTK_ENGINE(par->gfit->engine), par->index, value);
        gfit_fit_recalc(par->gfit);
    	break;
    case PAR_FIT:
    	if (GTK_IS_TOGGLE_BUTTON(widget)) {
          if (GTK_TOGGLE_BUTTON(widget)->active) {
            gtk_engine_set_user_data(GTK_ENGINE(par->gfit->engine), par->index, -1);
    	  } else {
            gtk_engine_set_user_data(GTK_ENGINE(par->gfit->engine), par->index, 0);
    	  }
    	}
    	break;
  }
  return(TRUE);
}

static void
gfit_connect_button(gfit_data *gfit, gint index, GtkWidget *entry, gint type, GtkWidget *button, gdouble def)
{
  parameter *par;

  par=(parameter *) g_malloc(1*sizeof(parameter));
  par->index=index;
  par->entry=entry;
  par->gfit=gfit;
  par->type=type;
  par->def=def;
  gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC (gfit_par_changed), par);
}

static gint
gfit_step_width_changed(GtkWidget *widget, GdkEventFocus *event, gfit_data *gfit)
{
  gchar *text;
  gchar buf[30];

  text=gtk_entry_get_text(GTK_ENTRY(widget));
  gfit->step_width=g_strtod(text, NULL);
  sprintf(buf, "%f\n", gfit->step_width);
  gtk_entry_set_text(GTK_ENTRY(widget), buf);

  return TRUE;
}

static gint
gfit_step_type_changed(GtkWidget *widget, gfit_data *gfit)
{
  gchar *text;

  text=gtk_entry_get_text(GTK_ENTRY(widget));
  if (!strcmp(text, "absolut")) {
    gfit->step_type=GFIT_STEP_TYPE_ABSOLUT;
  } else {
    gfit->step_type=GFIT_STEP_TYPE_PERCENT;
  }

  return TRUE;
}

static gint
gfit_parameter_changed(GtkWidget *widget, GdkEventKey *event, parameter *par)
{
  gchar *text;
  gchar buf[30];
  gdouble value;
  
  if (event->keyval!=GDK_Return && event->keyval!=GDK_KP_Enter) {
    return TRUE;
  }

  text=gtk_entry_get_text(GTK_ENTRY(widget));
  value=g_strtod(text, NULL);
  gtk_engine_set_value(GTK_ENGINE(par->gfit->engine), par->index, value);
  sprintf(buf, "%f\n", value);
  gtk_entry_set_text(GTK_ENTRY(widget), buf);

  gfit_fit_recalc(par->gfit);

  return TRUE;
}

static void
gfit_fit_recalc(gfit_data *gfit)
{
  gint i;
  gfloat *Z;
  gfloat *X;
  gint dimX;
  gdouble *x;
  gdouble *y;
  
  Z=GTK_DATABOX(gfit->box)->data.Z[GTK_DATABOX(gfit->box)->data.dimY-1];
  X=GTK_DATABOX(gfit->box)->data.X;
  dimX=GTK_DATABOX(gfit->box)->data.dimX;
  x=gtk_engine_get_value_pointer(GTK_ENGINE(gfit->engine), 
  			gtk_engine_get_value_index_by_name(GTK_ENGINE(gfit->engine), "X"));
  y=gtk_engine_get_value_pointer(GTK_ENGINE(gfit->engine), 
  			gtk_engine_get_value_index_by_name(GTK_ENGINE(gfit->engine), "Y"));

  if (!x || !y) {
    gfit_warning("Fit Function MUST contain X and Y (variable names are case sensitiv!)");
    return;
  }
  
  for (i=0; i<dimX; i++) {
    *x=X[i];
    gtk_engine_run(GTK_ENGINE(gfit->engine));
    Z[i]=*y;
  }
  gtk_databox_redraw(GTK_DATABOX(GTK_DATABOX(gfit->box)));
}

static void
gfit_show_engine(gfit_data *gfit)
{
  gint i;
  gint max_vars;
  GtkEngineVar *vars;
  gint num_vars;
  gint counter;
  gint left, right, up, down;
  GtkWidget *label;
  GtkWidget *combo;
  GList *combo_items = NULL;
  GtkWidget *frame;
  GtkWidget *entry;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *plus;
  GtkWidget *minus;
  GtkWidget *def;
  GtkWidget *fit;
  gchar dummy[30];
  parameter *par;
  
  if ((max_vars=gtk_engine_get_variables(GTK_ENGINE(gfit->engine), &vars))==-1) {
    gfit_warning("No fit engine loaded...\n");
    return;
  }
  

  GTK_DATABOX(gfit->box)->data.dimY++;
  GTK_DATABOX(gfit->box)->data.Y=(gfloat *) g_realloc((void *) GTK_DATABOX(gfit->box)->data.Y, GTK_DATABOX(gfit->box)->data.dimY*sizeof(gfloat));
  GTK_DATABOX(gfit->box)->data.Y[GTK_DATABOX(gfit->box)->data.dimY-1]=GTK_DATABOX(gfit->box)->data.dimY;
  GTK_DATABOX(gfit->box)->data.Z=(gfloat **) g_realloc((void *) GTK_DATABOX(gfit->box)->data.Z, GTK_DATABOX(gfit->box)->data.dimY*sizeof(gfloat *));
  GTK_DATABOX(gfit->box)->data.Z[GTK_DATABOX(gfit->box)->data.dimY-1]=(gfloat *) g_malloc0(GTK_DATABOX(gfit->box)->data.dimX*sizeof(gfloat));
  gfit->fit_flag=1;

  gfit_fit_recalc(gfit);

  num_vars=0;
  for (i=0; i<max_vars; i++) {
    vars[i].user_pointer=NULL;
    if (vars[i].user_data!=-10) {
      num_vars++;
    }
  }
  printf("num_vars: %d\n", num_vars);

  gfit->par_table=gtk_table_new(GFIT_FIT_TABLE_WIDTH, (num_vars+GFIT_FIT_TABLE_WIDTH-1)/GFIT_FIT_TABLE_WIDTH+2, FALSE);

  gtk_box_pack_start(GTK_BOX(gfit->vbox), gfit->par_table, FALSE, FALSE, 0);

  label=gtk_label_new(gtk_engine_get_name(GTK_ENGINE(gfit->engine)));
  gtk_table_attach(GTK_TABLE(gfit->par_table), label,
  		0, GFIT_FIT_TABLE_WIDTH,
  		0, 1,
  		GTK_FILL|GTK_EXPAND,
  		GTK_FILL|GTK_SHRINK,
  		0,0);
  gtk_widget_show(label);

  combo=gtk_combo_new();
  combo_items = g_list_append(combo_items, "0.000001");
  combo_items = g_list_append(combo_items, "0.00001");
  combo_items = g_list_append(combo_items, "0.0001");
  combo_items = g_list_append(combo_items, "0.001");
  combo_items = g_list_append(combo_items, "0.01");
  combo_items = g_list_append(combo_items, "0.1");
  combo_items = g_list_append(combo_items, "1");
  combo_items = g_list_append(combo_items, "10");
  combo_items = g_list_append(combo_items, "100");
  combo_items = g_list_append(combo_items, "1000");
  combo_items = g_list_append(combo_items, "10000");
  combo_items = g_list_append(combo_items, "100000");
  combo_items = g_list_append(combo_items, "1000000");
  gtk_combo_set_popdown_strings (GTK_COMBO (combo), combo_items);
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(combo)->entry), "1");
  gfit->step_width=1;
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry),
  			"focus_out_event", GTK_SIGNAL_FUNC (gfit_step_width_changed), gfit);

  frame=gtk_frame_new("Step Width");
  gtk_table_attach(GTK_TABLE(gfit->par_table), frame,
  		0, 1,
  		1, 2,
  		GTK_FILL|GTK_EXPAND,
  		GTK_FILL|GTK_SHRINK,
  		0,0);
  gtk_widget_show(frame);
  gtk_container_add(GTK_CONTAINER(frame), combo);
  gtk_widget_show(combo);

  combo=gtk_combo_new();
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(combo)->entry),
                             FALSE);
  combo_items = NULL;
  combo_items = g_list_append(combo_items, "absolut");
  combo_items = g_list_append(combo_items, "percent");
  gtk_combo_set_popdown_strings (GTK_COMBO (combo), combo_items);
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(combo)->entry), "absolut");
  gfit->step_type=GFIT_STEP_TYPE_ABSOLUT;
  gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry),
  			"changed", GTK_SIGNAL_FUNC (gfit_step_type_changed), gfit);

  frame=gtk_frame_new("Step Type");
  gtk_table_attach(GTK_TABLE(gfit->par_table), frame,
  		1, 2,
  		1, 2,
  		GTK_FILL|GTK_EXPAND,
  		GTK_FILL|GTK_SHRINK,
  		0,0);
  gtk_widget_show(frame);
  gtk_container_add(GTK_CONTAINER(frame), combo);
  gtk_widget_show(combo);

  counter=0;
  for (i=0; i<max_vars; i++) {
    if (vars[i].user_data==-10) continue;
    
    left=counter%GFIT_FIT_TABLE_WIDTH+0;
    right=counter%GFIT_FIT_TABLE_WIDTH+1;
    up=counter/GFIT_FIT_TABLE_WIDTH+2;
    down=counter/GFIT_FIT_TABLE_WIDTH+3;

    counter++;
    
    frame=gtk_frame_new(vars[i].desc);

    gtk_table_attach(GTK_TABLE(gfit->par_table), frame,
    		left, right,
    		up, down,
                GTK_FILL|GTK_EXPAND,
                GTK_FILL|GTK_SHRINK,
                0,0);

    gtk_widget_show(frame);
    vbox=gtk_vbox_new(FALSE, 4);
    gtk_container_add(GTK_CONTAINER(frame), vbox);
    gtk_widget_show(vbox);
    entry=gtk_entry_new();
    gtk_widget_set_usize(entry, 100, -1);
    sprintf(dummy,"%f",vars[i].value);
    gtk_entry_set_text(GTK_ENTRY(entry), dummy);
    gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 2);
    gtk_widget_show(entry);
    hbox=gtk_hbox_new(FALSE, 4);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
    plus=gtk_button_new_with_label("  +  ");
    gtk_widget_show(plus);
    gtk_box_pack_start(GTK_BOX(hbox), plus, FALSE, FALSE, 2);
    minus=gtk_button_new_with_label("  -  ");
    gtk_box_pack_start(GTK_BOX(hbox), minus, FALSE, FALSE, 2);
    gtk_widget_show(minus);
    def=gtk_button_new_with_label("Reset");
    gtk_box_pack_start(GTK_BOX(hbox), def, FALSE, FALSE, 2);
    gtk_widget_show(def);
    fit=gtk_check_button_new_with_label("Fit");
    if (vars[i].user_data) {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(fit), 1);
    } else {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(fit), 0);
    }
    gtk_box_pack_start(GTK_BOX(hbox), fit, FALSE, FALSE, 2);
    gtk_widget_show(fit);

    vars[i].user_pointer=(gpointer) entry;
    gfit_connect_button(gfit, i, entry, PAR_PLUS, plus, vars[i].value);
    gfit_connect_button(gfit, i, entry, PAR_MINUS, minus, vars[i].value);
    gfit_connect_button(gfit, i, entry, PAR_RESET, def, vars[i].value);
    gfit_connect_button(gfit, i, entry, PAR_FIT, fit, vars[i].value);
    par=(parameter *) g_malloc(1*sizeof(parameter));
    par->index=i;
    par->entry=entry;
    par->gfit=gfit;
    par->type=10000;
    gtk_signal_connect_after(GTK_OBJECT(entry), "key_press_event", 
    		GTK_SIGNAL_FUNC (gfit_parameter_changed), par);

  }
  gtk_widget_show(gfit->par_table);
  
  
}

static gint
gfit_find_func_ptr(gchar *s)
{
  gint i;
  gint len;

  if (!s) return -1;

  len=strlen(s);

  for (i=0; i<gfit_fit_engine_functions_num; i++) {
    if (len==strlen(gfit_fit_engine_functions[i].name) && !strcmp(s, gfit_fit_engine_functions[i].name)) return i;
  }

  return -1;
}

static gint
gfit_fit_write(gfit_data *gfit, gchar *path)
{
  FILE *fp;
  GtkEngine *engine=GTK_ENGINE(gfit->engine);
  gint i, j, k;
  
  if (!(fp=fopen(path, "w"))) {
    return 1;
  }
  
  fprintf(fp, "Function: %s\n", engine->name);
  fprintf(fp, "Variables: %d\n", engine->num_vars);
  for (i=0; i<engine->num_vars; i++) {
    fprintf(fp, "%s\t%f\t%d\t%s\n", engine->vars[i].name, engine->vars[i].value, engine->vars[i].user_data, engine->vars[i].desc);
  }
  fprintf(fp, "Algorithm: %d\n", GTK_ENGINE(gfit->engine)->num_functions);
  for (i=0; i<engine->num_functions; i++) {
    for (j=0; j<engine->num_vars; j++) {
      if (&engine->vars[j].value==engine->functions[i].target) {
        fprintf(fp, "%s ", engine->vars[j].name);
        break;
      }
    }
    fprintf(fp, "%s", engine->functions[i].name);
    for (j=0; j<engine->functions[i].num; j++) {
      for (k=0; k<engine->num_vars; k++) {
        if (&engine->vars[k].value==engine->functions[i].value[j]) {
          fprintf(fp, " %s", engine->vars[k].name);
          break;
        }
      }
    }
    fprintf(fp, "\n");
  }
  
  fclose(fp);
  
  return 0;
}

static gint
gfit_fit_load(gfit_data *gfit, gchar *path)
{
  FILE *fp;
  gchar *line=NULL;
  guint num_vars;
  GtkEngineVar *vars;
  guint num_functions;
  GtkEngineFunction *functions;

  gchar *scanner;
  gint max_len=0;
  gint len;
  gchar *name=NULL;
  gchar *desc=NULL;
  gchar *def=NULL;
  gchar *var=NULL;
  gint index;
  gint i;

  printf("Loading Fit Function %s\n", path);

  if (!(fp=fopen(path, "r"))) {
    printf("File %s could not be opened\n", path);
    return -1;
  }

  if ((len=gmcal_read_line(&line, &max_len, fp))==-1) {
    fclose(fp);
    gfit_warning("First line has to be \"Function: <Name>\"\n");
    return -1;
  }

  line[strlen(line)-1]='\0';

  gfit->engine=gtk_engine_new();
  gtk_engine_set_name(GTK_ENGINE(gfit->engine), line+strlen("Function: "));
  if (!strlen(gtk_engine_get_name(GTK_ENGINE(gfit->engine)))) {
    fclose(fp);
    gfit_warning("First line has to be \"Function: <Name>\"\n");
    return -1;
  }

  printf("line: %s\nname: %s\n", line, gtk_engine_get_name(GTK_ENGINE(gfit->engine)));

  if ((len=gmcal_read_line(&line, &max_len, fp))==-1) {
    fclose(fp);
    gfit_warning("Second line has to be \"Variables: <Number of variables>\"\n");
    return -1;
  }
  sscanf(line, "Variables: %d\n", &num_vars);
  if (num_vars<1) {
    fclose(fp);
    gfit_warning("Number of variables has to be 1 at least (line 2)\n");
    return -1;
  }
  printf("line: %s\nVariables: %d\n", line, num_vars);

  vars=(GtkEngineVar *) g_malloc(num_vars*sizeof(GtkEngineVar));


  for (i=0; i<num_vars; i++) {
    if ((len=gmcal_read_line(&line, &max_len, fp))==-1) {
      fclose(fp);
      gfit_warning("Variable description line missing!\n");
      return -1;
    }

    line[strlen(line)-1]='\0';

    name=(gchar *) g_realloc((void *) name, len*sizeof(gchar));
    def=(gchar *) g_realloc((void *) def, len*sizeof(gchar));
    var=(gchar *) g_realloc((void *) var, len*sizeof(gchar));
    desc=(gchar *) g_realloc((void *) desc, len*sizeof(gchar));
    sscanf(line,"%s %s %s\n", name, def, var);
    vars[i].name=g_strdup(name);
    vars[i].desc=g_strdup(strrchr(line, '\t')+1);
    vars[i].value=g_strtod(def, NULL);
    vars[i].user_data=strtol(var, NULL, 10);
    printf("name: %s, desc: %s, value: %f, user_data: %d\n", vars[i].name, vars[i].desc, vars[i].value, vars[i].user_data);
  }

  gtk_engine_set_variables(GTK_ENGINE(gfit->engine), num_vars, vars);

  if ((len=gmcal_read_line(&line, &max_len, fp))==-1) {
    fclose(fp);
    gfit_warning("Following the variable section there has to be: \"Algorithm: <Number of actions>\"\n");
    return -1;
  }
  sscanf(line, "Algorithm: %d\n", &num_functions);
  if (num_functions<1) {
    fclose(fp);
    gfit_warning("Number of algorithm actions has to be 1 at least (line 2)\n");
    return -1;
  }
  printf("line: %s\nAlgorithm: %d\n", line, num_functions);

  functions=(GtkEngineFunction *) g_malloc(num_functions*sizeof(GtkEngineFunction));

  for (i=0; i<num_functions; i++) {
    if ((len=gmcal_read_line(&line, &max_len, fp))==-1) {
      fclose(fp);
      gfit_warning("Algorithm line missing!\n");
      return -1;
    }
    name=(gchar *) g_realloc((void *) name, len*sizeof(gchar));
    sscanf(line,"%s\n", name);
    index=gtk_engine_get_value_index_by_name(GTK_ENGINE(gfit->engine), name);
    if (index==-1) {
      fclose(fp);
      gfit_warning("First string in an algorithm line has to be a variable descriptor!\n");
      return -1;
    }

    printf("target: %s, index: %d\n", name, index);

    functions[i].target=gtk_engine_get_value_pointer(GTK_ENGINE(gfit->engine), index);

    scanner=strchr(line, ' ');
    if (!scanner) {
      fclose(fp);
      gfit_warning("No function found (Algorithm strings MUST be separated by [SPACE]\n");
      return -1;
    }

    sscanf(scanner,"%s", name);
    index=gfit_find_func_ptr(name);
    if (index==-1) {
      fclose(fp); 
      gfit_warning("Second string in an algorithm line has to be a function descriptor!\n");
      return -1;
    }

    printf("function: %s\n",name);
    functions[i].fnct=gfit_fit_engine_functions[index].fnct;
    functions[i].name=strdup(gfit_fit_engine_functions[index].name);
    functions[i].value=NULL;
    functions[i].num=0;
    
    while((scanner=strchr(scanner+1, ' '))) {
      functions[i].num++;
      functions[i].value=(gdouble **) g_realloc((void *) functions[i].value, functions[i].num*sizeof(gdouble *));
      sscanf(scanner,"%s", name);
      printf("variable argument%s\n", name);
      index=gtk_engine_get_value_index_by_name(GTK_ENGINE(gfit->engine), name);
      if (index!=-1) {
        functions[i].value[functions[i].num-1]=gtk_engine_get_value_pointer(GTK_ENGINE(gfit->engine), index);
      } else {
        fclose(fp);
        gfit_warning("Argument descriptor not found.\n");
        return -1;
      }
    }
  }
  
  gtk_engine_set_functions(GTK_ENGINE(gfit->engine), num_functions, functions);
  
  return 0;
}

