#include <string.h>
#include <sys/stat.h>
#include <gfit.h>
#include <gfit_load.h>
#include <gfit_file.h>
#include <gtk/gtkdatabox.h>
#include <gmcal/gmcalbasic.h>
#include <gmcal/gmcalread.h>

/*--------------------------------------------------------------
 *
 *               EXTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

void gfit_load_data(gfit_data *gfit, gint file_type)
{
  gchar *filename;
  gchar *path;
  struct stat buf;
  gint error=0;

  filename=gtk_entry_get_text(GTK_ENTRY(GTK_FILE_SELECTION(gfit->dialog)->selection_entry));

  if (!filename || !strlen(filename)) {
    gfit_warning("No filename specified...");
    return;
  }

  path=gtk_file_selection_get_filename(GTK_FILE_SELECTION(gfit->dialog));
  if (stat(path, &buf)) {
    gfit_warning("Could not retrieve stats for file\nIt does not seem to exist...");
    return;
  }
#ifndef _WIN32
  if (!S_ISREG(buf.st_mode)) {
    gfit_warning("No regular file\nPlease choose another one...");
    return;
  }
#endif
  gfit->path=g_strdup(path);
  
  gmcal_basic_matrix_free(&GTK_DATABOX(gfit->box)->data.X, &GTK_DATABOX(gfit->box)->data.Y, &GTK_DATABOX(gfit->box)->data.Z, GTK_DATABOX(gfit->box)->data.dimX, GTK_DATABOX(gfit->box)->data.dimY);

  switch(file_type) {
    case GFIT_FILE_OPEN_GMCAL_ASCII:
    	printf("Opening as gmcal ascii matrix...\n");
    	error=gmcal_read_float_ascii_matrix_gmcal(path, &GTK_DATABOX(gfit->box)->data.X, &GTK_DATABOX(gfit->box)->data.Y, &GTK_DATABOX(gfit->box)->data.Z, &GTK_DATABOX(gfit->box)->data.dimX, &GTK_DATABOX(gfit->box)->data.dimY, &gfit->info);
    	printf("dimX: %d, dimY: %d\n", GTK_DATABOX(gfit->box)->data.dimX, GTK_DATABOX(gfit->box)->data.dimY);
    	if (error==GMCAL_SUCCESS) gtk_databox_new_data(GTK_DATABOX(gfit->box), GTK_DATABOX_POINTS_2D);
    	break;
    case GFIT_FILE_OPEN_STAR_BINARY:
    	printf("Opening as star binary...\n");
    	error=gmcal_read_float_bin_vector_simple(path, &GTK_DATABOX(gfit->box)->data.X, &GTK_DATABOX(gfit->box)->data.Y, &GTK_DATABOX(gfit->box)->data.Z, &GTK_DATABOX(gfit->box)->data.dimX, &GTK_DATABOX(gfit->box)->data.dimY, &gfit->info);
    	if (error==GMCAL_SUCCESS) gtk_databox_new_data(GTK_DATABOX(gfit->box), GTK_DATABOX_POINTS_2D);
    	break;
    case GFIT_FILE_OPEN_CSMA_BINARY:
    	printf("Opening as csma binary...\n");
    	error=gmcal_read_float_bin_matrix_csma(path, &GTK_DATABOX(gfit->box)->data.X, &GTK_DATABOX(gfit->box)->data.Y, &GTK_DATABOX(gfit->box)->data.Z, &GTK_DATABOX(gfit->box)->data.dimX, &GTK_DATABOX(gfit->box)->data.dimY, &gfit->info);
    	if (error==GMCAL_SUCCESS) gtk_databox_new_data(GTK_DATABOX(gfit->box), GTK_DATABOX_POINTS_2D);
    	break;
    default:
    	gfit_warning("File type not implemeted yet...");
    	error=1;
    	return;
  }

  if (error) {
    printf("error: %d\n", error);
    gfit_warning("An error occurred...");
    gtk_databox_new_data(GTK_DATABOX(gfit->box), GTK_DATABOX_NO_DATA);
  }


  gtk_widget_destroy(gfit->dialog);
}

/*--------------------------------------------------------------
 *
 *               INTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

