#include <gfit.h>
#include <gfit_menus.h>
#include <gfit_window.h>
#include <gfit_file.h>
#include <gfit_fit.h>
#include <gfit_chi.h>

#ifndef _WIN32
#define _GFIT_USE_ITEM_FACTORY
#else
#undef _GFIT_USE_ITEM_FACTORY
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void gfit_menu_file_new_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_file_open_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_file_sample_data_cb	(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_file_close_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_file_exit_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);

static void gfit_menu_fit_start_automatic_fit_cb(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_fit_save_fit_cb	(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_fit_choose_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void gfit_menu_fit_sample_create_cb	(gpointer callback_data, guint callback_action, GtkWidget *widget);

static void gfit_menu_help_about_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);

static GtkItemFactoryEntry
menu_main_entries[]=
{
  {"/_File", 				NULL, NULL,		 		0, 	"<Branch>"},
  {"/File/New Window", 			NULL, gfit_menu_file_new_cb, 		0, 	NULL},
  {"/File/Open File...", 		NULL, gfit_menu_file_open_cb, 		0, 	NULL},
  {"/File/Sample Data", 		NULL, NULL, 				0, 	"<Branch>"},
  {"/File/Sample Data/100 Points", 	NULL, gfit_menu_file_sample_data_cb, 	100, 	NULL},
  {"/File/Sample Data/1000 Points", 	NULL, gfit_menu_file_sample_data_cb, 	1000, 	NULL},
  {"/File/Sample Data/5000 Points", 	NULL, gfit_menu_file_sample_data_cb, 	5000, 	NULL},
  {"/File/Sample Data/10000 Points", 	NULL, gfit_menu_file_sample_data_cb, 	10000, 	NULL},
  {"/File/Close Window", 		NULL, gfit_menu_file_close_cb, 		0, 	NULL},
  {"/File/Exit", 			NULL, gfit_menu_file_exit_cb, 		0, 	NULL},
  {"/F_it", 				NULL, NULL, 				0, 	"<Branch>"},
  {"/Fit/Start Automatic Fit", 		NULL, gfit_menu_fit_start_automatic_fit_cb, 		0, 	NULL},
  {"/Fit/Save Fit", 			NULL, gfit_menu_fit_save_fit_cb, 		0, 	NULL},
  {"/Fit/Choose fit function...", 	NULL, gfit_menu_fit_choose_cb, 		0, 	NULL},
  {"/Fit/Sample fit function...", 	NULL, gfit_menu_fit_sample_create_cb, 	0, 	NULL},
  {"/_Help", 				NULL, NULL, 				0, 	"<LastBranch>"},
  {"/Help/About...", 			NULL, gfit_menu_help_about_cb, 		0, 	NULL},
};

static gint
menu_main_num = sizeof (menu_main_entries) / sizeof (menu_main_entries[0]);

#else/* _GFIT_USE_ITEM_FACTORY */

static void gfit_menu_file_new_cb		(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_open_cb		(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_sample_data_100_cb	(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_sample_data_1000_cb	(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_sample_data_5000_cb	(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_sample_data_10000_cb	(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_close_cb		(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_file_exit_cb		(GtkWidget *widget, gpointer callback_data);

static void gfit_menu_fit_start_automatic_fit_cb(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_fit_save_fit_cb		(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_fit_choose_cb		(GtkWidget *widget, gpointer callback_data);
static void gfit_menu_fit_sample_create_cb	(GtkWidget *widget, gpointer callback_data);

static void gfit_menu_help_about_cb		(GtkWidget *widget, gpointer callback_data);

#endif _GFIT_USE_ITEM_FACORY

/*--------------------------------------------------------------
 *
 *               EXTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

#ifdef _GFIT_USE_ITEM_FACTORY
GtkWidget *gfit_menu_bar_create(gchar *title, gfit_data *gfit)
{
  GtkWidget *menu_bar;
  GtkItemFactory *item_factory;

  item_factory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, title, NULL);

  gtk_item_factory_create_items(item_factory, menu_main_num, menu_main_entries, gfit);
  menu_bar=gtk_item_factory_get_widget(item_factory, title);
  return menu_bar;
}
#else /* _GFIT_USE_ITEM_FACTORY */
GtkWidget *gfit_menu_bar_create(gchar *title, gfit_data *gfit)
{
  GtkWidget *menu_bar;
  GtkWidget *menu;
  GtkWidget *submenu;
  GtkWidget *menu_item;
  
  menu_bar=gtk_menu_bar_new();
  menu=gtk_menu_new();
  menu_item=gtk_menu_item_new_with_label("New");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_new_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("Open File...");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_open_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  submenu=gtk_menu_new();
  menu_item=gtk_menu_item_new_with_label("100 Points");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_sample_data_100_cb, gfit);
  gtk_menu_append(GTK_MENU(submenu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("1000 Points");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_sample_data_1000_cb, gfit);
  gtk_menu_append(GTK_MENU(submenu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("5000 Points");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_sample_data_5000_cb, gfit);
  gtk_menu_append(GTK_MENU(submenu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("10000 Points");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_sample_data_10000_cb, gfit);
  gtk_menu_append(GTK_MENU(submenu), menu_item);
  gtk_widget_show(menu_item);

  menu_item=gtk_menu_item_new_with_label("Sample Data");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), submenu);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  
  menu_item=gtk_menu_item_new_with_label("Close");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_close_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("Exit");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_file_exit_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);

  menu_item=gtk_menu_item_new_with_label("File");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
  gtk_widget_show(menu_item);
  
  menu=gtk_menu_new();
  menu_item=gtk_menu_item_new_with_label("Start Automatic Fit...");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_fit_start_automatic_fit_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("Choose Fit Function...");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_fit_choose_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("Sample Fit Function");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_fit_sample_create_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);

  menu_item=gtk_menu_item_new_with_label("Fit");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
  gtk_widget_show(menu_item);
  
  menu=gtk_menu_new();
  menu_item=gtk_menu_item_new_with_label("About...");
  gtk_signal_connect(GTK_OBJECT(menu_item), "activate", gfit_menu_help_about_cb, gfit);
  gtk_menu_append(GTK_MENU(menu), menu_item);
  gtk_widget_show(menu_item);
  menu_item=gtk_menu_item_new_with_label("Help");
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
  gtk_widget_show(menu_item);
  
  return menu_bar;
}
#endif /* _GFIT_USE_ITEM_FACTORY */

/*--------------------------------------------------------------
 *
 *               INTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_file_new_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_new();

  return;
}
#else 
static void
gfit_menu_file_new_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_new();

  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_file_open_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_file_open((gfit_data *) callback_data);
  return;
}
#else
static void
gfit_menu_file_open_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_file_open((gfit_data *) callback_data);
  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_file_sample_data_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_sample_create((gfit_data *) callback_data, callback_action);
  return;
}
#else
static void
gfit_menu_file_sample_data_100_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_sample_create((gfit_data *) callback_data, 100);
  return;
}
static void
gfit_menu_file_sample_data_1000_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_sample_create((gfit_data *) callback_data, 1000);
  return;
}
static void
gfit_menu_file_sample_data_5000_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_sample_create((gfit_data *) callback_data, 5000);
  return;
}
static void
gfit_menu_file_sample_data_10000_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_sample_create((gfit_data *) callback_data, 10000);
  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_file_close_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_close((gfit_data *) callback_data);
  return;
}
#else
static void
gfit_menu_file_close_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_close((gfit_data *) callback_data);
  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_file_exit_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_exit();
  return;
}
#else
static void
gfit_menu_file_exit_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_exit();
  return;
}
#endif

/*------------------------------------------------------------------------
 *
 *                         MENU FIT CALLBACKS
 *
 *------------------------------------------------------------------------*/

#ifdef _GFIT_USE_ITEM_FACTORY
static void 
gfit_menu_fit_start_automatic_fit_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  if (gfit_chi((gfit_data *) callback_data)) {
    gfit_fit_external((gfit_data *) callback_data);
  }

  return;
}
#else
static void
gfit_menu_fit_start_automatic_fit_cb(GtkWidget *widget, gpointer callback_data)
{
  if (gfit_chi((gfit_data *) callback_data)) {
    gfit_fit_external((gfit_data *) callback_data);
  }

  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void 
gfit_menu_fit_save_fit_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_fit_engine_save((gfit_data *) callback_data);

  return;
}
#else
static void
gfit_menu_fit_save_fit_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_fit_engine_save((gfit_data *) callback_data);

  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void 
gfit_menu_fit_choose_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_fit_engine_open((gfit_data *) callback_data);

  return;
}
#else
static void
gfit_menu_fit_choose_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_fit_engine_open((gfit_data *) callback_data);

  return;
}
#endif

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_fit_sample_create_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_fit_sample_create((gfit_data *) callback_data);
  
  return;
}
#else
static void
gfit_menu_fit_sample_create_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_fit_sample_create((gfit_data *) callback_data);
  
  return;
}
#endif

/*------------------------------------------------------------------------
 *
 *                         MENU HELP CALLBACKS
 *
 *------------------------------------------------------------------------*/

#ifdef _GFIT_USE_ITEM_FACTORY
static void
gfit_menu_help_about_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gfit_warning("Not implemented yet\n");
  return;
}
#else
static void
gfit_menu_help_about_cb(GtkWidget *widget, gpointer callback_data)
{
  gfit_warning("Not implemented yet\n");
  return;
}
#endif