#include <string.h>
#include <stdio.h>
#include <gfit.h>
#include <gtk/gtkdatabox.h>
#include <gfit_show.h>

enum {
  SHOW_BOX,
  SHOW_ACTUAL_X,
  SHOW_ACTUAL_Y,
  SHOW_MARKED_X,
  SHOW_MARKED_Y,
  SHOW_DELTA_X,
  SHOW_DELTA_Y,
  SHOW_NUM_ENTRIES
};

static GtkWidget *	gfit_show_entry(GtkWidget *hbox, gchar *text);
static void 		gfit_show_motion_notify_cb(GtkWidget *widget, GdkEventMotion *event, GtkWidget **entries);
static void 		gfit_show_button_press_cb(GtkWidget *widget, GdkEventButton *event, GtkWidget **entries);
static void 		gfit_show_zoom_cb(GtkWidget *widget, GtkDatabox *box);

/*--------------------------------------------------------------
 *
 *               EXTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

GtkWidget *
gfit_show_create(GtkWidget *container)
{
  GtkWidget *box;
  GtkWidget *hbox;
  GtkWidget **entries;

  entries=(GtkWidget **) g_malloc0(SHOW_NUM_ENTRIES*sizeof(GtkWidget *));

  hbox=gtk_hbox_new(TRUE,3);
  gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, TRUE, 0);


  entries[SHOW_ACTUAL_X]=gfit_show_entry(hbox, "Actual X");
  entries[SHOW_ACTUAL_Y]=gfit_show_entry(hbox, "Actual Y");
  entries[SHOW_MARKED_X]=gfit_show_entry(hbox, "Marked X");
  entries[SHOW_MARKED_Y]=gfit_show_entry(hbox, "Marked Y");
  entries[SHOW_DELTA_X] =gfit_show_entry(hbox, "Delta X");
  entries[SHOW_DELTA_Y] =gfit_show_entry(hbox, "Delta Y");

  entries[SHOW_BOX]=box=gtk_databox_new ();
  gtk_container_add (GTK_CONTAINER (container), box);

  gtk_signal_connect_after(GTK_OBJECT(GTK_DATABOX(box)->draw), "motion_notify_event",
  				GTK_SIGNAL_FUNC(gfit_show_motion_notify_cb),
  				entries);
  gtk_signal_connect_after(GTK_OBJECT(GTK_DATABOX(box)->draw), "button_press_event",
  				GTK_SIGNAL_FUNC(gfit_show_button_press_cb),
  				entries);
  gtk_signal_connect_after(GTK_OBJECT(box), "gtk_databox_zoom",
  				GTK_SIGNAL_FUNC(gfit_show_zoom_cb),
  				box);

 return box;
}

/*--------------------------------------------------------------
 *
 *               INTERNAL FUNCTIONS
 *
 *--------------------------------------------------------------*/

static void
gfit_show_zoom_cb(GtkWidget *widget, GtkDatabox *box)
{
  printf("Zoom callback\n");
}

static void
gfit_show_button_press_cb(GtkWidget *widget, GdkEventButton *event, GtkWidget **entries)
{
  gfloat coord_x, coord_y;
  gchar dummy[100];
  gchar *text;
  GtkDatabox *box=GTK_DATABOX(entries[SHOW_BOX]);

  if (event->type!=GDK_BUTTON_PRESS)
    return;

  gtk_databox_get_marked_coordinates(box, &coord_x, &coord_y);

  sprintf(dummy,"%g", coord_x);
  text=strdup(dummy);
  gtk_entry_set_text(GTK_ENTRY(entries[SHOW_MARKED_X]), text);
  sprintf(dummy,"%g", coord_y);
  text=strdup(dummy);
  gtk_entry_set_text(GTK_ENTRY(entries[SHOW_MARKED_Y]), text);
}

static void
gfit_show_motion_notify_cb(GtkWidget *widget, GdkEventMotion *event, GtkWidget **entries)
{
  gint x, y;
  gfloat coord_x, coord_y;
  GdkModifierType state;
  gchar dummy[100];
  gchar *text;
  GtkDatabox *box=GTK_DATABOX(entries[SHOW_BOX]);

  x = event->x;
  y = event->y;
  state = event->state;

  gtk_databox_get_coordinates(box, x, y, &coord_x, &coord_y);

  sprintf(dummy,"%g", coord_x);
  text=strdup(dummy);
  gtk_entry_set_text(GTK_ENTRY(entries[SHOW_ACTUAL_X]), text);
  sprintf(dummy,"%g", coord_y);
  text=strdup(dummy);
  gtk_entry_set_text(GTK_ENTRY(entries[SHOW_ACTUAL_Y]), text);

  if (state & GDK_BUTTON1_MASK) {
    gtk_databox_get_delta_coordinates(box, &coord_x, &coord_y);

    sprintf(dummy,"%g", coord_x);
    text=strdup(dummy);
    gtk_entry_set_text(GTK_ENTRY(entries[SHOW_DELTA_X]), text);
    sprintf(dummy,"%g", coord_y);
    text=strdup(dummy);
    gtk_entry_set_text(GTK_ENTRY(entries[SHOW_DELTA_Y]), text);
  }
}

static GtkWidget *
gfit_show_entry(GtkWidget *hbox, gchar *text)
{
  GtkWidget *frame;
  GtkWidget *entry;

  frame=gtk_frame_new(text);
  gtk_container_add(GTK_CONTAINER(hbox), frame);
  entry=gtk_entry_new();
  gtk_widget_set_usize(entry, 20, -1);
  gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
  gtk_container_add(GTK_CONTAINER(frame), entry);

  return entry;
}

